#include "AnimationBase.h"

AnimationBase::AnimationBase()
{}
AnimationBase::AnimationBase(HGE *aHge)
: Sprite(aHge)
{
	mySheetList.Init(5, 5);
	myNumberOfSpriteList.Init(5, 5);
	myFrameElapseTimeList.Init(5, 5);
	myCurrentSpriteWidthList.Init(5, 5);

	myNumberOfSheet = 0;
	myFrameElapseTime = 0.0f;

	myActiveSheet = 0;
	myCurrentSprite = 0;
	myCurrentCornerX = 0.0f;
	myTimer = 0.0f;
	myAnimationTimer = 0.0f;
	myPause = false;
}
AnimationBase::AnimationBase(const AnimationBase &aAnimationBase)
: Sprite(aAnimationBase)
{
	// instancer from this class
	myTexture = aAnimationBase.myTexture;
	mySprite = aAnimationBase.mySprite;
	
	mySheetList = aAnimationBase.mySheetList;
	myNumberOfSpriteList = aAnimationBase.myNumberOfSpriteList;
	myFrameElapseTimeList = aAnimationBase.myFrameElapseTimeList;
	myCurrentSpriteWidthList = aAnimationBase.myCurrentSpriteWidthList;

	myNumberOfSheet = aAnimationBase.myNumberOfSheet;
	myActiveSheet = aAnimationBase.myActiveSheet;
	myCurrentSprite = aAnimationBase.myCurrentSprite;
	myCurrentCornerX = aAnimationBase.myCurrentCornerX;
	myTimer = aAnimationBase.myTimer;
	myAnimationTimer = aAnimationBase.myAnimationTimer;
	myPause = aAnimationBase.myPause;
}
AnimationBase::~AnimationBase()
{

}

void AnimationBase::Initialize()
{
	mySprite = new hgeSprite(mySheetList[myActiveSheet], 0.0f, 0.0f, myCurrentSpriteWidthList[myActiveSheet], mySpriteHeight);	
	mySprite->SetZ(1.0, 3);
}

void AnimationBase::Update(float aDeltaTime)
{
	aDeltaTime = aDeltaTime;
}

void AnimationBase::Render(float aDeltaTime)
{
	myTimer += aDeltaTime;
	
	if(myPause == false)
	{
		myAnimationTimer += myTimer;
	}
	else
	{
		myAnimationTimer = 0.0f;
	}
	while ( myAnimationTimer >= myFrameElapseTimeList[myActiveSheet] )
	{
		myCurrentCornerX = myCurrentSprite * myCurrentSpriteWidthList[myActiveSheet];
		mySprite->SetTextureRect(myCurrentCornerX, 0.0f, mySprite->GetWidth(), mySprite->GetHeight(), false);
		myCurrentSprite++;
		myAnimationTimer = 0.0f;
		if(myCurrentSprite >= myNumberOfSpriteList[myActiveSheet])
		{
			myCurrentCornerX = 0.0f;
			myCurrentSprite = 0;
		}
	}
	if(myIsVisible == true)
	{
		mySprite->RenderEx(myPosition.x, myPosition.y, myRotation, myScale.x, myScale.y);
	}
}

void AnimationBase::RenderAtPosition(float aDeltaTime, Vector2<float> aPosition)
{
	if (myPause == false)
	{
		myTimer += aDeltaTime;
	}	

	while ( myTimer >= myFrameElapseTimeList[myActiveSheet] )
	{
		myCurrentCornerX = myCurrentSprite * myCurrentSpriteWidthList[myActiveSheet];
		mySprite->SetTextureRect(myCurrentCornerX, 0.0f, mySprite->GetWidth(), mySprite->GetHeight(), false);
		myTimer -= myFrameElapseTimeList[myActiveSheet];
		myCurrentSprite++;
		if(myCurrentSprite >= (myNumberOfSpriteList[myActiveSheet] - 1))
		{
			myCurrentSprite = 0;
		}
	}
	if(myIsVisible == true)
	{
		mySprite->RenderEx(aPosition.x, aPosition.y, 0.0f, 1.0f, 1.0f);
	}
}

void AnimationBase::AddSpriteSheet(std::string aSheetName, int aNumberOfSprite, float aFrameElapseTime)
{
	myTexture = myHGE->Texture_Load(aSheetName.c_str());
	assert(myTexture && "Can't load the sheet!");

	mySpriteWidth = static_cast<float>(myHGE->Texture_GetWidth(myTexture, false)) / aNumberOfSprite;
	mySpriteHeight = static_cast<float>(myHGE->Texture_GetHeight(myTexture, false));

	mySheetList.Add(myTexture);
	myNumberOfSpriteList.Add(aNumberOfSprite);
	myFrameElapseTimeList.Add(aFrameElapseTime);
	myCurrentSpriteWidthList.Add(mySpriteWidth);

	myNumberOfSheet++;
}

void AnimationBase::SetActiveSheet(int aActiveSheetIndex)
{
	myActiveSheet = aActiveSheetIndex;
	mySprite->SetTexture(mySheetList[myActiveSheet]);
}
int AnimationBase::GetActiveSheet()
{
	return myActiveSheet;
}

void AnimationBase::Play()
{
	myPause = false;
}
void AnimationBase::Pause()
{
	myPause = true;
}

int AnimationBase::GetLastAnimationIndex()
{
	return (myNumberOfSheet - 1);
}