#ifndef _GROWINGARRAY_H_
#define _GROWINGARRAY_H_

#include <string.h>
#include <assert.h>

namespace Tools
{

	#define MacroSafeDelete(macroPointer) { if(macroPointer != 0) {delete (macroPointer); (macroPointer)=0;} }
	#define MacroSafeDeleteArray(macroPointer) { if(macroPointer != 0) {delete[] (macroPointer); (macroPointer)=0;} }

	template <class Type,class CountType =unsigned short>
	class GrowingArray
	{
	public:
		GrowingArray();
		GrowingArray(CountType aNrOfRecomendedItems,CountType aItemIncreaseSize, bool aUseSafeModeFlag=true);
		GrowingArray(const GrowingArray& aGrowingArray);
		~GrowingArray();

		GrowingArray& operator=(const GrowingArray& aGrowingArray);		
		
		void Init(CountType aNrOfRecomendedItems,CountType aItemIncreaseSize,bool aUseSafeModeFlag=true);
		void ReInit(CountType aNrOfRecomendedItems,CountType aItemIncreaseSize,bool aUseSafeModeFlag=true);

		inline Type& operator[](const CountType &aIndex) const;

		inline void Add(const Type& aObjetc);
		inline void Insert(CountType aIndex,Type& aObject);
		inline void DeleteCyclic(Type& aObject);
		inline void DeleteCyclicAtIndex(CountType aItemNumber);
		inline void RemoveCyclic(const Type& aObject);
		inline void RemoveCyclicAtIndex(CountType aItemNumber);


		inline void RemoveAll();
		inline void DeleteAll();

		void Optimize();
		__forceinline CountType Count() const;

	private:
		inline void ReSize(CountType aNewSize);

		CountType myCurrentSize;
		CountType myMaxSize;
		CountType myIncreaseSize;
		Type *myArray;
		bool myInitialized;
		bool mySafeMode;

	};
};

template <class Type,class CountType>
Tools::GrowingArray< Type, CountType >::GrowingArray()
{
	myCurrentSize = 0;
	myMaxSize = 0;
	myIncreaseSize =0;
	myArray = 0;
	myInitialized = 0;
	mySafeMode = 0;
}

template <class Type,class CountType>
Tools::GrowingArray< Type, CountType >::GrowingArray(CountType aNrOfRecomendedItems,CountType aItemIncreaseSize, bool aUseSafeModeFlag=true)
{
	myMaxSize = aNrOfRecomendedItems;
	myCurrentSize = 0;
	myIncreaseSize = aItemIncreaseSize;
	mySafeMode = aUseSafeModeFlag;
	myArray = new Type[myMaxSize];
	myInitialized = true;
}

template <class Type,class CountType>
Tools::GrowingArray< Type, CountType >::GrowingArray(const GrowingArray& aGrowingArray)
{
	myCurrentSize = aGrowingArray.myCurrentSize;
	myMaxSize = aGrowingArray.myMaxSize;
	myIncreaseSize = aGrowingArray.myIncreaseSize;
	myInitialized = aGrowingArray.myInitialized;
	mySafeMode = aGrowingArray.mySafeMode;

	myArray = new Type[aGrowingArray.myMaxSize];
	if ( mySafeMode == true )
	{
		int bytesToCopy = sizeof(Type) * myCurrentSize;
		memcpy(myArray, aGrowingArray.myArray, bytesToCopy);
	}
	else
	{
		for ( signed int positionCopied = 0; positionCopied < myCurrentSize; positionCopied++)
		{
			myArray[positionCopied] = aGrowingArray.myArray[positionCopied];
		}
	}
}

template <class Type,class CountType>
Tools::GrowingArray< Type, CountType >::~GrowingArray()
{
	myCurrentSize = 0;
	myMaxSize = 0;
	myIncreaseSize = 0;
	myInitialized = 0;
	mySafeMode = 0;

	MacroSafeDeleteArray(myArray);
}

template <class Type,class CountType>
Tools::GrowingArray<Type, CountType>& Tools::GrowingArray< Type, CountType >::operator =(const Tools::GrowingArray<Type,CountType> &aGrowingArray)
{
	myCurrentSize = aGrowingArray.myCurrentSize;
	myMaxSize = aGrowingArray.myMaxSize;
	myIncreaseSize = aGrowingArray.myIncreaseSize;
	myInitialized = aGrowingArray.myInitialized;
	mySafeMode = aGrowingArray.mySafeMode;
	
	MacroSafeDeleteArray(myArray);

	myArray = new Type[aGrowingArray.myMaxSize];
	if ( mySafeMode == true )
	{
		int bytesToCopy = sizeof(Type) * myCurrentSize;
		memcpy(myArray, aGrowingArray.myArray, bytesToCopy);
	}
	else
	{
		for ( int positionCopied = 0; positionCopied < myCurrentSize; positionCopied++)
		{
			myArray[positionCopied] = aGrowingArray.myArray[positionCopied];
		}
	}
	return *this;

}

template <class Type,class CountType>
void Tools::GrowingArray< Type, CountType >::Init(CountType aNrOfRecomendedItems, CountType aItemIncreaseSize, bool aUseSafeModeFlag = true)
{
	if ( myInitialized == false )
	{
		myCurrentSize = 0;
		myMaxSize = aNrOfRecomendedItems;
		myIncreaseSize = aItemIncreaseSize;
		mySafeMode = aUseSafeModeFlag;
		MacroSafeDeleteArray(myArray);
		myArray = new Type[myMaxSize];
		myInitialized = true;
	}
	else
	{
		assert(false &&"GrowingArray: Called Init of already initialized array");
	}
}

template <class Type,class CountType>
void Tools::GrowingArray< Type, CountType >::ReInit(CountType aNrOfRecomendedItems, CountType aItemIncreaseSize, bool aUseSafeModeFlag = true)
{
	if ( myInitialized == true )
	{
		myMaxSize = aNrOfRecomendedItems;
		myCurrentSize = 0;
		myIncreaseSize = aItemIncreaseSize;
		mySafeMode = aUseSafeModeFlag;
		MacroSafeDeleteArray(myArray);
		myArray = new Type[myMaxSize];
		myInitialized = true;
	}
	else
	{
		assert(false &&"GrowingArray: Tried to ReInit an uninitialized class.");
	}
}

template <class Type,class CountType>
inline Type& Tools::GrowingArray< Type, CountType >::operator [](const CountType &aIndex) const
{
	assert(myInitialized == true && "GrowingArray: Tried to use the [] operator on an uninitialized class.");
	assert( (aIndex >= 0 && aIndex <= (myCurrentSize-1)) && "GrowingArray:  [] - Out of bounds.");
	
	return myArray[aIndex];

}

template <class Type,class CountType>
inline void Tools::GrowingArray< Type, CountType >::Add(const Type &aObject)
{
	if ( myInitialized == true )
	{
		if (myCurrentSize == myMaxSize)
		{
			ReSize(myMaxSize + myIncreaseSize);
		}
		myArray[myCurrentSize] = aObject;
		myCurrentSize++;
	}
	else
	{	
		assert(false && "GrowingArray: Tried to use the Add function on an uninitialized class.");
	}
}

template <class Type,class CountType>
inline void Tools::GrowingArray< Type, CountType >::ReSize(CountType aNewSize)
{
	Type * const temp = new Type[aNewSize];
	if ( mySafeMode == true )
	{
		int bytesToCopy = sizeof(Type) * myCurrentSize;
		memcpy(temp, myArray, bytesToCopy);
	}
	else
	{
		for (int copiedPosition = 0; copiedPosition < myCurrentSize; copiedPosition++)
		{
			temp[copiedPosition] = myArray[copiedPosition];
		}
	}
	MacroSafeDeleteArray(myArray);
	myArray = temp;
	myMaxSize += myIncreaseSize;
}

template <class Type,class CountType>
inline void Tools::GrowingArray< Type, CountType >::Insert(CountType aIndex, Type &aObject)
{
	//	function should move all the positions futher into the array
	assert ( (myInitialized == true) && "GrowingArray: Tried to use the Insert function on an uninitialized class.");
	assert ( (aIndex >= 0 && aIndex <= (myCurrentSize-1)) && "GrowingArray: Insert - Out of Bounds.");
	assert ( (myCurrentSize < myMaxSize) && "GrowingArray: Insert - Size of array bigger than max size!");

	if ( myCurrentSize == myMaxSize )
	{
		ReSize(myMaxSize + myIncreaseSize);
	}

	for ( int objectMoved = (myCurrentSize-1); objectMoved >= aIndex; objectMoved--)
	{
		myArray[objectMoved+1] = myArray[objectMoved];
	}
	myArray[aIndex] = aObject;
	myCurrentSize++;

}

template <class Type,class CountType>
inline void Tools::GrowingArray< Type, CountType >::DeleteCyclic(Type &aObject)
{
	if ( myInitialized == true )
	{
		bool objectFound = false;
		for ( int objectChecked = 0; objectChecked < myCurrentSize; objectChecked++)
		{
			if ( aObject == myArray[objectChecked] )
			{
				objectFound = true;
				delete(myArray[objectChecked]);
				myArray[objectChecked] = myArray[myCurrentSize-1];
				myCurrentSize--;
				break;
			}
		}
		if ( objectFound == false )
		{
			assert(false &&"GrowingArray: DeleteCyclic - object not found in GrowingArray.");
		}
	}
	else
	{	
		assert(false &&"GrowingArray: Tried to use the DeleteCyclic function on an uninitialized class.");
	}
}

template <class Type,class CountType>
inline void Tools::GrowingArray< Type, CountType >::DeleteCyclicAtIndex(CountType aItemNumber)
{
	if ( myInitialized == true )
	{
		if ( aItemNumber >= 0 && aItemNumber <= (myCurrentSize-1) )
		{
			delete(myArray[aItemNumber]);
			myArray[aItemNumber] = myArray[myCurrentSize-1];
			myCurrentSize--;
		}
		else
		{
			assert(false &&"GrowingArray: DeleteCyclicAtIndex - Out of bounds.");
		}
	}
	else
	{	
		assert(false &&"GrowingArray: Tried to use the DeleteCyclic function on an uninitialized class.");
	}
}

template <class Type,class CountType>
inline void Tools::GrowingArray< Type, CountType >::RemoveCyclic(const Type &aObject)
{
	if ( myInitialized == true )
	{
		bool objectFound = false;
		for ( int objectChecked = 0; objectChecked < myCurrentSize; objectChecked++)
		{
			if ( aObject == myArray[objectChecked] )
			{
				objectFound = true;
				myArray[objectChecked] = myArray[myCurrentSize-1];
				myArray[myCurrentSize-1] = 0;
				myCurrentSize--;
				break;
			}
		}
		if ( objectFound == false )
		{
			assert(false &&"GrowingArray: RemoveCyclic - object not found in GrowingArray.");
		}
	}
	else
	{	
		assert(false &&"GrowingArray: Tried to use the RemoveCyclic function on an uninitialized class.");
	}
}

template <class Type,class CountType>
inline void Tools::GrowingArray< Type, CountType >::RemoveCyclicAtIndex(CountType aItemNumber)
{
	if ( myInitialized == true )
	{
		if ( aItemNumber >= 0 && aItemNumber <= (myCurrentSize-1) )
		{
			myArray[aItemNumber] = myArray[myCurrentSize-1];
			myArray[myCurrentSize-1] = 0;
			myCurrentSize--;
		}
		else
		{
			assert(false &&"GrowingArray: RemoveCyclicAtIndex - Out of bounds.");
		}
	}
	else
	{	
		assert(false &&"GrowingArray: Tried to use the RemoveCyclicAtIndex function on an uninitialized class.");
	}
}

template <class Type,class CountType>
inline void Tools::GrowingArray< Type, CountType >::RemoveAll()
{
	if ( myInitialized == true )
	{
		myCurrentSize = 0;
	}
	else
	{	
		assert(false &&"GrowingArray: Tried to use the RemoveAll() function on an uninitialized class.");
	}
}

template <class Type,class CountType>
inline void Tools::GrowingArray< Type, CountType >::DeleteAll()
{
	if ( myInitialized == true )
	{
		for ( int objectDeleted = 0; objectDeleted < myCurrentSize; objectDeleted++)
		{
			delete myArray[objectDeleted];
			//delete;
		}
		myCurrentSize = 0;
	}
	else
	{	
		assert(false &&"GrowingArray: Tried to use the DeleteAll() function on an uninitialized class.");
	}
}

template <class Type,class CountType>
void Tools::GrowingArray< Type, CountType >::Optimize()
{
	if ( myInitialized == true)
	{
		ReSize(myCurrentSize);
		myIncreaseSize = 1;
	}
	else
	{
		assert(false &&"GrowingArray: Tried to use the Optimize() function on an uninitialized class.");
	}
}

template <class Type,class CountType>
__forceinline CountType Tools::GrowingArray< Type, CountType >::Count() const
{
	return myCurrentSize;
}

#endif // _GROWINGARRAY_H_