#ifndef _STATICARRAY_H_
#define _STATICARRAY_H_

#include <assert.h>

namespace Tools
{

	#define MacroSafeDelete(macroPointer) { if(macroPointer != 0) {delete (macroPointer); (macroPointer)=0;} }

	template <class Type,int Size>
	class StaticArray
	{
	public:
		StaticArray();
		StaticArray(const StaticArray& aStaticArray);
		~StaticArray();

		StaticArray& operator=(const StaticArray& aGrowingArray);

		inline Type& operator[](const int& aIndex) const;

	// Utility functions 

		inline void Insert(int aIndex,Type& aObject);
		inline void DeleteAll();

	private:
		Type myArray[Size];

	};
}

template <class Type,int Size>
Tools::StaticArray< Type, Size >::StaticArray()
{
	for ( unsigned int positionCleared = 0; positionCleared < Size; positionCleared++)
	{
		myArray[positionCleared] = 0; 
	};
}

template <class Type,int Size>
Tools::StaticArray< Type, Size >::~StaticArray()
{
	for ( unsigned int positionCleared = 0; positionCleared < Size; positionCleared++)
	{
		myArray[positionCleared] = 0; 
	};
}

template <class Type,int Size>
Tools::StaticArray< Type, Size >::StaticArray(const Tools::StaticArray<Type,Size> &aStaticArray)
{
	for ( unsigned int positionCopied = 0; positionCopied < Size; positionCopied++)
	{
		myArray[positionCopied] = aStaticArray.myArray[positionCopied];	
	};
}

template <class Type,int Size>
Tools::StaticArray<Type, Size>& Tools::StaticArray< Type, Size >::operator =(const Tools::StaticArray<Type,Size> &aStaticArray)
{
	for ( unsigned int positionCopied = 0; positionCopied < Size; positionCopied++)
	{
		myArray[positionCopied] = aStaticArray.myArray[positionCopied];	
	};
	return *this;
}

template <class Type,int Size>
inline Type& Tools::StaticArray< Type, Size >::operator [](const int &aIndex) const
{
	assert( (aIndex >= 0 && aIndex <= Size) && "StaticArray: operator [] - Out of bounds.");
	return static_cast<Type>(myArray[aIndex]);
}

template <class Type,int Size>
inline void Tools::StaticArray< Type, Size >::Insert(int aIndex, Type &aObject)
{
	assert( (aIndex >= 0 && aIndex <= Size) && "StaticArray: Insert - Out of bounds.");
		
	for ( int objectMoved = (Size-1); objectMoved >= aIndex; objectMoved--)
	{
		myArray[objectMoved+1] = myArray[objectMoved];
	}
	myArray[aIndex] = aObject;
}

template <class Type,int Size>
inline void Tools::StaticArray< Type, Size >::DeleteAll()
{
	for (unsigned int objectDeleted = 0; objectDeleted < Size; objectDeleted++)
	{
		MacroSafeDelete(myArray[objectDeleted]);
	}
}

#endif