#include "Avatar.h"
#include "CollisionMap.h"
#include "EventSingelton.h"

Avatar::Avatar()
{
}

Avatar::Avatar(HGE* aHge, Vector2<float> aPositionInWorld)
:myHge(aHge)
{
	myAnimation = AnimationBase(aHge);
	myWorldSpacePosition = aPositionInWorld;
	myCameraSpacePosition;
	myRotation = 0.0f;
	myScale = Vector2<float>(1.0f, 1.0f);

	myDirection = RIGHT_WALK;
	myLastDirection = myDirection;

	interractionButton = StaticSprite( aHge, "img/Untitled.png" );	
}

Avatar::Avatar(const Avatar &aAvatar):
myHge(aAvatar.myHge)
{
	myAnimation = aAvatar.myAnimation;
	myCameraSpacePosition = aAvatar.myCameraSpacePosition;
	myWorldSpacePosition = aAvatar.myWorldSpacePosition;
	myRotation = aAvatar.myRotation;
	myScale = aAvatar.myScale;

	myDirection = aAvatar.myDirection;
	myLastDirection = aAvatar.myLastDirection;
	myTranslatingSpeed = aAvatar.myTranslatingSpeed;

	myIsInside = aAvatar.myIsInside;

	interractionButton = aAvatar.interractionButton;
}

Avatar::~Avatar()
{
}

void Avatar::LoadAnimationSheet()
{
	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_stand_up.dds", 1, 0.01f);
	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_walk_up.dds", 16, 50.0f);

	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_stand_up_right.dds", 1, 0.01f);
	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_walk_right_up.dds", 16, 50.0f);

	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_stand_right.dds", 1, 0.01f);
	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_walk_right.dds", 16, 50.0f);

	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_stand_down_right.dds", 1, 0.01f);
	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_walk_right_down.dds", 16, 50.0f);

	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_stand_down.dds", 1, 0.01f);
	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_walk_down.dds", 16, 50.0f);

	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_stand_down_left.dds", 1, 0.01f);
	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_walk_left_down.dds", 16, 50.0f);

	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_stand_left.dds", 1, 0.01f);
	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_walk_left.dds", 16, 50.0f);

	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_stand_up_left.dds", 1, 0.01f);
	myAnimation.AddSpriteSheet("img/animation/HeroineAni/h_walk_left_up.dds", 16, 50.0f);


}

void Avatar::Initialize(Vector2<float> aCameraPosition)
{
	LoadAnimationSheet();
	myAnimation.Initialize();
	myAnimation.SetActiveSheet(myDirection);
	//myPosition.x -= (myAnimation.GetSpriteWidth() / 2);
	//myPosition.y -= (myAnimation.GetSpriteHeight() / 2);
	myCameraSpacePosition = myWorldSpacePosition - aCameraPosition;
	myTranslatingSpeed = 0.2f;
	myAnimation.SetPosition(myCameraSpacePosition);
}

void Avatar::Input(InputWrapper &aInput, Vector2<float>& aCamPosition, CollisionMap aCollisionMap)
{
	if(aInput.KeyDown(DIK_W) )
	{
		Vector2<float> tempWorldPosition = myWorldSpacePosition;
		myDirection = UP_WALK;
		tempWorldPosition.y -= myTranslatingSpeed;

		int tempOKPoints = CreatePoints(tempWorldPosition, aCollisionMap);
		
		if(tempOKPoints == 4)
		{
			myWorldSpacePosition.y -= myTranslatingSpeed;
			myCameraSpacePosition = myWorldSpacePosition - aCamPosition;
			Vector2<float> collisionPoinInCamera = tempWorldPosition - aCamPosition;
			if ( collisionPoinInCamera.y <= (768 / 5) * 2 )
			{
				aCamPosition.y -= myTranslatingSpeed;
			}
		}
	}
	else if(aInput.KeyDown(DIK_S) )
	{
		Vector2<float> tempWorldPosition = myWorldSpacePosition;
		myDirection = DOWN_WALK;
		tempWorldPosition.y += myTranslatingSpeed;

		int tempOKPoints = CreatePoints(tempWorldPosition, aCollisionMap);
		
		if(tempOKPoints == 4)
		{
			myWorldSpacePosition.y += myTranslatingSpeed;
			myCameraSpacePosition = myWorldSpacePosition - aCamPosition;
			Vector2<float> collisionPoinInCamera = tempWorldPosition - aCamPosition;
			if ( collisionPoinInCamera.y >= ((768 / 5) *3))
			{
				aCamPosition.y += myTranslatingSpeed;
			}
		}
	}

	if(aInput.KeyDown(DIK_A) )
	{
		Vector2<float> tempWorldPosition = myWorldSpacePosition;
		myDirection = LEFT_WALK;
		tempWorldPosition.x -= myTranslatingSpeed;

		int tempOKPoints = CreatePoints(tempWorldPosition, aCollisionMap);
		
		if(tempOKPoints == 4)
		{
			myWorldSpacePosition.x -= myTranslatingSpeed;
			myCameraSpacePosition = myWorldSpacePosition - aCamPosition;
			Vector2<float> collisionPoinInCamera = tempWorldPosition - aCamPosition;
			if ( collisionPoinInCamera.x <= (1024 / 5) * 2 )
			{
				aCamPosition.x -= myTranslatingSpeed;
			}
		}
	}
	else if(aInput.KeyDown(DIK_D) )
	{
		Vector2<float> tempWorldPosition = myWorldSpacePosition;
		myDirection = RIGHT_WALK;
		tempWorldPosition.x += myTranslatingSpeed;

		int tempOKPoints = CreatePoints(tempWorldPosition, aCollisionMap);
		
		if(tempOKPoints == 4)
		{
			myWorldSpacePosition.x += myTranslatingSpeed;
			myCameraSpacePosition = myWorldSpacePosition - aCamPosition;
			Vector2<float> collisionPoinInCamera = tempWorldPosition - aCamPosition;
			if ( collisionPoinInCamera.x >= ((1024 / 5) *3) )
			{
				aCamPosition.x += myTranslatingSpeed;
			}
		}
	}

	if(aInput.KeyDown(DIK_W) && (aInput.KeyDown(DIK_D) ) )
	{
		myDirection = UP_RIGHT_WALK;
	}

	if(aInput.KeyDown(DIK_D) && (aInput.KeyDown(DIK_S) ) )
	{
		myDirection = RIGHT_DOWN_WALK;
	}

	if(aInput.KeyDown(DIK_A) && (aInput.KeyDown(DIK_S) ) )
	{
		myDirection = DOWN_LEFT_WALK;
	}

	if(aInput.KeyDown(DIK_A) && (aInput.KeyDown(DIK_W) ) )
	{
		myDirection = LEFT_UP_WALK;
	}

	if(aInput.AllKeysReleased())
	{
		if((myDirection != 0) && ((myDirection % 2) > 0))
		{
			myDirection -= 1;		
		}
	}

	myAnimation.SetActiveSheet(myDirection);

}

void Avatar::Update(float aDeltaTime, Vector2<float> aCameraPosition)
{
	UpdateCameraSpacePosition(aCameraPosition);
	myAnimation.SetPosition(myCameraSpacePosition);
	myAnimation.Update(aDeltaTime);
	
	if (EventSingelton::GetInstance()->myInventoryActive == true 
		|| EventSingelton::GetInstance()->myNotebookActive == true
		|| EventSingelton::GetInstance()->myConversationActive == true
		|| EventSingelton::GetInstance()->myQuitMenuActive == true)
	{
		StopAnimation();
	}
	else
	{
		StartAnimation();
	}
}

void Avatar::Render(float aDeltaTime)
{
	myAnimation.Render(aDeltaTime);
}

void Avatar::SetWorldPosition(Vector2<float> aPosition)
{
	myWorldSpacePosition = aPosition;
}

Vector2<float> Avatar::GetWorldSpacePosition()
{
	return myWorldSpacePosition;
}

Vector2<float> Avatar::GetCameraSpacePosition()
{
	return myCameraSpacePosition;
}

void Avatar::SetDirection(int aDirection)
{
	myDirection = aDirection;
}

int Avatar::GetDirection()
{
	return myDirection;
}

void Avatar::SetTranslatingSpeed(float aSpeed)
{
	myTranslatingSpeed = aSpeed;
}

float Avatar::GetTranslatingSpeed()
{
	return myTranslatingSpeed;
}

bool Avatar::InsideExtendedBoundingBox( Vector2<float> &aWorldSpacePosition )
{
	if( (myWorldSpacePosition.myX < aWorldSpacePosition.myX + 100) && (myWorldSpacePosition.myX > aWorldSpacePosition.myX - 100) )
	{
		if( (myWorldSpacePosition.myY < aWorldSpacePosition.myY + 100) && (myWorldSpacePosition.myY > aWorldSpacePosition.myY - 100) )
		{
			myIsInside = true;
			return true;
		}
	}

	myIsInside = false;

	return false;

}

void Avatar::HaltAvatar()
{
	if(myDirection % 2 != 0)
	{
		myDirection -= 1;
	}
	myAnimation.SetActiveSheet(myDirection);
}

void Avatar::StopAnimation()
{
	myAnimation.Pause();
}

void Avatar::StartAnimation()
{
	myAnimation.Play();
}

AnimationBase& Avatar::GetAnimationRef()
{
	return myAnimation;
}

void Avatar::Render(RenderData &someRenderData)
{
	RenderMessage tempMessage;
	tempMessage.myPosition = myCameraSpacePosition;
	tempMessage.myType = animationRenderMessageType;
	someRenderData.myMessages.push_back( tempMessage );
	someRenderData.myAnimations.push_back( &myAnimation );
}

void Avatar::UpdateCameraSpacePosition(Vector2<float> aCameraPosition)
{
	myCameraSpacePosition = myWorldSpacePosition - aCameraPosition;
}

int Avatar::CreatePoints(Vector2<float>& aWorldPosition, CollisionMap aCollisionMap)
{
	std::vector<Vector2<float> > tempPointsToTest;		

	Vector2<float> tempVector4(aWorldPosition.x + myAnimation.GetSpriteWidth() - 21.0f, aWorldPosition.y + myAnimation.GetSpriteHeight() - 28.0f);
	tempPointsToTest.push_back(tempVector4);

	Vector2<float> tempVector3(aWorldPosition.x + myAnimation.GetSpriteWidth() - 50.0f, aWorldPosition.y + myAnimation.GetSpriteHeight() - 28.0f);
	tempPointsToTest.push_back(tempVector3);

	Vector2<float> tempVector2(aWorldPosition.x + myAnimation.GetSpriteWidth() - 21.0f, aWorldPosition.y + myAnimation.GetSpriteHeight() - 50.0f);
	tempPointsToTest.push_back(tempVector2);

	Vector2<float> tempVector1(aWorldPosition.x + myAnimation.GetSpriteWidth() - 50.0f, aWorldPosition.y + myAnimation.GetSpriteHeight() - 50.0f);
	tempPointsToTest.push_back(tempVector1);

	int tempInt = 0;


	for(unsigned int i = 0; i < tempPointsToTest.size(); i++)
	{
		if ( aCollisionMap.CheckIfPassable(tempPointsToTest[i]) == true )
		{
			tempInt++;
		}
	}

	return (tempInt);
}