#ifndef _AVATAR_H_
#define _AVATAR_H_
#include "animationbase.h"
#include "inputwrapper/inputwrapper.h"
#include "StaticSprite.h"
#include "CollisionMap.h"
#include "RenderInformation.h"

class Avatar
{
public:
	enum Direction
	{
		UP_STAND,			// 0
		UP_WALK,			// 1

		UP_RIGHT_STAND,		// 2
		UP_RIGHT_WALK,		// 3

		RIGHT_STAND,		// 4
		RIGHT_WALK,			// 5

		RIGHT_DOWN_STAND,	// 6
		RIGHT_DOWN_WALK,	// 7

		DOWN_STAND,			// 8
		DOWN_WALK,			// 9

		DOWN_LEFT_STAND,	// 10
		DOWN_LEFT_WALK,		// 11

		LEFT_STAND,			// 12
		LEFT_WALK,			// 13

		LEFT_UP_STAND,		// 14
		LEFT_UP_WALK		// 15
	};

	Avatar();
	Avatar(HGE* aHge, Vector2<float> aPositionInWorld);
	Avatar(const Avatar& aAvatar);
	~Avatar();

	void SetDirection(int aDirection);
	void SetTranslatingSpeed(float aSpeed);
	void SetWorldPosition(Vector2<float> aPosition);

	void UpdateCameraSpacePosition(Vector2<float> aCameraPosition);
	void Update(float aDeltaTime, Vector2<float> aCameraPosition);

	Vector2<float> GetCameraSpacePosition();
	Vector2<float> GetWorldSpacePosition();	
	int GetDirection();
	float GetTranslatingSpeed();

	void LoadAnimationSheet();
	void Initialize(Vector2<float> aCameraPosition);
	void Input(InputWrapper &aInput, Vector2<float>& aCamPosition, CollisionMap aCollisionMap );
	
	void Render(float aDeltaTime);
	void Render(RenderData &someRenderData);

	void HaltAvatar();
	void StopAnimation();
	void StartAnimation();

	bool InsideExtendedBoundingBox( Vector2<float> &aRobotInCameraSpaceCoordinates );

	int CreatePoints(Vector2<float>& aWorldPosition, CollisionMap aCollisionMap);

	bool myIsInside;

	AnimationBase& GetAnimationRef();

	StaticSprite interractionButton;

private:

	int myDirection;
	int myLastDirection;

	HGE* myHge;
	AnimationBase myAnimation;

	Vector2<float> myWorldSpacePosition;
	Vector2<float> myCameraSpacePosition;
	Vector2<float> myScale;
	float myRotation;
	float myTranslatingSpeed;

};
#endif