#ifndef _BIRD_CUTSCENE_H_
#define _BIRD_CUTSCENE_H_
#include "Vector.h"
#include "StaticSprite.h"

class BirdCutscene
{
public:
	BirdCutscene(){}
	BirdCutscene( HGE *aHGE )
	{
		myPosition = Vector2<float>( 0.0f, 0.0f );
		myBirdPosition = Vector2<float>( 0.0f, -125.0f );
		myColor = Vector4<int>( 255, 255, 255, 0 );
		myBirdColor = Vector4<int>( 255, 255, 255, 0 );
		myBirdSprite = StaticSprite( aHGE, "img/CutsceneImages/cutscene_bird.DDS" );
		myBlackSprite = StaticSprite( aHGE, "img/curtain.png" );
		myBirdSprite.SetPosition( myBirdPosition );
		myBlackSprite.SetPosition( myPosition );
		myBirdSprite.SetColor(myBirdColor);
		myBlackSprite.SetColor(myColor);
	
		
		myCutsceneFinish = false;

		myBlackFadeIn = true;
		myBirdFadeIn = false;
		myStartTimer = false;
		myBirdFadeOut = false;
		myBlackFadeOut = false;

		myBlackIsFading = false;
		myBirdIsFading = false;

		myTimer = 0.0;
	}
	~BirdCutscene(){}

	void Play(float aElapseTime)
	{
		if(myBlackFadeIn == true)
		{
			FadeBlackIn();
		}
		if(myBirdFadeIn == true)
		{
			FadeBirdIn();
			if(myBirdFadeIn == false)
			{
				myStartTimer = true;
			}
		}
		if(myStartTimer == true)
		{
			myTimer += aElapseTime;
			if(myTimer >= 5000)
			{
				myBirdFadeOut = true;
			}
		}
		if(myBirdFadeOut == true)
		{
			FadeBirdOut();
		}
		if(myBlackFadeOut == true)
		{
			FadeBlackOut();
		}
	}
	void FadeBlackIn()
	{
		myBlackIsFading = true;
		myColor.a += 1;
		if(myColor.a >= 255)
		{
			myColor.a = 255;
			myBlackFadeIn = false;
			myBirdFadeIn = true;
			
		}
		myBlackSprite.SetColor(myColor);
	}
	void FadeBlackOut()
	{
		myBlackIsFading = true;
		myColor.a -= 1;
		if(myColor.a <= 0)
		{
			myColor.a = 0;
			myCutsceneFinish = true;
		}
		myBlackSprite.SetColor(myColor);
	}

	void FadeBirdIn()
	{
		myBirdIsFading = true;
		myBirdColor.a += 1;
		if(myBirdColor.a >= 255)
		{
			myBirdColor.a = 255;
			myBirdFadeIn = false;
		}
		myBirdSprite.SetColor(myBirdColor);
	}

	void FadeBirdOut()
	{
		myBirdIsFading = true;
		myBirdColor.a -= 1;
		if(myBirdColor.a <= 0)
		{
			myBirdColor.a = 0;
			myBirdFadeOut = false;
			myBlackFadeOut = true;
		}
		myBirdSprite.SetColor(myBirdColor);
	}

	void Render( RenderData &someRenderData )
	{
		RenderMessage tempMessage;
		if(myBlackIsFading == true)
		{
			tempMessage.myPosition = myPosition;
			tempMessage.myType = spriteRenderMessageType;
			someRenderData.myMessages.push_back(tempMessage);
			someRenderData.mySprites.push_back(&myBlackSprite);
		}
		if(myBirdIsFading == true)
		{
			tempMessage.myPosition = myBirdPosition;
			tempMessage.myType = spriteRenderMessageType;
			someRenderData.myMessages.push_back(tempMessage);
			someRenderData.mySprites.push_back(&myBirdSprite);
		}

	}

	bool myCutsceneFinish;
private:
	StaticSprite myBirdSprite;
	StaticSprite myBlackSprite;
	Vector2<float> myPosition;
	Vector2<float> myBirdPosition;
	Vector4<int> myColor;
	Vector4<int> myBirdColor;

	float myTimer;
	bool myStartTimer;
	bool myBirdFadeIn;
	bool myBirdFadeOut;
	bool myBlackFadeIn;
	bool myBlackFadeOut;

	bool myBlackIsFading;
	bool myBirdIsFading;
};
#endif