#include "Cave.h"
#include "EventSingelton.h"

Cave::Cave(void)
:LevelBaseData()
{
}

Cave::Cave(const LevelBaseData& aLevelBaseData)
:LevelBaseData(aLevelBaseData)
{

}

Cave::~Cave(void)
{
}

void Cave::Initialize(HGE *aHGE)
{
	myHge = aHGE;

	StaticSprite tempSprite = StaticSprite(myHge, "img/Levels/scene_cave.dds");
	myMap.AddFloorSprite(tempSprite, Vector2<float>(0, 0));
	tempSprite = StaticSprite(myHge, "img/Levels/scene_caves_overlay.dds");
	myMap.AddOverlaySprite(tempSprite, Vector2<float>(0, 0));
	myCollisionMap = CollisionMap(16, "img/Levels/collision_cave.bmp", myHge);

	myItems.push_back(Item (myHge, "img/items/sprite_sulfur.dds", "img/items/icon_sulfur.dds", Vector2<float>(212.0f, 420.0f), "Sulphur"));
	myItems.push_back(Item (myHge, "img/Items/sprite_guano.dds", "img/Items/icon_coconutGuano.dds", Vector2<float>(432.0f, 337.0f), "Guano"));

	myRobot = Robot(myHge);
	myRobot.Initialize();
	myRobot.SetWorldSpacePosition( Vector2<float>( -10000.0f, -10000.0f ) );

	InitializeConversations();

	myThisLevelMusic = "Vulcano";

	myMagneticStonePH = MagneticStonePH(myHge, "img/items/sprite_magnetic_stone.dds", Vector2<float>( 0, 352 ) );

	myCaveHasBeenBlownUp = false;

	myEndOfGame = false;
}

void Cave::LevelSpecificHandleInput(InputWrapper &aInputWrapper, float aDelta, Inventory &aInventory)
{
	aDelta;
	if((aInputWrapper.KeyPressed(DIK_E)) || (aInputWrapper.KeyPressed(DIK_RETURN)))
	{
		if(myDoneConversation.at(GetConversationIndex("Activate Roderick")) == false)
		{
			myRobot.MakeActive();
		}

		if( myMagneticStonePH.IsActive( ) == true )
		{
			if( myDoneConversation.at(GetConversationIndex("Close to rock")) == false && aInventory.CheckForItem("gunpowder") == true )
			{
				myActiveConversationID = "bye bye cave";
			}
		}
	}
}

void Cave::LevelSpecificUpdate(Avatar &aAvatar, Camera &aCamera, float aDelta)
{
	Vector2<float> tempVector = aAvatar.GetWorldSpacePosition();
	//myRobot.ChaseAvatar( (aAvatar.GetWorldSpacePosition()), aDelta);
	myRobot.Update(aDelta, aCamera.myPosition, tempVector );
	myMagneticStonePH.Update( aCamera.myPosition, tempVector );
}

void Cave::LevelSpecificRender(RenderData &aRenderData, Camera &aCamera)
{
	aCamera;
	myRobot.Render(aRenderData);
	myMagneticStonePH.Render( aRenderData );
}

void Cave::InitializeConversations()
{
	myConversationList.reserve(101);
	myConversationList.push_back(LevelConversation(myHge, "NONE", false));

	myConversationList.push_back (LevelConversation(myHge, "Entering", false));
	myConversationList.push_back (LevelConversation(myHge, "guano eww", false));
	myConversationList.push_back (LevelConversation(myHge, "Picked up guano", false));
	myConversationList.push_back (LevelConversation(myHge, "Picked up sulphur", false));
	myConversationList.push_back (LevelConversation(myHge, "Close to rock", false));
	myConversationList.push_back (LevelConversation(myHge, "bye bye cave", false));

	for(unsigned int index = 0; index < myConversationList.size(); index++)
	{
		myConversationList[index].Initialize();
		myConversationList[index].myActiveConversation = false;
		myDoneConversation.set(index);
	}

	myConversationList[GetConversationIndex("Entering")].myDialoge.SetLeftPortrait( "img/Conversation/heroinePortrait.dds", "Esther");
	//myConversationList[GetConversationIndex("Entering")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("Entering")].AddDialogeText("Roderick: : Bzz...zz...;");
	myConversationList[GetConversationIndex("Entering")].AddDialogeText("Esther: : What's Wrong, Roderick?;");
	myConversationList[GetConversationIndex("Entering")].AddDialogeText("Roderick: : Sfrzzkk... I feel-feel-feel obligued-Kzzt- inform yo-you that we Tksrff- enter-entering a string-string-st-Magnetic field.;");
	myConversationList[GetConversationIndex("Entering")].AddDialogeText("Roderick: : Plea-zzzzk-do not put cred-it-i-it cards, cell phones and p-pacemakers Skkrffzzz o-on-n Krzzkt...;");
	myConversationList[GetConversationIndex("Entering")].AddDialogeText("Esther: : Brilliant, a broken robot... and what is this stench in here?;");

	myConversationList[GetConversationIndex("guano eww")].myDialoge.SetLeftPortrait( "img/Conversation/heroinePortrait.dds", "Esther");
	//myConversationList[GetConversationIndex("guano eww")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("guano eww")].AddDialogeText("Esther: : Ah, guano. So that's what smells. Fascinating.;");
	myConversationList[GetConversationIndex("guano eww")].AddDialogeText("Esther: : Hey, did you know they used to make gunpowder out of this gunk? Who'd have known bat-poo would be loaded with salpetre.;");
	myConversationList[GetConversationIndex("guano eww")].AddDialogeText("Esther: : Oh right. You're conked out... Anyway, I'd better keep some of it. You never know when it can come in handy.;");
	myConversationList[GetConversationIndex("guano eww")].AddDialogeText("Esther: : But what should I put it in? Can't just do it with my hands. Perhaps some sort of bowl or shell... hmm...;");
	
	myConversationList[GetConversationIndex("Picked up guano")].myDialoge.SetLeftPortrait( "img/Conversation/heroinePortrait.dds", "Esther");
	//myConversationList[GetConversationIndex("Picked up guano")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("Picked up guano")].AddDialogeText("Esther: : Maybe I can scoop up the bat poop in the coconut shell.;");
	myConversationList[GetConversationIndex("Picked up guano")].AddDialogeText("Esther: : There! What handy things coconuts are, I never knew. Now open wide, Roderick!;");
	myConversationList[GetConversationIndex("Picked up guano")].AddDialogeText("Roderick: : N-nzzkrt-N-no...;");
	myConversationList[GetConversationIndex("Picked up guano")].AddDialogeText("Esther: : Oh, don't be silly. It doesn't smell that much.;");

	myConversationList[GetConversationIndex("Picked up sulphur")].myDialoge.SetLeftPortrait( "img/Conversation/heroinePortrait.dds", "Esther");
	//myConversationList[GetConversationIndex("Picked up sulphur")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("Picked up sulphur")].AddDialogeText("Esther: : Ohhh! Thats quite the sulphur pile! Let's pick that up, it's probably gonna become useful if we ever want to stink something out or make a bomb of some sort.;");
	myConversationList[GetConversationIndex("Picked up sulphur")].AddDialogeText("Roderick: : ... Bzzz-Bom-b ... Sal... petr-ic ...;");

	myConversationList[GetConversationIndex("Close to rock")].myDialoge.SetLeftPortrait( "img/Conversation/heroinePortrait.dds", "Esther");
	//myConversationList[GetConversationIndex("Close to rock")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Esther: : Here it is! Magnetic Rock!;");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Esther: : And of course I can't get it because its stuck to the wall. Oh man. We're lucky enough to find something magnetic, but it's stuck in a cave wall! Blast it all!;");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Esther: : Hm... Blast... That might be an idea.;");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Esther: : How can we make a blast, Roderick?;");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Roderick: : ...;");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Esther: : Alright then. Not getting anything useful out of you at the moment. Now let's think... I'm a scientist... Thats what I do... Hmmm... Heureka, I got it!;");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Esther: : The fireworks I made when I was young, combined with my brilliant scientific mind, has made a marvelous discovery! Now let's see what we should have to get to create something sufficient for the task...;");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Esther: : To make a basic destructive element I would need some salpetric acid, regular charcoal and some sulphur!;");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Esther: : I recall that during the interesting nights watching the discovery channel, some one mentioned that Guano, bat poop, may have salpetric acids...;");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Esther: : Charcoal shouldn't be hard to make, just need to burn some dry wood somehow...;");
	myConversationList[GetConversationIndex("Close to rock")].AddDialogeText("Esther: : Sulphur... Frequently acompanied by volcanic regions! Or vice versa actually... Should be able to find something around here, been smelling it all along!;");

	myConversationList[GetConversationIndex("bye bye cave")].myDialoge.SetLeftPortrait( "img/Conversation/heroinePortrait.dds", "Esther");
	//myConversationList[GetConversationIndex("bye bye cave")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("bye bye cave")].AddDialogeText("Esther: : You ready, Roderick? I'm going to ignite the gunpowder now.;");
	myConversationList[GetConversationIndex("bye bye cave")].AddDialogeText("Roderick: : Bzzrt!;");
	myConversationList[GetConversationIndex("bye bye cave")].AddDialogeText("Esther: : Thought you'd say that. So here we go...;");
	myConversationList[GetConversationIndex("bye bye cave")].AddDialogeText("Esther: : There, it's lit! ... Wait! How could I miss that! If I push here... and here... pull here a bit and poke here... YES! I got the stone out Roderick! Now we don't have to blow anything up!;");
	myConversationList[GetConversationIndex("bye bye cave")].AddDialogeText("Esther: : Which reminds me... I already lit the fuse, so... I'M OUTA HERE!;");

	myActiveConversationID = "";
	myActiveConversationIndex = 0;
}

void Cave::ConversationEvent(float elapsTime, Avatar &aAvatar, Inventory &aInventory)
{
	aAvatar;
	myConversationDelta += elapsTime;

	if(myDoneConversation.at(GetConversationIndex("Entering")) == true )
	{
		myActiveConversationID = "Entering";
	}

	if(myDoneConversation.at(GetConversationIndex("Picked up sulphur")) == true && aInventory.CheckForItem("Sulphur") == true )
	{
		myActiveConversationID = "Picked up sulphur";
	}

	if( aInventory.CheckForItem( "Guano" ) )
	{
		if (myDoneConversation.at(GetConversationIndex("guano eww")) == true)
		{
			myActiveConversationID = "guano eww";
		}		
		else if ( aInventory.CheckForItem( "coconut half 2" ) )
		{
			aInventory.DeleteItem( "coconut half 2" );
			myActiveConversationID = "Picked up guano";
		}

		if (myDoneConversation.at(GetConversationIndex("Picked up guano")) == true)
		{
			Item guano;
			if ( aInventory.TakeOutItem("Guano", guano) == true )
			{
				myItems.push_back(guano);
			}
		}		
	}

	if( myMagneticStonePH.IsInterractable( ) == true && myMagneticStonePH.IsActive( ) == false )
	{
		myActiveConversationID = "Close to rock";
		myMagneticStonePH.MakeActive( );
	}

	if( myDoneConversation.at(GetConversationIndex("bye bye cave") ) == false)
	{
		Item tempItem(myHge, "img/items/sprite_magnetic_stone.dds", "img/items/icon_magneticStone.dds", Vector2<float>(1000.0f, 1000.0f), "MagneticRock");
		aInventory.AddItem( tempItem );
		aInventory.DeleteItem( "gunpowder" );
		myCaveHasBeenBlownUp = true;
		EventSingelton::GetInstance()->myCaveBlownUp = true;
		myChangedLevelIndex = 5;
		myChangedLevel = true;
	}
}