#include "CollisionMap.h"
#include <string>
#include <assert.h>

CollisionMap::CollisionMap()
{
	myInitiated = false;
}

CollisionMap::CollisionMap(int aPixTileSize, std::string aCollisionCollisionMapFileName, HGE* aHge)
{
	myInitiated = true;
	myPixTileSize = aPixTileSize;
	myHGE = aHge;

	myCollisionTexture = myHGE->Texture_Load(aCollisionCollisionMapFileName.c_str());
	assert(myCollisionTexture && "Can't load the sheet!");
	myPixWidth = myHGE->Texture_GetWidth(myCollisionTexture);
	myDimensions.x = static_cast<float>(myPixWidth);
	myDimensions.y =  static_cast<float>(myHGE->Texture_GetHeight(myCollisionTexture));
}

CollisionMap::CollisionMap(const CollisionMap &aCollisionMap)
{
	myPixTileSize = aCollisionMap.myPixTileSize;
	myHGE = aCollisionMap.myHGE;
	myCollisionTexture = aCollisionMap.myCollisionTexture;
	myInitiated = aCollisionMap.myInitiated;
	myPixWidth = aCollisionMap.myPixWidth;
	myDimensions = aCollisionMap.myDimensions;
}

CollisionMap::~CollisionMap()
{
	
}

int CollisionMap::ConvertToIndex(Vector2<float> aCollisionMapPosition)
{
	assert(myInitiated && "Tried to use an uninitiated CollisionMap.");
	Vector2<int> tileCollisionMapPosition;
	tileCollisionMapPosition.x = static_cast<int>(aCollisionMapPosition.x / myPixTileSize);
	tileCollisionMapPosition.y = (static_cast<int>(aCollisionMapPosition.y / myPixTileSize));
	int result = (tileCollisionMapPosition.y * myPixWidth) + tileCollisionMapPosition.x;
	return result;
}

bool CollisionMap::CheckIfPassable(Vector2<float> aCollisionMapPosition)
{
	assert(myInitiated && "Tried to use an uninitiated CollisionMap.");
	DWORD *textureData = myHGE->Texture_Lock( myCollisionTexture );
	DWORD pixelColor = textureData[ConvertToIndex(aCollisionMapPosition)];
	myHGE->Texture_Unlock( myCollisionTexture );
	
	if (pixelColor == 4278190080) // black
	{
		return false;
	}
	return true;
}

Vector2<float> CollisionMap::GetDimensions()
{
	return myDimensions;
}