#include "DeepJungle.h"
#include <assert.h>
#include "Item.h"

DeepJungle::DeepJungle(void)
:LevelBaseData()
{
}

DeepJungle::DeepJungle(const LevelBaseData& aLevelBaseData)
:LevelBaseData(aLevelBaseData)
{

}

DeepJungle::~DeepJungle(void)
{
}

void DeepJungle::Initialize(HGE *aHGE)
{
	myHge = aHGE;

	StaticSprite tempSprite = StaticSprite(myHge, "img/Levels/scene_deepjungle.dds");
	myMap.AddFloorSprite(tempSprite, Vector2<float>(0, 0));
	tempSprite = StaticSprite(myHge, "img/Levels/scene_deepjungle_overlay.dds");
	myMap.AddOverlaySprite(tempSprite, Vector2<float>(0, 0));
	myCollisionMap = CollisionMap(16, "img/Levels/collision_deepjungle.bmp", myHge);

	myRobot = Robot(myHge);
	myRobot.Initialize();

	InitializeConversations();

	myThisLevelMusic = "Jungle";

	myItems.push_back(Item( myHge, "img/empty.dds", "img/Items/icon_coconut.dds", Vector2<float>(721.0f, 622.0f), "coconut"));
	//myItems.push_back(Item( myHge, "img/empty.dds", "img/Items/icon_coconutLatex.dds", Vector2<float>(199, 656), "rubber"));
	myItems.push_back(Item( myHge, "img/empty.dds", "img/empty.dds", Vector2<float>(199, 656), "rubber tree"));
	myItems.push_back(Item( myHge, "img/empty.dds", "img/Items/icon_log.dds", Vector2<float>(431, 289), "log"));
	myItems.push_back(Item( myHge, "img/Items/sprite_dried_leaves.dds", "img/Items/icon_dried_leaves.dds", Vector2<float>(267, 425), "dried leaves"));

	myEndOfGame = false;
}

void DeepJungle::LevelSpecificHandleInput(InputWrapper &aInputWrapper, float aDelta, Inventory &aInventory)
{
	aInventory;
	aDelta;
	if((aInputWrapper.KeyPressed(DIK_E)) || (aInputWrapper.KeyPressed(DIK_RETURN)))
	{
		if(myDoneConversation.at(GetConversationIndex("Activate Roderick")) == false)
		{
			myRobot.MakeActive();
		}
	}
}

void DeepJungle::LevelSpecificUpdate(Avatar &aAvatar, Camera &aCamera, float aDelta)
{
	Vector2<float> tempVector = aAvatar.GetWorldSpacePosition( );
	myRobot.ChaseAvatar( tempVector, aDelta);
	myRobot.Update(aDelta, aCamera.myPosition, tempVector );
}

void DeepJungle::LevelSpecificRender(RenderData &aRenderData, Camera &aCamera)
{
	aCamera;
	myRobot.Render(aRenderData);
}

void DeepJungle::InitializeConversations()
{
	myConversationList.reserve(101);
	myConversationList.push_back(LevelConversation(myHge, "NONE", false));

	myConversationList.push_back(LevelConversation(myHge, "palm", false));
	myConversationList.push_back(LevelConversation(myHge, "no axe", false));
	myConversationList.push_back(LevelConversation(myHge, "no shell", false));
	myConversationList.push_back(LevelConversation(myHge, "tapped", false));
	myConversationList.push_back(LevelConversation(myHge, "rubber", false));
	myConversationList.push_back(LevelConversation(myHge, "log", false));
	myConversationList.push_back(LevelConversation(myHge, "explosion escapeist", false));
	myConversationList.push_back(LevelConversation(myHge, "dry leafs", false));

	for(unsigned int index = 0; index < myConversationList.size(); index++)
	{
		myConversationList[index].Initialize();
		myConversationList[index].myActiveConversation = false;
		myDoneConversation.set(index);
	}

	myConversationList[GetConversationIndex("palm")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("palm")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("palm")].AddDialogeText("Esther: : There is a coconut up in this palm. Maybe if I kick it really hard...;");
	myConversationList[GetConversationIndex("palm")].AddDialogeText("Esther: : Eureka! I have a coconut!;");
	myConversationList[GetConversationIndex("palm")].AddDialogeText("Rodrick: : Since you apparently don't know this I feel obliged to point out that the phrase 'eureka' is reserved for scientific discovery. And even if you might be the first human to encounter a coconut, this is not one.;");
	myConversationList[GetConversationIndex("palm")].AddDialogeText("Esther: : Delicious looking coconut...;");
	myConversationList[GetConversationIndex("palm")].AddDialogeText("Rodrick: : ...;");

	myConversationList[GetConversationIndex("no axe")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("no axe")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");	
	myConversationList[GetConversationIndex("no axe")].AddDialogeText("Esther: : I could cut a hole in the tree and tap some of the rubber. That is how they do it on rubber plantations.;");
	myConversationList[GetConversationIndex("no axe")].AddDialogeText("Roderick: : Cut a hole with what, exactly?;");
	myConversationList[GetConversationIndex("no axe")].AddDialogeText("Esther: : That is a more urgent problem, of course...;");

	myConversationList[GetConversationIndex("no shell")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("no shell")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");	
	myConversationList[GetConversationIndex("no shell")].AddDialogeText("Esther: : Well, I have an axe. So I suppose I could cut a hole in the bark and tap some of the rubber.;");
	myConversationList[GetConversationIndex("no shell")].AddDialogeText("Roderick: : And gather the rubber in what, exactly?;");
	myConversationList[GetConversationIndex("no shell")].AddDialogeText("Esther: : You?;");
	myConversationList[GetConversationIndex("no shell")].AddDialogeText("Roderick: : No. Rubber is liquid. I do not want liquid without container in my box. Neither would it be of any use to you spilled.;");
	myConversationList[GetConversationIndex("no shell")].AddDialogeText("Esther: : That's true. I should've given you jugs instead of boxes.;");
	myConversationList[GetConversationIndex("no shell")].AddDialogeText("Roderick: : Why don't you go back in time and fix this problem?;");
	myConversationList[GetConversationIndex("no shell")].AddDialogeText("Esther: : Ha. Ha. Very funny.;");

	myConversationList[GetConversationIndex("tapped")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("tapped")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");	
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Esther: : I could easily tap some of the rubber in a coconut shell if I cut a hole in the tree with my axe.;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Roderick: : Somewhat easily. I am sure it will take some time.;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Esther: : Maybe, but I'm not exactly in a hurry. Let's see...ha!;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Roderick: : How cruel, cutting the tree like that.;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Esther: : Look, rubber is coming out!;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Roderick: : ...;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Esther: : ...;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Roderick: : Very slowly.;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Esther: : Patience.;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Roderick: : ...;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Esther: : ...;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Roderick: : There! A full coconut of rubber. That was fun.;");
	myConversationList[GetConversationIndex("tapped")].AddDialogeText("Esther: : Very.;");

	myConversationList[GetConversationIndex("rubber")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("rubber")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("rubber")].AddDialogeText("Esther: : I think this might be a rubber tree.;");
	myConversationList[GetConversationIndex("rubber")].AddDialogeText("Roderick: : My sensors show that you are, for once, correct;");
	myConversationList[GetConversationIndex("rubber")].AddDialogeText("Esther: : For once?;");
	myConversationList[GetConversationIndex("rubber")].AddDialogeText("Roderick: : Does this seem familiar to you: 'Oh, time travel is a great idea, I am sure this engine is sturdy enough' or is it just me remembering things?;");
	myConversationList[GetConversationIndex("rubber")].AddDialogeText("Esther: : Oh, shut up.;");
	myConversationList[GetConversationIndex("rubber")].AddDialogeText("Roderick: : Also, did you know that the chemical process of converting rubber to more durable materials is called vulcanization?;");
	myConversationList[GetConversationIndex("rubber")].AddDialogeText("Esther: : No, I didn't. Hm... perhaps some rubber can come in handy?;");
	myConversationList[GetConversationIndex("rubber")].AddDialogeText("Roderick: : Many have thought this thought, and many have been right.;");	

	myConversationList[GetConversationIndex("log")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	//myConversationList[GetConversationIndex("rubber")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("log")].AddDialogeText("Well, well, well. Dry, perfectly good firewood. Better hold on to it.;");

	myConversationList[GetConversationIndex("explosion escapeist")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("explosion escapeist")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("explosion escapeist")].AddDialogeText("Esther: : Phew! I'm glad I made it out of there alive, that was one hell of a blast! That bat really didn't eat very well to have that much salpetric acid in his poop...;");
	myConversationList[GetConversationIndex("explosion escapeist")].AddDialogeText("Roderick: : I'm glad you made it out alive, Esther, with a seemingly Magnetic Rock;");
	myConversationList[GetConversationIndex("explosion escapeist")].AddDialogeText("Esther: : You're being awefully nice all of a sudden Roderick, did the bat poop change your mind?;");
	myConversationList[GetConversationIndex("explosion escapeist")].AddDialogeText("Roderick: : Yes, because now I still have someone to mock with my overpowering mind.;");
	myConversationList[GetConversationIndex("explosion escapeist")].AddDialogeText("Esther: : And yet you stand powerless against some magnetism... Maybe I should let you carry it?;");
	myConversationList[GetConversationIndex("explosion escapeist")].AddDialogeText("Roderick: : ARGH! KEEP THAT THING AWAY FROM ME!;");
	myConversationList[GetConversationIndex("explosion escapeist")].AddDialogeText("Esther: : Fine, fine. I'll carry it... Useless robot...;");
	myConversationList[GetConversationIndex("explosion escapeist")].AddDialogeText("Roderick: : Ahem.;");

	myConversationList[GetConversationIndex("dry leafs")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("dry leafs")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("dry leafs")].AddDialogeText("Esther: : Look Roderick, some dry leafs. I remember when I was a little kid and made fires with dry leafs and some flint stones while watching up towards the sky, pondering...;");
	myConversationList[GetConversationIndex("dry leafs")].AddDialogeText("Esther: : Yes indeed, you can make fire with dry leafs, if you have the flintstones for it.;");

	myActiveConversationID = "";
	myActiveConversationIndex = 0;
}

void DeepJungle::ConversationEvent(float elapsTime, Avatar &aAvatar, Inventory &aInventory)
{
	aAvatar;
	
	if ( aInventory.CheckForItem("log") && myDoneConversation.at(GetConversationIndex("log")) == true )
	{
		myActiveConversationID = "log";	
	}

	if( aInventory.CheckForItem("dried leaves") && myDoneConversation.at(GetConversationIndex("dry leafs")) == true )
	{
		myActiveConversationID = "dry leafs";
	}

	myConversationDelta += elapsTime;

	if ( aInventory.CheckForItem("coconut") && myDoneConversation.at(GetConversationIndex("palm")) == true )
	{
		myActiveConversationID = "palm";	
	}

	if ( aInventory.CheckForItem("rubber tree") == true )
	{
		myActiveConversationID = "rubber";

		if ( myDoneConversation.at(GetConversationIndex("rubber")) == false )
		{
			if ( aInventory.CheckForItem("axe") == false )
			{
				if ( myDoneConversation.at(GetConversationIndex("no axe")) == true )
				{
					myActiveConversationID = "no axe";		
				}
			}
			else if ( aInventory.CheckForItem("coconut half 1") == false 
				&& myDoneConversation.at(GetConversationIndex("tapped")) == true)
			{
				if (myDoneConversation.at(GetConversationIndex("no shell")) == true)
				{
					myActiveConversationID = "no shell";
				}
			}
			else if ( myDoneConversation.at(GetConversationIndex("tapped")) == true )
			{
				myActiveConversationID = "tapped";
				aInventory.DeleteItem("coconut half 1");

				Item rubber = Item( myHge, "img/empty.dds", "img/Items/icon_coconutLatex.dds", Vector2<float>(199, 656), "rubber");
				aInventory.AddItem(rubber);
			}
		}

		Item rubberTree;
		if (aInventory.TakeOutItem("rubber tree", rubberTree) == true)
		{
			if ( aInventory.CheckForItem("rubber") == false)
			{
				myItems.push_back(rubberTree);
			}
		}
	}

	if( aInventory.CheckForItem("MagneticRock") == true && myDoneConversation.at(GetConversationIndex("explosion escapeist")) == true)
	{
		myActiveConversationID = "explosion escapeist";
	}
}