#ifndef ___HOT_SPOT_H___
#define ___HOT_SPOT_H___


#include "Vector.h"


class HotSpot
{

public:

	HotSpot( const HotSpot &aHotSpot )
	{

		myBoxSpot = aHotSpot.myBoxSpot;
		myHotSpotConnex = aHotSpot.myHotSpotConnex;
		myHeight = aHotSpot.myHeight;
		myWidth = aHotSpot.myWidth;
		myID = aHotSpot.myID;

	}

	/*By sending in position the height and width starting point
	is already defined so therefor you only need to send in a
	height and a width.
	
	
	 ______________________________________________________
	|<-aPosition.myX && aPosition.myY              aWidth->|
	|                                                      |
	|                                                      |
	|                                                      |
	|                                                      |
	|                                                      |
	|                                                      |
	|                                                      |
	|                       aHeight                        |
	|                          |                           |
	|                          v                           |
	 */
	HotSpot( Vector2<float> aPosition, float aWidth, float aHeight, std::string anID )
	{

		myBoxSpot.myX = aPosition.myX;
		myBoxSpot.myY = aPosition.myY;
		myBoxSpot.myZ = aPosition.myX + aWidth;
		myBoxSpot.myW = aPosition.myY + aHeight;

		myHeight = aHeight;
		myWidth = aWidth;

		myID = anID;

	}

	~HotSpot( )
	{
	}

	bool IsAvatarInsideMe( Vector2<float> &aPosition )
	{
		if( aPosition.myX < myBoxSpot.myX )
		{
			return false;
		}
		if( aPosition.myY < myBoxSpot.myY )
		{
			return false;
		}
		if( aPosition.myX > myBoxSpot.myZ )
		{
			return false;
		}
		if( aPosition.myY > myBoxSpot.myW )
		{
			return false;
		}

		if( InsideTopOrBottom( aPosition ) == true )
		{
			aPosition = Vector2<float>( myHotSpotConnex->GetBoxSpot( ).myX + myHotSpotConnex->myWidth / 2, myHotSpotConnex->GetBoxSpot( ).myW + myHotSpotConnex->myHeight / 2 );
			aPosition.myX -= 32.0f;
			aPosition.myY -= 100.0f;
			return true;
		}

		aPosition = Vector2<float>( myHotSpotConnex->GetBoxSpot( ).myX + myHotSpotConnex->myWidth / 2, myHotSpotConnex->GetBoxSpot( ).myY - myHotSpotConnex->myHeight / 2 );
		aPosition.myX -= 32.0f;
		aPosition.myY -= 100.0f;
		return true;

	}

	Vector4<float> GetBoxSpot( )
	{
		return myBoxSpot;
	}

	void ConnectHotspot( HotSpot &aHotSpot )
	{
		myHotSpotConnex = &aHotSpot;
	}

	std::string GetHotSpotID( )
	{
		return myID;
	}

private:

	float myHeight;
	float myWidth;

	std::string myID;

	HotSpot *myHotSpotConnex;

	Vector4<float> myBoxSpot;

	HotSpot( )
	{

	}

	bool InsideTopOrBottom( Vector2<float> &aPosition )
	{
		if( aPosition.myY <= myBoxSpot.myY + myHeight / 2 )
		{
			return true;
		}

		return false;
	}

};


#endif