#include "InputWrapper.h"

#include <assert.h>
#include <iostream>

InputWrapper::InputWrapper()
{
	myDIPointer = NULL;
	myDIMousePointer = NULL;
	myDIKeyboardPointer = NULL;

	for(int i = 0; i < 256; i++)
	{
		myKeyState[i] = 0;
		myOldState[i] = 0;
	}

	myMousePosition.myX = 512;
	myMousePosition.myY = 384;
}

InputWrapper::InputWrapper(const InputWrapper &aInputWrapper)
{
	myDIPointer = aInputWrapper.myDIPointer;
	myDIKeyboardPointer = aInputWrapper.myDIKeyboardPointer;
	myDIMousePointer = aInputWrapper.myDIMousePointer;

	for(unsigned int i = 0; i < 256; i++)
	{
		myKeyState[i] = aInputWrapper.myKeyState[i];
		myOldState[i] = aInputWrapper.myOldState[i];
	}

	myMouseState = aInputWrapper.myMouseState;
	myOldMouseState = aInputWrapper.myOldMouseState;

	myPressedKeys = aInputWrapper.myPressedKeys;
	myPressedButtons = aInputWrapper.myPressedButtons;

	myMousePosition = aInputWrapper.myMousePosition;
}

InputWrapper::~InputWrapper()
{
	if(myDIKeyboardPointer != NULL)
	{
		myDIKeyboardPointer->Unacquire();
		myDIKeyboardPointer->Release();
		myDIKeyboardPointer = NULL;
	}

	if(myDIMousePointer != NULL)
	{
		myDIMousePointer->Unacquire();
		myDIMousePointer->Release();
		myDIMousePointer = NULL;
	}

	if(myDIPointer != NULL)
	{
		myDIPointer->Release();
		myDIPointer = NULL;
	}	
}

//Init Code
void InputWrapper::IntializeDirectInput(HGE *aHGE)
{
	//DI
	if(FAILED(DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&myDIPointer, NULL) ) )
	{
		assert(0 && "Initialize DirectInput before use!"); 
	}

	//Keyboard
	if(FAILED(myDIPointer->CreateDevice(GUID_SysKeyboard, &myDIKeyboardPointer, NULL) ) )
	{
		assert(0 && "Initialization of Keyboard failed");
	}

	myDIKeyboardPointer->SetDataFormat(&c_dfDIKeyboard);

	myDIKeyboardPointer->SetCooperativeLevel(aHGE->System_GetState(HGE_HWND), DISCL_NONEXCLUSIVE | DISCL_FOREGROUND | DISCL_NOWINKEY);	

	myDIKeyboardPointer->Acquire();

	//Mouse
	if(FAILED(myDIPointer->CreateDevice(GUID_SysMouse, &myDIMousePointer, NULL) ) )
	{
		assert("Initialization of Mouse failed");
	}

	myDIMousePointer->SetDataFormat(&c_dfDIMouse2);

	myDIMousePointer->SetCooperativeLevel(aHGE->System_GetState(HGE_HWND), DISCL_EXCLUSIVE | DISCL_FOREGROUND);

	myDIMousePointer->Acquire();

}

void InputWrapper::InitializeObject()
{
	if(FAILED(DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&myDIPointer, NULL) ) )
	{
		assert(0 && "Initialize DirectInput before use!"); 
	}
}

void InputWrapper::InitializeKeyboard(HGE *aHGE)
{
	if(FAILED(myDIPointer->CreateDevice(GUID_SysKeyboard, &myDIKeyboardPointer, NULL) ) )
	{
		assert(0 && "Initialization of Keyboard failed");
	}

	myDIKeyboardPointer->SetDataFormat(&c_dfDIKeyboard);

	myDIKeyboardPointer->SetCooperativeLevel(aHGE->System_GetState(HGE_HWND), DISCL_NONEXCLUSIVE | DISCL_FOREGROUND | DISCL_NOWINKEY);	

	myDIKeyboardPointer->Acquire();
}

void InputWrapper::InitializeMouse(HGE *aHGE)
{
	if(FAILED(myDIPointer->CreateDevice(GUID_SysMouse, &myDIMousePointer, NULL) ) )
	{
		assert("Initialization of Mouse failed");
	}

	myDIMousePointer->SetDataFormat(&c_dfDIMouse2);

	myDIMousePointer->SetCooperativeLevel(aHGE->System_GetState(HGE_HWND), DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);

	myDIMousePointer->Acquire();
}


//Keyboard Functions
void InputWrapper::UpdateKeyboard()
{	
	for(int i = 0; i < 256; i++)
	{
		if(myKeyState[i] & 0x80)
		{
			myOldState[i] = myKeyState[i];
		}
	}
	//memcpy(myOldState, myKeyState,sizeof(myKeyState);

	myDIKeyboardPointer->GetDeviceState(sizeof(myKeyState), (LPVOID)&myKeyState);
	myDIKeyboardPointer->Acquire();
	
	//myDIKeyboardPointer->GetDeviceState(sizeof(myKeyState), (LPVOID)&myKeyState);
}

bool InputWrapper::KeyDown(int aKey)
{
	UpdateKeyboard();
	
	if( (myKeyState[aKey] & 0x80) )
	{		
		myKeyState[aKey] = NULL;
		myPressedKeys.reset();
		myPressedKeys[aKey] = DOWN;
		return true;
	}	
	else if(myPressedKeys[aKey] == DOWN)
	{
		myPressedKeys[aKey] = UP;
	}
		
	return false;
}

bool InputWrapper::KeyUp(int aKey)
{
	UpdateKeyboard();

	if( (myKeyState[aKey] & 0x80) == 0)
	{
		myPressedKeys[aKey] = UP;
		return true;
	}
	else if(myPressedKeys[aKey] == UP)
	{
		myPressedKeys[aKey] = DOWN;
	}
	return false;
}

bool InputWrapper::KeyPressed(int aKey)
{
	UpdateKeyboard();

	if(myOldState[aKey] & 0x80)
	{
		if(myKeyState[aKey] & 0x80)
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	return false;	
}

bool InputWrapper::KeyReleased(int aKey)
{
	if(myPressedKeys[aKey] == DOWN)
	{
		if((myKeyState[aKey] & 0x80) == 0)
		{
			return true;
		}
	}
	//Returns true if the key IS released
	return false;
}

bool InputWrapper::AllKeysReleased()
{
	for(int keyIndex = 0; keyIndex < 256; keyIndex++)
	{
		if(myKeyState[keyIndex] & 0x80)
		{
			return false;
		}
	}
	return true;
}
//Mouse Functions
void InputWrapper::UpdateMouse()
{
	for(int i = 0; i < 8; i++)
	{
		if(myMouseState.rgbButtons[i] & 0x80)
		{
			myOldMouseState.rgbButtons[i] = myMouseState.rgbButtons[i];
		}
	}

	myDIMousePointer->GetDeviceState(sizeof(DIMOUSESTATE2), (LPVOID)&myMouseState);
	myDIMousePointer->Acquire();

	if((myMousePosition.myX + (myMouseState.lX) < 1004) && (myMousePosition.myX + (myMouseState.lX) > 0) && (myMousePosition.myY + (myMouseState.lY) < 748) && (myMousePosition.myY + (myMouseState.lY) > 0))
	{
		myMousePosition.myX += (myMouseState.lX);
		myMousePosition.myY += (myMouseState.lY);
	}
}

//MOUSE IN VALUES:
//0 is the left mouse-button, 
//1 is the right button, 
//2 is the middle button, 
//3 is the back button and 
//4 is the forward button
//5,6 and 7 are eventual other buttons

bool InputWrapper::ButtonDown(int aButton)
{	

	if( (myMouseState.rgbButtons[aButton] & 0x80) != 0 )
	{
		myPressedButtons[aButton] = DOWN;
		return true;
	}
	else if(myPressedButtons[aButton] == DOWN)
	{
		myPressedButtons[aButton] = UP;
	}
	return false;
}

bool InputWrapper::ButtonUp(int aButton)
{

	if( (myMouseState.rgbButtons[aButton] & 0x80) == 0 )
	{
		myPressedButtons[aButton] = UP;
		return true;
	}
	else if(myPressedButtons[aButton] == UP)
	{
		myPressedButtons[aButton] = DOWN;
	}
	return true;
}

bool InputWrapper::ButtonReleased(int aButton)
{
	if(myPressedButtons[aButton] == DOWN)
	{
		if(myMouseState.rgbButtons[aButton] & 0x80)
		{
			return false;
		}
	}
	//Returns true if the button IS released!!!
	return true;
}

bool InputWrapper::ButtonClicked(int aButton)
{
	UpdateMouse();

	if(myOldMouseState.rgbButtons[aButton] & 0x80)
	{
		if(myMouseState.rgbButtons[aButton] & 0x80)
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	return false;	
}

Vector2<long> InputWrapper::GetMousePosition()
{
	return (myMousePosition);
}

void InputWrapper::ResetPressedKeys()
{
	for(int i = 0; i < 256; i++)
	{
		myOldState[i] = 0;
	}
}

void InputWrapper::ResetClickedButtons()
{
	for(int i = 0; i < 8; i++)
	{
		myOldMouseState.rgbButtons[i] = 0;
	}
}
