#ifndef INPUT_WRAPPER_H_
#define INPUT_WRAPPER_H_

#include <windows.h>
#include <bitset>

#include "..\HGE\hge.h"
#include "dinput.h"
#include "..\Vector2.h"


class InputWrapper
{
public:
	InputWrapper();
	InputWrapper(const InputWrapper& aInputWrapper);
	~InputWrapper();

	//Master Init
	void IntializeDirectInput(HGE *aHGE);
	
	//Inits
	void InitializeObject();
	void InitializeKeyboard(HGE *aHGE);
	void InitializeMouse(HGE *aHGE);

	//Keyboard Functions
	void UpdateKeyboard();
	bool KeyDown(int aKey);
	bool KeyUp(int aKey);
	bool KeyReleased(int aKey);
	bool KeyPressed(int aKey);


	//Mouse Functions
	void UpdateMouse();
	bool ButtonDown(int aButton);
	bool ButtonUp(int aButton);
	bool ButtonReleased(int aButton);
	bool ButtonClicked(int aButton);

	bool AllKeysReleased();

	Vector2<long> GetMousePosition();

	void ResetPressedKeys();
	void ResetClickedButtons();

private:

	enum 
	{
		UP,		// 0
		DOWN,	// 1
		RELEASED,
	};

	LPDIRECTINPUT8 myDIPointer;
	LPDIRECTINPUTDEVICE8 myDIKeyboardPointer;
	LPDIRECTINPUTDEVICE8 myDIMousePointer;

	char myKeyState[256];
	char myOldState[256];
	DIMOUSESTATE2 myMouseState;
	DIMOUSESTATE2 myOldMouseState;

	std::bitset<256> myPressedKeys;
	std::bitset<8> myPressedButtons;

	Vector2<long> myMousePosition;
};


#endif
