#ifndef ___ITEM_H___
#define ___ITEM_H___


#include "StaticSprite.h"
#include "Vector2.h"
#include "RenderInformation.h"
#include "hge.h"

#include <string>


class Item
{

public:

	Item( const Item &someItem )
	{
		myWorldSprite = someItem.myWorldSprite;
		myInventorySprite = someItem.myInventorySprite;
		myWorldPosition = someItem.myWorldPosition;
		myCameraPosition = someItem.myCameraPosition;
		myAvatarNear = someItem.myAvatarNear;
		myFollowingMouse = someItem.myFollowingMouse;
		myID = someItem.myID;
		myInterractionButton = someItem.myInterractionButton;
		myPickupFlag = someItem.myPickupFlag;
		myDisabled = someItem.myDisabled;
		myInvisible = someItem.myInvisible;
	}

	Item( HGE *aHGE, std::string aWorldFileName, std::string aInventoryFileName, std::string anID, const bool aPickupFlag = true )
	{

		myWorldSprite = StaticSprite( aHGE, aWorldFileName );
		myInventorySprite = StaticSprite( aHGE, aInventoryFileName );
		myID = anID;
		myAvatarNear = false;
		myFollowingMouse = false;
		myInterractionButton = StaticSprite( aHGE, "img/icon/icon_e.dds" );
		myPickupFlag = aPickupFlag;
		myDisabled = false;
		myInvisible = false;
	}

	Item( HGE *aHGE, std::string aWorldFileName, std::string aInventoryFileName, Vector2<float> aPosition, std::string anID, const bool aPickupFlag = true )
	{
		myDisabled = false;
		myWorldSprite = StaticSprite( aHGE, aWorldFileName );
		myInventorySprite = StaticSprite( aHGE, aInventoryFileName );
		myID = anID;
		myWorldPosition = aPosition;
		myFollowingMouse = false;
		myAvatarNear = false;
		myInterractionButton = StaticSprite( aHGE, "img/icon/icon_e.dds" );
		myPickupFlag = aPickupFlag;
		myInvisible = false;
	}

	~Item( )
	{
	}

	void FollowMouse( )
	{
		myFollowingMouse = true;
	}
	void StopFollowMouse( )
	{
		myFollowingMouse = false;
	}
	bool IsFollowingMouse( )
	{
		return myFollowingMouse;
	}
	bool IsInterractable( )
	{
		return myAvatarNear;
	}
	bool IsPickupable() const
	{
		return myPickupFlag;
	}
	void MakeInterractable( )
	{
		myAvatarNear = true;
	}
	void MakeNotInterractable( )
	{
		myAvatarNear = false;
	}

	void MakeNotPickupable() {
		myPickupFlag = false;
	}

	void MakeInvisible()
	{
		myInvisible = true;
		myWorldSprite.SetVisible(false);
	}

	void MakeVisible()
	{
		myInvisible = false;
		myWorldSprite.SetVisible(true);
	}

	void Render( RenderData &someRenderData, float anXPos, float anYPos )
	{
		if ( myInvisible == false)
		{
			RenderMessage tempMessage;

			tempMessage.myPosition = Vector2<float>( anXPos, anYPos );
			tempMessage.myType = spriteRenderMessageType;

			someRenderData.myMessages.push_back( tempMessage );
			someRenderData.mySprites.push_back( &myInventorySprite );
		}
	}

	void RenderAtWorld( RenderData &someRenderData, bool someIsRobotActivated )
	{
		if ( myInvisible == false)
		{
			RenderMessage tempMessage;

			tempMessage.myPosition = myCameraPosition;
			tempMessage.myType = spriteRenderMessageType;

			someRenderData.myMessages.push_back( tempMessage );
			someRenderData.mySprites.push_back( &myWorldSprite );

			if( IsInterractable( ) == true && IsPickupable() == true )
			{
				if( someIsRobotActivated == true)
				{
					tempMessage.myPosition = Vector2<float>( myCameraPosition.myX, myCameraPosition.myY - 50);
					tempMessage.myType = spriteRenderMessageType;

					someRenderData.myMessages.push_back( tempMessage );
					someRenderData.mySprites.push_back( &myInterractionButton );
				}
			}
		}
	}

	void Update( Vector2<float> &aCameraPosition, Vector2<float> &anAvatarPosition )
	{
		myCameraPosition = myWorldPosition - aCameraPosition;

		if (myPickupFlag == true) {
			IsAvatarInProximity( anAvatarPosition );
		}
	}

	bool IsAvatarInProximity( Vector2<float> &anAvatarPosition )
	{
		if( anAvatarPosition.myX < myWorldPosition.myX + 100 && anAvatarPosition.myX > myWorldPosition.myX - 100 )
		{
			if( anAvatarPosition.myY < myWorldPosition.myY + 100 && anAvatarPosition.myY > myWorldPosition.myY - 100 )
			{
				MakeInterractable( );
				return true;
			}
		}

		MakeNotInterractable( );
		return false;
	}

	StaticSprite myWorldSprite;
	StaticSprite myInventorySprite;
	StaticSprite myInterractionButton;

	Vector2<float> myWorldPosition;
	Vector2<float> myCameraPosition;

	bool myFollowingMouse;
	bool myAvatarNear;
	bool myPickupFlag;
	bool myDisabled;
	bool myInvisible;

	std::string myID;

	Item( )
	{

	}

};


#endif