#include "JungleCrashSite.h"
#include "EventSingelton.h"

JungleCrashSite::JungleCrashSite(void)
:LevelBaseData()
{
}

JungleCrashSite::JungleCrashSite(const LevelBaseData& aLevelBaseData)
:LevelBaseData(aLevelBaseData)
{
}

JungleCrashSite::~JungleCrashSite(void)
{
}

void JungleCrashSite::Initialize(HGE *aHGE)
{
	myHge = aHGE;

	StaticSprite tempSprite = StaticSprite(myHge, "img/Levels/scene_jungle_crash.dds");
	myMap.AddFloorSprite(tempSprite, Vector2<float>(0, 0));
	tempSprite = StaticSprite(myHge, "img/Levels/scene_jungle_crash_overlay.dds");
	myMap.AddOverlaySprite(tempSprite, Vector2<float>(0, 0));

	myRobot = Robot(myHge);
	myRobot.Initialize();

	InitializeConversations();

	myCollisionMap = CollisionMap(16, "img/Levels/collision_jungle_crash.bmp", myHge);

	myItems.push_back(Item( myHge, "img/items/sprite_stick.dds", "img/items/icon_twig01.dds", Vector2<float>(543.0f, 371.0f), "stick"));
	myItems.push_back(Item( myHge, "img/items/sprite_flint02.dds", "img/items/sprite_flint02.dds", Vector2<float>(302.0f, 829.0f), "flintstone2"));
	myItems.push_back(Item( myHge, "img/items/sprite_flint02.dds", "img/items/sprite_flint02.dds", Vector2<float>(185.0f, 571.0f), "flintstone1"));
	myItems.push_back(Item( myHge, "img/items/sprite_flint01.dds", "img/items/icon_flintstone1.dds", Vector2<float>(260.0f, 327.0f), "sharpFlintstone"));

	myThisLevelMusic = "Jungle";

	myRaft = Raft(myHge, "img/items/icon_raft.dds", Vector2<float>( 831, 468 ) );

	myCurtain = Curtain( myHge );

	myLogTree = LogTree(myHge, "img/items/icon_raft_tree.dds", Vector2<float>( 653, 170 ) );

	myChangingConversation = false;
	myHaveCompletedRaftConversation = false;

	myEndOfGame = false;
}

void JungleCrashSite::LevelSpecificHandleInput(InputWrapper &aInputWrapper, float aDelta, Inventory &aInventory)
{
	aDelta;
	if((aInputWrapper.KeyPressed(DIK_E)) || (aInputWrapper.KeyPressed(DIK_RETURN)))
	{
		if(myDoneConversation.at(GetConversationIndex("Activate Roderick")) == false && myRobot.GetAvatarNear( ) == true)
		{
			myRobot.MakeActive();
			EventSingelton::GetInstance()->myRobotIsActiveFlag = true;
		}
		myRaft.InputHandling(aInputWrapper);

		if(myLogTree.IsInterractable( ) == true && myRobot.GetActive( ) == true )
		{
			myActiveConversationID = "before axe with tree";
		}
		if( myChangingConversation == true && myLogTree.IsInterractable() )
		{
			myChangingConversation = false;
			myHaveCompletedRaftConversation = true;
			myActiveConversationID = "with axe with tree intermediation";

			myLogTree = LogTree();
			myLogTree.MakeNotInterractable( );
			Item tempItem(myHge, "img/items/sprite_logs.dds", "img/items/icon_logs.dds", "logs");
			aInventory.AddItem( tempItem );
		}
	}
}

void JungleCrashSite::LevelSpecificUpdate(Avatar &aAvatar, Camera &aCamera, float aDelta)
{
	Vector2<float> tempVector = aAvatar.GetWorldSpacePosition();
	myCurtain.FadeIn( );
	myRobot.ChaseAvatar( tempVector, aDelta);
	myRobot.Update(aDelta, aCamera.myPosition, tempVector );


	if( GetActivatedRaft( ) == true )
	{
		Vector2<float> tempRobotPosition = myRobot.GetWorldSpacePosition( );
		Vector2<float> tempAvatarWorldPosition = aAvatar.GetWorldSpacePosition();
		Vector2<float> tempVector = myRaft.Update( aCamera.myPosition, tempAvatarWorldPosition, tempRobotPosition );

		if( tempVector.myX != 0 && tempVector.myY != 0 )
		{
			for( unsigned i = 0; i < 300; i++ )
			{
				myCurtain.FadeOut( );
			}
			aAvatar.SetWorldPosition( tempVector );
			myRobot.SetWorldSpacePosition( tempRobotPosition );
			aCamera.CenterOnAvatar( aAvatar.GetWorldSpacePosition( ) );
		}
	}

	myLogTree.Update( aCamera.myPosition, tempVector );

}

void JungleCrashSite::LevelSpecificRender(RenderData &aRenderData, Camera &aCamera)
{
	aCamera;
	myRobot.Render(aRenderData);
	myRaft.Render( aRenderData );
	myCurtain.Render( aRenderData );
	myLogTree.Render( aRenderData );
}

void JungleCrashSite::InitializeConversations()
{
	myConversationList.reserve(101);
	myConversationList.push_back(LevelConversation(myHge, "NONE", false));
	//myConversationHotSpotList.push_back(ConversationHotSpot(Vector2<float>(0.0f, 0.0f), 0.0f, 0.0f, "NONE"));

	myConversationList.push_back(LevelConversation(myHge, "Spott flint stone", false));
	//myConversationHotSpotList.push_back(ConversationHotSpot(Vector2<float>(0.0f, 0.0f), 0.0f, 0.0f, "Spott flint stone"));

	myConversationList.push_back(LevelConversation(myHge, "Put flint stone in Roderick", false));
	//myConversationHotSpotList.push_back(ConversationHotSpot(Vector2<float>(0.0f, 0.0f), 0.0f, 0.0f, "Put flint stone in Roderick"));

	myConversationList.push_back(LevelConversation(myHge, "Activate Roderick", false));
	//myConversationHotSpotList.push_back(ConversationHotSpot(Vector2<float>(myRobot.GetWorldSpacePosition( ).myX -1000, myRobot.GetWorldSpacePosition( ).myY -50), 2000, 100, "Activate Roderick"));

	myConversationList.push_back(LevelConversation(myHge, "After Roderick activated", false));
	//myConversationHotSpotList.push_back(ConversationHotSpot(Vector2<float>(0.0f, 0.0f), 0.0f, 0.0f, "After Roderick activated"));

	myConversationList.push_back(LevelConversation(myHge, "You can't go there", false));

	myConversationList.push_back(LevelConversation(myHge, "That's a branch stupid", false));

	myConversationList.push_back(LevelConversation(myHge, "sharpFlintstone", false));

	myConversationList.push_back(LevelConversation(myHge, "before axe with tree", false));

	myConversationList.push_back(LevelConversation(myHge, "with axe with tree intermediation", false));

	myConversationList.push_back(LevelConversation(myHge, "after axe with tree", false));

	myConversationList.push_back(LevelConversation(myHge, "have raft", false));

	for(unsigned int index = 0; index < myConversationList.size(); index++)
	{
		myConversationList[index].Initialize();
		myConversationList[index].myActiveConversation = false;
		myDoneConversation.set(index);
	}

	// Ester found flintstone after Roderick is activate
	myConversationList[GetConversationIndex("Spott flint stone")].myDialoge.SetLeftPortrait("img/Conversation/RobotPortraitLeft.dds", "Roderick");
	myConversationList[GetConversationIndex("Spott flint stone")].AddDialogeText("Roderick: : Oh, you found some flint stones.;");
	myConversationList[GetConversationIndex("Spott flint stone")].AddDialogeText("Roderick: : Since we arrived in this age I have been unable to access wikipedia but my internal database suggests that this sedimentary rock was widely used to make tools and light fires before the invention of the zippo lighter.;");
	myConversationList[GetConversationIndex("Spott flint stone")].AddDialogeText("Roderick: : You should keep them, in case you need fire or... You know. Tools.;");
	myConversationList[GetConversationIndex("Spott flint stone")].AddDialogeText("Roderick: : I will carry them for you, put them into my belly box.;");

	//// Ester put flintstone in Roderick
	myConversationList[GetConversationIndex("Put flint stone in Roderick")].myDialoge.SetLeftPortrait("img/Conversation/RobotPortraitLeft.dds", "Roderick");
	myConversationList[GetConversationIndex("Put flint stone in Roderick")].AddDialogeText("Roderick: : If you find anything else, I have plenty of space and you can always open and close your inventory to get your items back by pressing I.;");
	
	myConversationList[GetConversationIndex("Activate Roderick")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("Activate Roderick")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("Activate Roderick")].AddDialogeText("Roderick: :  Bzzz...;");

	myConversationList[GetConversationIndex("After Roderick activated")].myDialoge.SetLeftPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("After Roderick activated")].AddDialogeText("Roderick: :  AI activated. Lead the way!;");

	myConversationList[GetConversationIndex("You can't go there")].myDialoge.SetLeftPortrait("img/Conversation/EmptyPortrait.dds", "");
	myConversationList[GetConversationIndex("You can't go there")].myDialoge.SetRightPortrait("img/Conversation/EmptyPortrait.dds", "");	
	myConversationList[GetConversationIndex("You can't go there")].AddDialogeText("You can't go there right now, maybe you should activate Roderick first with the E or RETURN buttons;");

	myConversationList[GetConversationIndex("That's a branch stupid")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("That's a branch stupid")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("That's a branch stupid")].AddDialogeText("Roderick: : Branch added to inventory. What a scientist you are, collecting twigs...;");
	myConversationList[GetConversationIndex("That's a branch stupid")].AddDialogeText("Esther: : Hey Roderick.;");
	myConversationList[GetConversationIndex("That's a branch stupid")].AddDialogeText("Roderick: : Yes stupid?;");
	myConversationList[GetConversationIndex("That's a branch stupid")].AddDialogeText("Esther: : I got a body that can move freely, and you don't.;");
	myConversationList[GetConversationIndex("That's a branch stupid")].AddDialogeText("Roderick: : ...;");

	myConversationList[GetConversationIndex("sharpFlintstone")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("sharpFlintstone")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("sharpFlintstone")].AddDialogeText("Roderick: : A sharp flintstone!;");
	myConversationList[GetConversationIndex("sharpFlintstone")].AddDialogeText("Roderick: : It's made by hammering an object against a flintstone along it's linings as it will chip off layer by layer and eventually leave a very sharp edge!;");
	myConversationList[GetConversationIndex("sharpFlintstone")].AddDialogeText("Roderick: : Here, I'll show you how to do it properly Est...;");
	myConversationList[GetConversationIndex("sharpFlintstone")].AddDialogeText("Esther: : RODERICK!;");
	myConversationList[GetConversationIndex("sharpFlintstone")].AddDialogeText("Esther: : Put the flintstone down! You know what happend last time.;");
	myConversationList[GetConversationIndex("sharpFlintstone")].AddDialogeText("Roderick: : Oh yea... Sorry Esther...;");
	myConversationList[GetConversationIndex("sharpFlintstone")].AddDialogeText("Esther: : Poor person...;");
	myConversationList[GetConversationIndex("sharpFlintstone")].AddDialogeText("Roderick: : Oh well! Let's keep going, onward to new scientific knowlage!;");

	myConversationList[GetConversationIndex("before axe with tree")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("before axe with tree")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("before axe with tree")].AddDialogeText("Roderick: : This is a very ideal tree for building an good raft. Now if you just had some kind of sharp tool to chop down trees with, just like the woodchuckers used before the invention of the Slicinator.;");
	myConversationList[GetConversationIndex("before axe with tree")].AddDialogeText("Esther: : You mean the Axe?;");
	myConversationList[GetConversationIndex("before axe with tree")].AddDialogeText("Roderick: : Correct. Now get to work human!;");
	myConversationList[GetConversationIndex("before axe with tree")].AddDialogeText("Esther: : Why did I build a robot with a highly advanced AI to help me when it can't even chop down a tree for me...;");
	myConversationList[GetConversationIndex("before axe with tree")].AddDialogeText("Roderick: : I have no idea.;");

	myConversationList[GetConversationIndex("with axe with tree intermediation")].myDialoge.SetLeftPortrait("img/empty.dds", " ");
	myConversationList[GetConversationIndex("with axe with tree intermediation")].myDialoge.SetRightPortrait("img/empty.dds", " ");
	myConversationList[GetConversationIndex("with axe with tree intermediation")].AddDialogeText("Chop. Chop. Chop-eti-chop.;");

	myConversationList[GetConversationIndex("after axe with tree")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("after axe with tree")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("after axe with tree")].AddDialogeText("Esther: : There! It's all chopped down and good to become a raft!;");
	myConversationList[GetConversationIndex("after axe with tree")].AddDialogeText("Roderick: : You must feel amazing, managing to chop down a tree with a tool essentially made for it.;");
	myConversationList[GetConversationIndex("after axe with tree")].AddDialogeText("Esther: : Oh be quiet you sourpuss.;");
	myConversationList[GetConversationIndex("after axe with tree")].AddDialogeText("Roderick: : I will not be silenced by my mere creator. By the way, what is, with your brilliantly advanced mind, the next step to make a proper raft now then?;");
	myConversationList[GetConversationIndex("after axe with tree")].AddDialogeText("Esther: : Go right ahead, just bite the hand that feeds you... I intend to bind the logs together somehow. You have a plentyful amount of wire I can use but I sense that it will not be enough to seal the deal. I need some good 'ol rope to fasten the logs securely.;");
	myConversationList[GetConversationIndex("after axe with tree")].AddDialogeText("Roderick: : Great idea! The next question comes up now, where are we going to get this good 'ol rope from?;");
	myConversationList[GetConversationIndex("after axe with tree")].AddDialogeText("Esther: : I... Don't know.;");
	myConversationList[GetConversationIndex("after axe with tree")].AddDialogeText("Roderick: : Good job.;");
	myConversationList[GetConversationIndex("after axe with tree")].AddDialogeText("Esther: : Bite me.;");

	myConversationList[GetConversationIndex("have raft")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("have raft")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Esther: : Look Roderick, I made a raft!;");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Roderick: : Woop-di-do, now you can float on water! On the other hand I already had a floating device for myself, so I could have done it from the beginning...;");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Esther: : ... Oh come on, stop being so rude! Now we both can get across that river next to the time machine .;");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Roderick: : We sure can. *coughuselesscough*;");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Esther: : What was that?;");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Roderick: : Oh, nothing, nothing at all.;");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Esther: : Right. I didn't even know you could make that kind of noise...;");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Roderick: : There's alot of things you don't know about me.;");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Esther: : I built you! I know ALL about you!;");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Roderick: : . . .;");
	myConversationList[GetConversationIndex("have raft")].AddDialogeText("Esther: : Never mind that now, let's put the raft over by the river so we can cross it.;");

	myActiveConversationID = "";
	myActiveConversationIndex = 0;
}

void JungleCrashSite::ConversationEvent(float elapsTime, Avatar &aAvatar, Inventory &aInventory)
{
	aAvatar;
	myConversationDelta += elapsTime;

	if(myRobot.GetActive() == false && myRobot.GetAvatarNear() == true)
	{
		myActiveConversationID = "Activate Roderick";
	}

	if(myRobot.GetActive() == true && myDoneConversation.at(GetConversationIndex("Activate Roderick")) == false && myDoneConversation.at(GetConversationIndex("After Roderick activated")) == true)
	{
		myActiveConversationID = "After Roderick activated";
	}

	if( aInventory.CheckForItem( "flintstone2" ) )
	{
		if( myDoneConversation.at(GetConversationIndex("Spott flint stone")) == true )
		{
			myActiveConversationID = "Spott flint stone";
		}
	}

	if( aInventory.CheckForItem( "flintstone1" ) )
	{
		if( myDoneConversation.at(GetConversationIndex("Put flint stone in Roderick")) == true )
		{
			myActiveConversationID = "Put flint stone in Roderick";
		}
	}

	if( aInventory.CheckForItem( "sharpFlintstone" ) )
	{
		if( myDoneConversation.at(GetConversationIndex("sharpFlintstone")) == true )
		{
			myActiveConversationID = "sharpFlintstone";
		}
	}

	if( aInventory.CheckForItem( "stick" ) )
	{
		if( myDoneConversation.at(GetConversationIndex("That's a branch stupid")) == true )
		{
			myActiveConversationID = "That's a branch stupid";
		}
	}

	if( aInventory.CheckForItem("logs") == false && aInventory.CheckForItem("axe") == true && myDoneConversation.at(GetConversationIndex("before axe with tree")) == false && myHaveCompletedRaftConversation == false )
	{
		myChangingConversation = true;
	}

	if( myDoneConversation.at(GetConversationIndex("with axe with tree intermediation") ) == false && myDoneConversation.at(GetConversationIndex("after axe with tree") ) == true )
	{
		myActiveConversationID = "after axe with tree";
	}

	if(myActiveConversationID != "")
	{
		myActiveConversationIndex = GetConversationIndex(myActiveConversationID);
	}

	if( myRaft.IsActive() == false
		&& EventSingelton::GetInstance()->myRaftCreated == true )
	{
		if (EventSingelton::GetInstance()->myInventoryActive == false
			&& EventSingelton::GetInstance()->myNotebookActive == false)
		{
			MakeRaftActive();
		}
	}

	if( EventSingelton::GetInstance()->myRaftCreated && myDoneConversation.at(GetConversationIndex("have raft")) == true )
	{
		if (EventSingelton::GetInstance()->myInventoryActive == false
			&& EventSingelton::GetInstance()->myNotebookActive == false)
		{
			myActiveConversationID = "have raft";
		}		
	}
}

bool JungleCrashSite::GetActivatedRaft( )
{
	return myRaft.GetAppear( );
}

void JungleCrashSite::MakeRaftActive( )
{
	myRaft.MakeAppear( );
}

void JungleCrashSite::MakeRaftNotActive( )
{
	myRaft.MakeNotAppear( );
}