#include "LevelBaseData.h"
#include "EventSingelton.h"
#include <assert.h>

LevelBaseData::LevelBaseData()
{
	myHge = NULL;
	myConversationDelta = 0;
	//myMap = aLevelBaseData.myMap;

	//myConversation = aLevelBaseData.myConversation;
	//myRobot = aLevelBaseData.myRobot;
	//myConversationList = aLevelBaseData.myConversation;
	//myDoneConversation = aLevelBaseData.myDoneConversation;
	//myActiveConversationID = aLevelBaseData.myActiveConversationID;
	//myActiveConversationIndex = aLevelBaseData.myActiveConversationIndex;
	//myItems = aLevelBaseData.myItems;
	//myHotSpots = aLevelBaseData.myHotSpots;
	//myCollisionMap = aLevelBaseData.myCollisionMap;
}

LevelBaseData::~LevelBaseData()
{
}

LevelBaseData::LevelBaseData(const LevelBaseData &aLevelBaseData)
{
	myHge = aLevelBaseData.myHge;
	myMap = aLevelBaseData.myMap;

	for ( unsigned int copiedConversation = 0; copiedConversation < aLevelBaseData.myConversationList.size(); copiedConversation++)
	{
		myConversationList.push_back(aLevelBaseData.myConversationList[copiedConversation]);
	}
	myDoneConversation = aLevelBaseData.myDoneConversation;
	myActiveConversationID = aLevelBaseData.myActiveConversationID;
	myActiveConversationIndex = aLevelBaseData.myActiveConversationIndex;
	myItems = aLevelBaseData.myItems;
	myHotSpots = aLevelBaseData.myHotSpots;
	myCollisionMap = aLevelBaseData.myCollisionMap;
	myConversationDelta = aLevelBaseData.myConversationDelta;
	myRobot = aLevelBaseData.myRobot;
	myThisLevelMusic = aLevelBaseData.myThisLevelMusic;
}

void LevelBaseData::Initialize(HGE *aHGE)
{
	myHge = aHGE;

	StaticSprite tempSprite = StaticSprite(myHge, "img/Levels/scene_jungle_crash.dds");
	myMap.AddFloorSprite(tempSprite, Vector2<float>(0, 0));
	tempSprite = StaticSprite(myHge, "img/Levels/scene_jungle_crash_overlay.dds");
	myMap.AddOverlaySprite(tempSprite, Vector2<float>(0, 0));

	myRobot = Robot(myHge);
	myRobot.Initialize();

	InitializeConversations();

	myCollisionMap = CollisionMap(16, "img/Levels/collision_jungle_crash.bmp", myHge);

	myChangedLevel = false;

	myEndOfGame = false;

	myChangedLevelIndex = 0;
}

void LevelBaseData::InitializeConversations()
{
	
}


int LevelBaseData::GetConversationIndex(std::string aID)
{
	int confirmedID = 0;
	for(unsigned int index = 0; index < myConversationList.size(); index++)
	{
		if(myConversationList[index].myConversationID == aID)
		{
			confirmedID = index;
		}
	}
	myActiveConversationIndex = confirmedID;
	return myActiveConversationIndex;
}

void LevelBaseData::StartAndDisableConversation(float elapsTime, Avatar &aAvatar, Inventory &aInventory)
{
	ConversationEvent(elapsTime, aAvatar, aInventory);
	GetConversationIndex(myActiveConversationID);

	if(myDoneConversation.at( myActiveConversationIndex ) == true && myActiveConversationIndex != 0)
	{
		aAvatar.StopAnimation();
		EventSingelton::GetInstance()->myConversationActive = true;
		myConversationList[ myActiveConversationIndex ].myActiveConversation = true;
	}

	if(myConversationList[ myActiveConversationIndex ].myDialoge.myActiveConversation == false)
	{
		aAvatar.StartAnimation();
		EventSingelton::GetInstance()->myConversationActive = false;
		myConversationList[ myActiveConversationIndex ].myActiveConversation = false;	// this disable all the conversation box sprite and font text
		myDoneConversation.reset( myActiveConversationIndex );							// this set about this conversation has finsihed (had talked)
		myActiveConversationID = "";
	}
}

void LevelBaseData::RenderMyItems(RenderData& aRenderData)
{
	for( unsigned itemRendered = 0; itemRendered < myItems.size( ); itemRendered++ )
	{
		myItems[itemRendered].RenderAtWorld( aRenderData, myRobot.GetActive( ) );
	}
}

bool LevelBaseData::UpdateHotSpots( Avatar &anAvatar, Camera &aCamera )
{
	for( unsigned i = 0; i < myHotSpots.size( ); i++ )
	{
		Vector2<float> tempVector( Vector2<float>(anAvatar.GetWorldSpacePosition( ).myX + anAvatar.GetAnimationRef().GetSpriteWidth() / 2.0f, anAvatar.GetWorldSpacePosition( ).myY + 100.0f  ));
		if( myHotSpots[i].IsAvatarInsideMe( tempVector ) == true )
		{
			anAvatar.SetWorldPosition( tempVector );
			aCamera.CenterOnAvatar( anAvatar.GetWorldSpacePosition( ) );
			if( myHotSpots[i].GetHotSpotID( ) == "JungleCrashTransfer" )
			{
				myChangedLevelIndex = 0;
				return true;
			}
			else if( myHotSpots[i].GetHotSpotID( ) == "DeepJungleTransfer" )
			{
				if( myRobot.GetActive( ) == true )
				{
					myChangedLevelIndex = 1;
					return true;
				}
				else
				{
					anAvatar.SetWorldPosition( Vector2<float>(myHotSpots[0].GetBoxSpot( ).myX + 100, myHotSpots[0].GetBoxSpot( ).myY -175) );
					aCamera.CenterOnAvatar( anAvatar.GetWorldSpacePosition( ) );
					myActiveConversationID = "You can't go there";
				}
			}
			else if( myHotSpots[i].GetHotSpotID( ) == "TimeMachineLevel" )
			{
				myChangedLevelIndex = 2;
				return true;
			}
			else if( myHotSpots[i].GetHotSpotID( ) == "JungleCrashTransferFromTimeMachine" )
			{
				myChangedLevelIndex = 3;
				return true;
			}
			else if( myHotSpots[i].GetHotSpotID( ) == "VillageTransferFromJungleCrash" )
			{
				if( myRobot.GetActive( ) == true )
				{
					myChangedLevelIndex = 6;
					return true;
				}
				else
				{
					anAvatar.SetWorldPosition( Vector2<float>(myHotSpots[2].GetBoxSpot( ).myX + 80, myHotSpots[2].GetBoxSpot( ).myY +50) );
					aCamera.CenterOnAvatar( anAvatar.GetWorldSpacePosition( ) );
					myActiveConversationID = "You can't go there";
				}
			}
			else if( myHotSpots[i].GetHotSpotID( ) == "JungleCrashTransferFromVillage" )
			{
				myChangedLevelIndex = 7;
				return true;
			}
		}
	}

	for( unsigned index = 0; index < myVerticalHotSpots.size( ); index++ )
	{
		Vector2<float> tempVector( Vector2<float>(anAvatar.GetWorldSpacePosition( ).myX + anAvatar.GetAnimationRef().GetSpriteWidth() / 2.0f, anAvatar.GetWorldSpacePosition( ).myY + 100.0f  ));
		if( myVerticalHotSpots[index].IsAvatarInsideMe( tempVector ) )
		{
			anAvatar.SetWorldPosition( tempVector );
			aCamera.CenterOnAvatar( anAvatar.GetWorldSpacePosition( ) );
			if( myVerticalHotSpots[index].GetVerticalHotSpotID( ) == "VolcanoTransferFromJungleCrash" )
			{
				myChangedLevelIndex = 100;
				return true;
			}
			else if( myVerticalHotSpots[index].GetVerticalHotSpotID( ) == "JungleCrashTransferFromVolcano" )
			{
				myChangedLevelIndex = 200;
				return true;
			}
		}
	}

	Vector2<float> tempVector = anAvatar.GetWorldSpacePosition();

	for( unsigned i = 0; i < myRandomHotSpots.size( ); i++ )
	{
		if( myRandomHotSpots[i].IsAvatarInsideMe( tempVector ) == true )
		{
			if( myRandomHotSpots[i].GetRandomHotSpotID( ) == "CaveFromDeepJungle" )
			{
				myChangedLevelIndex = 4;
				return true;
			}
			else if( myRandomHotSpots[i].GetRandomHotSpotID( ) == "DeepJungleFromCave" )
			{
				myChangedLevelIndex = 5;
				return true;
			}
			else if( myRandomHotSpots[i].GetRandomHotSpotID( ) == "DeepJungleFromBlownUpCave" )
			{
				myChangedLevelIndex = 5;
				return true;
			}
		}
	}

	return false;
}

void LevelBaseData::HandleInput(InputWrapper &aInputWrapper, float aDelta, Inventory &aInventory)
{
	aDelta;
	if(myConversationList[ myActiveConversationIndex ].myActiveConversation == true)
	{
		myConversationList[ myActiveConversationIndex ].Input(aInputWrapper);
	}
	
	if (EventSingelton::GetInstance()->myNotebookActive == false
		&& EventSingelton::GetInstance()->myConversationActive == false
		&& EventSingelton::GetInstance()->myInventoryActive == false
		&& EventSingelton::GetInstance()->myQuitMenuActive == false)
	{
		if((aInputWrapper.KeyPressed(DIK_E)) || (aInputWrapper.KeyPressed(DIK_RETURN)))
		{
			for( unsigned i = 0; i < myItems.size( ); i++ )
			{
				if( myRobot.GetActive( ) == true 
					&& myItems[i].IsInterractable( ) == true 
					&& myConversationList[ myActiveConversationIndex ].myActiveConversation == false)
				{
					aInventory.AddItem( myItems[i] );
					myItems.erase( myItems.begin( ) + i );
				}
			}
		}
	}
}

void LevelBaseData::Update(Avatar &aAvatar, Camera &aCamera, float aDelta, Inventory &aInventory)
{
	Vector2<float> tempVector = aAvatar.GetWorldSpacePosition();
	for( unsigned i = 0; i < myItems.size( ); i++ )
	{
		myItems[i].Update( aCamera.myPosition, tempVector );
	}

	if( UpdateHotSpots( aAvatar, aCamera ) == true )
	{
		myChangedLevel = true;
	}

	StartAndDisableConversation(aDelta, aAvatar, aInventory);
	if(myConversationList[ myActiveConversationIndex ].myActiveConversation == true)
	{
		myConversationList[ myActiveConversationIndex ].Update(aDelta);
	}
}

void LevelBaseData::RenderFloor(RenderData &aRenderData, Camera &aCamera)
{
	myMap.RenderFloor(aRenderData, aCamera.myPosition);
}

void LevelBaseData::RenderOverlay(RenderData &aRenderData, Camera &aCamera)
{
	myMap.RenderOverlay(aRenderData, aCamera.myPosition);
}

void LevelBaseData::RenderConversations(RenderData &aRenderData, Camera &aCamera)
{
	aCamera;
	if(myConversationList[ myActiveConversationIndex ].myActiveConversation == true)
	{
		myConversationList[ myActiveConversationIndex ].Render(aRenderData);
	}
}

bool LevelBaseData::GetItemIndex(std::string aItemID, int &aReturnItemID)
{
	int nbrOfItems = myItems.size();
	for (int itemChecked = 0; itemChecked < nbrOfItems; itemChecked++)
	{
		if ( myItems[itemChecked].myID.compare(aItemID) == 0 )
		{
			aReturnItemID = itemChecked;
			return true;
		}
	}
	return false;
}