#ifndef _LEVEL_BASE_H_
#define _LEVEL_BASE_H_

// static dictionary library
#include <vector>

// graphics
#include "HGE\hge.h"
#include "Avatar.h"
#include "Robot.h"
#include "Inventory.h"
#include "Villager.h"
#include "NativeVillager.h"
#include "NativeBoy.h"
#include "Raft.h"
#include "BirdCutscene.h"

// logic
#include "InputWrapper\InputWrapper.h"
#include "Timer\TimerManager.h"
#include "Map.h"
#include "Conversation.h"
#include "HotSpot.h"
#include "ConversationHotSpot.h"
#include "EventHotSpot.h"
#include "VerticalHotSpot.h"
#include "RandomHotSpot.h"
#include "Camera.h"
#include "Inventory.h"
#include "Avatar.h"
#include "Camera.h"

#include "EventSingelton.h"

///???
#include "LevelConversation.h"


class LevelBaseData
{

public:
	LevelBaseData();
	LevelBaseData(const LevelBaseData& aLevelBaseData);
	~LevelBaseData();

	//virtual void LoadGraphics();
	virtual void Initialize(HGE *aHGE);
	virtual void InitializeConversations();
	virtual void ConversationEvent(float elapsTime, Avatar &aAvatar, Inventory &aInventory) = 0;

	int GetConversationIndex(std::string aID);
	bool GetItemIndex(std::string aItemID, int &aReturnItemID);

	void StartAndDisableConversation(float elapsTime, Avatar &aAvatar, Inventory &aInventory);
	void Update(Avatar &aAvatar, Camera &aCamera, float aDelta, Inventory &aInventory);
	void RenderFloor(RenderData &aRenderData, Camera &aCamera);
	virtual void RenderOverlay(RenderData &aRenderData, Camera &aCamera);
	void RenderConversations(RenderData &aRenderData, Camera &aCamera);
	void HandleInput(InputWrapper &aInputWrapper, float aDelta, Inventory &aInventory);
	void RenderMyItems(RenderData& aRenderData);
	bool UpdateHotSpots( Avatar &anAvatar, Camera &aCamera );
	

	virtual void LevelSpecificUpdate(Avatar &aAvatar, Camera &aCamera, float aDelta) = 0;
	virtual void LevelSpecificRender(RenderData &aRenderData, Camera &aCamera) = 0;
	virtual void LevelSpecificHandleInput(InputWrapper &aInputWrapper, float aDelta, Inventory &aInventory) = 0;

	HGE* myHge;
	Map myMap;

	std::vector<LevelConversation> myConversationList;
	std::bitset<32> myDoneConversation;
	std::string myActiveConversationID;
	int myActiveConversationIndex;
	float myConversationDelta;
	bool myChangedLevel;
	bool myEndOfGame;
	int myChangedLevelIndex;

	std::vector<HotSpot> myHotSpots;
	std::vector<EventHotSpot> myEventHotSpots;
	std::vector<ConversationHotSpot> myConversationHotSpotList;
	std::vector<VerticalHotSpot> myVerticalHotSpots;
	std::vector<RandomHotSpot> myRandomHotSpots;

	Robot myRobot;
	Raft myRaft;
	std::vector<Item> myItems;
	CollisionMap myCollisionMap;

	std::string myThisLevelMusic;
	BirdCutscene myBirdCutscene;

};
#endif