#include "LogTree.h"
#include "EventSingelton.h"


LogTree::LogTree()
{
	myIsBeingCalled = false;
	myAvatarNear = false;
}

LogTree::LogTree( HGE* aHGE, std::string aFileName, Vector2<float> aWorldSpacePosition )
{
	mySprite = StaticSprite(aHGE, aFileName);
	myWorldSpacePosition = aWorldSpacePosition;

	myInterractionButton = StaticSprite( aHGE, "img/icon/icon_e.dds" );

	myIsBeingCalled = false;
	myAvatarNear = false;
}

LogTree::~LogTree()
{
}

void LogTree::InputHandling(InputWrapper& aInput)
{
	aInput;
// 	MakeActive( );
}

void LogTree::Render(RenderData &someRenderData)
{
	RenderMessage tempMessage;

	tempMessage.myPosition = myCameraSpacePosition;
	tempMessage.myType = spriteRenderMessageType;

	someRenderData.myMessages.push_back(tempMessage);
	someRenderData.mySprites.push_back(&mySprite);

	if(IsInterractable() == true && EventSingelton::GetInstance()->myRobotIsActiveFlag == true)
	{
		tempMessage.myPosition = Vector2<float>( myCameraSpacePosition.myX + 30, myCameraSpacePosition.myY - 60);
		tempMessage.myType = spriteRenderMessageType;

		someRenderData.myMessages.push_back( tempMessage );
		someRenderData.mySprites.push_back( &myInterractionButton );
	}
}

void LogTree::Update(Vector2<float> aCameraPosition, Vector2<float>& anAvatarPosition)
{
	myCameraSpacePosition = myWorldSpacePosition - aCameraPosition;
	IsAvatarInProximity(anAvatarPosition);
}

bool LogTree::IsAvatarInProximity(Vector2<float>& anAvatarPosition)
{
	if( (anAvatarPosition.myX < myWorldSpacePosition.myX + 130) && (anAvatarPosition.myX > myWorldSpacePosition.myX - 130) )
	{
		if( (anAvatarPosition.myY < myWorldSpacePosition.myY + 130) && (anAvatarPosition.myY > myWorldSpacePosition.myY - 130) )
		{
			MakeInterractable();
			return true;
		}
	}

	MakeNotInterractable();
	return false;
}

bool LogTree::IsInterractable()
{
	return (myAvatarNear);
}

void LogTree::MakeInterractable()
{
	myAvatarNear = true;
}

void LogTree::MakeNotInterractable()
{
	myAvatarNear = false;
}

void LogTree::SwitchState()
{
	if( IsActive() == false )
	{
		MakeActive();
	}
	else
	{
		MakeNotActive();
	}
}

void LogTree::MakeActive()
{
	myIsBeingCalled = true;
}

void LogTree::MakeNotActive()
{
	myIsBeingCalled = false;
}

bool LogTree::IsActive()
{
	return myIsBeingCalled;
}

