#ifndef _MATRIX_33_H_
#define _MATRIX_33_H_

template< class Type >
class Matrix33
{
public:
	enum
	{
		M300, M301, M302, 
		M310, M311, M312,
		M320, M321, M322
	};
	Matrix33();
	Matrix33(Matrix44<Type> aMatrix);
	~Matrix33();
	bool operator==( const Matrix33<Type>& aMatrix33 );

	Matrix33<Type> operator+( const Matrix33<Type>& aMatrix33 );
	Matrix33<Type> operator-( const Matrix33<Type>& aMatrix33 );
	Matrix33<Type> operator*( const Matrix33<Type>& aMatrix33 );

	Matrix33<Type>& operator=( const Matrix33<Type>& aMatrix33 );
	Matrix33<Type>& operator*=( const Matrix33<Type>& aMatrix33 );

	static Matrix33<Type> CreateRotateAroundX( Type aAngleInRadians );
	static Matrix33<Type> CreateRotateAroundY( Type aAngleInRadians );
	static Matrix33<Type> CreateRotateAroundZ( Type aAngleInRadians );
	static Matrix33<Type> Identity();

	Matrix33<Type> Inverse();
	
	Type myMatrix[9];
private:
	
};

template< class Type >
Matrix33<Type>::Matrix33()
{
	int placeIndex = 0;
	for(int matrixIndex = 0; matrixIndex < 9; matrixIndex ++)
	{
		if(matrixIndex == placeIndex)
		{
			myMatrix[matrixIndex] = 1;
			placeIndex += 4;
		}
		else
		{
			myMatrix[matrixIndex] = 0;
		}
	}
}
template< class Type >
Matrix33<Type>::Matrix33(Matrix44<Type> aMatrix)
{
	int increaseIndex = 0;
	int value = 3;

	for(int matrixIndex = 0; matrixIndex < 9; matrixIndex++)
	{
		if(matrixIndex == value)
		{
			increaseIndex++;
			value += 3;
		}
		myMatrix[matrixIndex] = aMatrix.myMatrix[matrixIndex + increaseIndex];
	}
}
template< class Type >
Matrix33<Type>::~Matrix33()
{}

template< class Type >
Matrix33<Type>& Matrix33<Type>::operator =(const Matrix33<Type> &aMatrix33)
{
	for(int matrixIndex = 0; matrixIndex < 9; matrixIndex++)
	{
		myMatrix[matrixIndex] = aMatrix33.myMatrix[matrixIndex];
	}
	return *this;
}
template< class Type >
bool Matrix33<Type>::operator ==(const Matrix33<Type> &aMatrix33)
{
	for(int matrixIndex = 0; matrixIndex < 9; matrixIndex++)
	{
		if(myMatrix[matrixIndex] != aMatrix33.myMatrix[matrixIndex])
		{
			return false;
		}
	}
	return true;	
}
template< class Type >
Matrix33<Type> Matrix33<Type>::operator +(const Matrix33<Type> &aMatrix33)
{
	Matrix33<Type> tempMatrix;
	for(int matrixIndex = 0; matrixIndex < 9; matrixIndex++)
	{
		tempMatrix.myMatrix[matrixIndex] = myMatrix[matrixIndex] + aMatrix33.myMatrix[matrixIndex];
	}
	return tempMatrix;
}
template< class Type >
Matrix33<Type> Matrix33<Type>::operator -(const Matrix33<Type> &aMatrix33)
{
	Matrix33<Type> tempMatrix;
	for(int matrixIndex = 0; matrixIndex < 9; matrixIndex++)
	{
		tempMatrix.myMatrix[matrixIndex] = myMatrix[matrixIndex] - aMatrix33.myMatrix[matrixIndex];
	}
	return tempMatrix;
}
template< class Type >
Matrix33<Type> Matrix33<Type>::operator *(const Matrix33<Type> &aMatrix33)
{
	Matrix33<Type> tempMatrix;
	int currentColumn = 0;
	int currentRowStart = 0;
	int currentRowEnd = 2; 	
	for(int matrixIndex = 0; matrixIndex < 9; matrixIndex++)
	{
		tempMatrix.myMatrix[matrixIndex] = 0;
		for(int column = 0; column < 3; column++)
		{
			tempMatrix.myMatrix[matrixIndex] += myMatrix[currentRowStart + column] * aMatrix33.myMatrix[currentColumn + column * 3];
		}
		currentColumn++;
		if(matrixIndex == currentRowEnd)
		{
			currentColumn = 0;
			currentRowEnd += 3;
			currentRowStart += 3;
		}
	}
	return tempMatrix;
}
template< class Type >
Matrix33<Type>& Matrix33<Type>::operator *=(const Matrix33<Type> &aMatrix33)
{
	Matrix33<Type> tempMatrix;
	int currentColumn = 0;
	int currentRowStart = 0;
	int currentRowEnd = 2; 	
	for(int matrixIndex = 0; matrixIndex < 9; matrixIndex++)
	{
		tempMatrix.myMatrix[matrixIndex] = 0;
		for(int column = 0; column < 3; column++)
		{
			tempMatrix.myMatrix[matrixIndex] += myMatrix[currentRowStart + column] * aMatrix33.myMatrix[currentColumn + column * 3];
		}
		currentColumn++;
		if(matrixIndex == currentRowEnd)
		{
			currentColumn = 0;
			currentRowEnd += 3;
			currentRowStart += 3;
		}
	}
	for(int index = 0; index < 9; index++)
	{
		myMatrix[index] = tempMatrix.myMatrix[index];
	}


	return *this;
}

template< class Type >
Matrix33<Type> Matrix33<Type>::CreateRotateAroundX(Type aAngleInRadians)
{
	Matrix33<Type> tempMatrix;

	tempMatrix.myMatrix[0] = 1;		tempMatrix.myMatrix[1] = 0;																	tempMatrix.myMatrix[2] = 0;
	tempMatrix.myMatrix[3] = 0;		tempMatrix.myMatrix[4] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[5] = static_cast<Type>(sin(static_cast<double>(aAngleInRadians)));
	tempMatrix.myMatrix[6] = 0;		tempMatrix.myMatrix[7] = static_cast<Type>(-sin(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[8] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));

	return tempMatrix;
}
template< class Type >
Matrix33<Type> Matrix33<Type>::CreateRotateAroundY(Type aAngleInRadians)
{
	Matrix33<Type> tempMatrix;

	tempMatrix.myMatrix[0] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[1] = 0;		tempMatrix.myMatrix[2] = static_cast<Type>(-sin(static_cast<double>(aAngleInRadians)));
	tempMatrix.myMatrix[3] = 0;																	tempMatrix.myMatrix[4] = 1;		tempMatrix.myMatrix[5] = 0;
	tempMatrix.myMatrix[6] = static_cast<Type>(sin(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[7] = 0;		tempMatrix.myMatrix[8] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		

	return tempMatrix;
}
template< class Type >
Matrix33<Type> Matrix33<Type>::CreateRotateAroundZ(Type aAngleInRadians)
{
	Matrix33<Type> tempMatrix;
	tempMatrix.myMatrix[0] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[1] = static_cast<Type>(sin(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[2] = 0;
	tempMatrix.myMatrix[3] = static_cast<Type>(-sin(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[4] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[5] = 0;
	tempMatrix.myMatrix[6] = 0;																	tempMatrix.myMatrix[7] = 0;																	tempMatrix.myMatrix[8] = 1;

	return tempMatrix;
}
template< class Type >
Matrix33<Type> Matrix33<Type>::Identity()
{
	Matrix33<Type> tempMatrix;
	int placeIndex = 0;
	for(int matrixIndex = 0; matrixIndex < 9; matrixIndex ++)
	{
		if(matrixIndex == placeIndex)
		{
			tempMatrix.myMatrix[matrixIndex] = 1;
			placeIndex += 4;
		}
		else
		{
			tempMatrix.myMatrix[matrixIndex] = 0;
		}
	}
	return tempMatrix;
}

template< class Type >
Matrix33<Type> Matrix33<Type>::Inverse()
{ 
	// find det(mymatrix)
	Type det = 
		myMatrix[M300] * ( myMatrix[M311] * myMatrix[M322] - myMatrix[M312] * myMatrix[M321] ) -
		myMatrix[M301] * ( myMatrix[M310] * myMatrix[M322] - myMatrix[M312] * myMatrix[M320] ) +
		myMatrix[M302] * ( myMatrix[M310] * myMatrix[M321] - myMatrix[M322] * myMatrix[M320] );

	// calculate confector matrix
	Matrix33<Type> myConfectorMatrix;
	
	myConfectorMatrix.myMatrix[M300] = (( myMatrix[M311] * myMatrix[M322] ) - ( myMatrix[M312] * myMatrix[M321] ));
	myConfectorMatrix.myMatrix[M301] = -(( myMatrix[M310] * myMatrix[M322] ) - ( myMatrix[M312] * myMatrix[M320] ));
	myConfectorMatrix.myMatrix[M302] = (( myMatrix[M310] * myMatrix[M321] ) - ( myMatrix[M311] * myMatrix[M320] ));

	myConfectorMatrix.myMatrix[M310] = -(( myMatrix[M301] * myMatrix[M322] ) - ( myMatrix[M302] * myMatrix[M321] ));
	myConfectorMatrix.myMatrix[M311] = (( myMatrix[M300] * myMatrix[M322] ) - ( myMatrix[M302] * myMatrix[M320] ));
	myConfectorMatrix.myMatrix[M312] = -(( myMatrix[M300] * myMatrix[M321] ) - ( myMatrix[M301] * myMatrix[M320] ));

	myConfectorMatrix.myMatrix[M320] = (( myMatrix[M301] * myMatrix[M312] ) - ( myMatrix[M302] * myMatrix[M311] ));
	myConfectorMatrix.myMatrix[M321] = -(( myMatrix[M300] * myMatrix[M312] ) - ( myMatrix[M302] * myMatrix[M310] ));
	myConfectorMatrix.myMatrix[M322] = (( myMatrix[M300] * myMatrix[M311] ) - ( myMatrix[M301] * myMatrix[M310] ));
	
	// make a transpose matrix of this
	Matrix33<Type> myTransposeMatrix;

	myTransposeMatrix.myMatrix[M300] = myConfectorMatrix.myMatrix[M300];
	myTransposeMatrix.myMatrix[M301] = myConfectorMatrix.myMatrix[M310];
	myTransposeMatrix.myMatrix[M302] = myConfectorMatrix.myMatrix[M320];

	myTransposeMatrix.myMatrix[M310] = myConfectorMatrix.myMatrix[M301];
	myTransposeMatrix.myMatrix[M311] = myConfectorMatrix.myMatrix[M311];
	myTransposeMatrix.myMatrix[M312] = myConfectorMatrix.myMatrix[M321];

	myTransposeMatrix.myMatrix[M320] = myConfectorMatrix.myMatrix[M302];
	myTransposeMatrix.myMatrix[M321] = myConfectorMatrix.myMatrix[M312];
	myTransposeMatrix.myMatrix[M322] = myConfectorMatrix.myMatrix[M322];

	Matrix33<Type> myInverseMatrix;

	for(int index = 0; index < 9; index++)
	{
		myInverseMatrix.myMatrix[index] = myTransposeMatrix.myMatrix[index] / det;
	}

	return myInverseMatrix;
}
#endif
