#ifndef _MATRIX_44_H_
#define _MATRIX_44_H_
template< class Type >
class Matrix44
{
public:
	enum
	{
		M411, M412, M413, M414,
		M421, M422, M423, M424,
		M431, M432, M433, M434,
		M441, M442, M443, M444
	};
	Matrix44();
	~Matrix44();
	bool operator==( const Matrix44<Type>& aMatrix44 );

	Matrix44<Type> operator=( const Matrix44<Type>& aMatrix44 );
	Matrix44<Type> operator+( const Matrix44<Type>& aMatrix44 );
	Matrix44<Type> operator-( const Matrix44<Type>& aMatrix44 );
	Matrix44<Type> operator*( const Matrix44<Type>& aMatrix44 );

	Matrix44<Type>& operator*=( const Matrix44<Type>& aMatrix44 );

	static Matrix44<Type> CreateRotateAroundX( Type aAngleInRadians );
	static Matrix44<Type> CreateRotateAroundY( Type aAngleInRadians );
	static Matrix44<Type> CreateRotateAroundZ( Type aAngleInRadians );
	static Matrix44<Type> Identity();

	Type myMatrix[16];
private:
	
};
template< class Type >
Matrix44<Type>::Matrix44()
{
	int placeIndex = 0;
	for(int matrixIndex = 0; matrixIndex < 16; matrixIndex ++)
	{
		if(matrixIndex == placeIndex)
		{
			myMatrix[matrixIndex] = 1;
			placeIndex += 5;
		}
		else
		{
			myMatrix[matrixIndex] = 0;
		}
	}	
}
template< class Type >
Matrix44<Type>::~Matrix44()
{}

template< class Type >
Matrix44<Type> Matrix44<Type>::operator =(const Matrix44<Type> &aMatrix44)
{
	for(int matrixIndex = 0; matrixIndex < 16; matrixIndex++)
	{
		myMatrix[matrixIndex] = aMatrix44.myMatrix[matrixIndex];
	}
	return *this;
}
template< class Type >
bool Matrix44<Type>::operator ==(const Matrix44<Type> &aMatrix44)
{
	for(int matrixIndex = 0; matrixIndex < 16; matrixIndex++)
	{
		if(myMatrix[matrixIndex] == aMatrix44.myMatrix[matrixIndex])
		{
			return true;
		}
	}
	return false;	
}
template< class Type >
Matrix44<Type> Matrix44<Type>::operator +(const Matrix44<Type> &aMatrix44)
{
	Matrix44<Type> tempMatrix;
	for(int matrixIndex = 0; matrixIndex < 16; matrixIndex++)
	{
		tempMatrix.myMatrix[matrixIndex] = myMatrix[matrixIndex] + aMatrix44.myMatrix[matrixIndex];
	}
	return tempMatrix;
}
template< class Type >
Matrix44<Type> Matrix44<Type>::operator -(const Matrix44<Type> &aMatrix44)
{
	Matrix44<Type> tempMatrix;
	for(int matrixIndex = 0; matrixIndex < 16; matrixIndex++)
	{
		tempMatrix.myMatrix[matrixIndex] = myMatrix[matrixIndex] - aMatrix44.myMatrix[matrixIndex];
	}
	return tempMatrix;
}
template< class Type >
Matrix44<Type> Matrix44<Type>::operator *(const Matrix44<Type> &aMatrix44)
{
	Matrix44<Type> tempMatrix;
	int currentColumn = 0;
	int currentRowStart = 0;
	int currentRowEnd = 3; 
	for(int matrixIndex = 0; matrixIndex < 16; matrixIndex++)
	{
		tempMatrix.myMatrix[matrixIndex] = 0;
		for(int column = 0; column < 4; column++)
		{
			tempMatrix.myMatrix[matrixIndex] += myMatrix[currentRowStart + column] * aMatrix44.myMatrix[currentColumn + column * 3];
		}
		currentColumn++;
		if(matrixIndex == currentRowEnd)
		{
			currentColumn = 0;
			currentRowEnd += 4;
			currentRowStart += 4;
		}
	}
	return tempMatrix;
}
template< class Type >
Matrix44<Type>& Matrix44<Type>::operator *=(const Matrix44<Type> &aMatrix44)
{
	Matrix44<Type> tempMatrix;

	int currentColumn = 0;
	int currentRowStart = 0;
	int currentRowEnd = 3; 
	for(int matrixIndex = 0; matrixIndex < 16; matrixIndex++)
	{
		tempMatrix.myMatrix[matrixIndex] = 0;
		for(int column = 0; column < 4; column++)
		{
			tempMatrix.myMatrix[matrixIndex] += myMatrix[currentRowStart + column] * aMatrix44.myMatrix[currentColumn + column * 3];
		}
		currentColumn++;
		if(matrixIndex == currentRowEnd)
		{
			currentColumn = 0;
			currentRowEnd += 4;
			currentRowStart += 4;
		}
	}

	for(int index = 0; index < 9; index++)
	{
		myMatrix[index] = tempMatrix.myMatrix[index];
	}

	return *this;
}

template< class Type >
Matrix44<Type> Matrix44<Type>::CreateRotateAroundX(Type aAngleInRadians)
{
	Matrix44<Type> tempMatrix;

	tempMatrix.myMatrix[0] = 1;		tempMatrix.myMatrix[1] = 0;																	tempMatrix.myMatrix[2] = 0;																	tempMatrix.myMatrix[3] = 0;
	tempMatrix.myMatrix[4] = 0;		tempMatrix.myMatrix[5] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[6] = static_cast<Type>(sin(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[7] = 0;
	tempMatrix.myMatrix[8] = 0;		tempMatrix.myMatrix[9] = static_cast<Type>(-sin(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[10] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[11] = 0;
	tempMatrix.myMatrix[12] = 0;	tempMatrix.myMatrix[13] = 0;																tempMatrix.myMatrix[14] = 0;																tempMatrix.myMatrix[15] = 1;
	
	return tempMatrix;
}
template< class Type >
Matrix44<Type> Matrix44<Type>::CreateRotateAroundY(Type aAngleInRadians)
{
	Matrix44<Type> tempMatrix;

	tempMatrix.myMatrix[0] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[1] = 0;		tempMatrix.myMatrix[2] = static_cast<Type>(-sin(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[3] = 0;
	tempMatrix.myMatrix[4] = 0;																	tempMatrix.myMatrix[6] = 1;		tempMatrix.myMatrix[6] = 0;																	tempMatrix.myMatrix[7] = 0;
	tempMatrix.myMatrix[8] = static_cast<Type>(sin(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[9] = 0;		tempMatrix.myMatrix[10] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[11] = 0;
	tempMatrix.myMatrix[12] = 0;																tempMatrix.myMatrix[13] = 0;	tempMatrix.myMatrix[14] = 0;																tempMatrix.myMatrix[15] = 1;
	
	return tempMatrix;
}
template< class Type >
Matrix44<Type> Matrix44<Type>::CreateRotateAroundZ(Type aAngleInRadians)
{
	Matrix44<Type> tempMatrix;

	tempMatrix.myMatrix[0] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[1] = static_cast<Type>(sin(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[2] = 0;		tempMatrix.myMatrix[3] = 0;		
	tempMatrix.myMatrix[4] = static_cast<Type>(-sin(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[5] = static_cast<Type>(cos(static_cast<double>(aAngleInRadians)));		tempMatrix.myMatrix[6] = 0;		tempMatrix.myMatrix[7] = 0;
	tempMatrix.myMatrix[8] = 0;																	tempMatrix.myMatrix[9] = 0;																	tempMatrix.myMatrix[10] = 1;	tempMatrix.myMatrix[11] = 0;
	tempMatrix.myMatrix[12] = 0;																tempMatrix.myMatrix[13] = 0;																tempMatrix.myMatrix[14] = 0;	tempMatrix.myMatrix[15] = 1;
	
	return tempMatrix;
}

template< class Type >
Matrix44<Type> Matrix44<Type>::Identity()
{
	Matrix44<Type> tempMatrix;
	int placeIndex = 0;
	for(int matrixIndex = 0; matrixIndex < 16; matrixIndex ++)
	{
		if(matrixIndex == placeIndex)
		{
			tempMatrix.myMatrix[matrixIndex] = 1;
			placeIndex += 5;
		}
		else
		{
			tempMatrix.myMatrix[matrixIndex] = 0;
		}
	}
	return tempMatrix;
}
#endif