#include "Menu.h"


Menu::Menu()
{
}

Menu::Menu(HGE* aHge)
:myHge(aHge)
{
	myItemList.Init(5,5);
	myHighlightedItem = 0;

	myMenuFont = "img/Font/HGE_font1.fnt";
	myMenuTextAlign = 0;
	myPosition = Vector2<float>(0.0f, 0.0f);
	myHighlightedColour = Vector4<int>(255, 255, 255, 0);
	myNormalColour =  Vector4<int>(255, 0, 206, 209);
	theInputElapseTime = 0;
}

Menu::Menu(const Menu &aMenuComponent)
:myHge(aMenuComponent.myHge)
{
	myItemList = aMenuComponent.myItemList;
	myHighlightedItem = aMenuComponent.myHighlightedItem;
	
	myMenuFont = aMenuComponent.myMenuFont;
	myMenuTextAlign = aMenuComponent.myMenuTextAlign;
	myPosition = aMenuComponent.myPosition;
	myHighlightedColour = aMenuComponent.myHighlightedColour;
	myNormalColour = aMenuComponent.myNormalColour;
	theInputElapseTime = aMenuComponent.theInputElapseTime;
}

Menu::~Menu()
{
}

void Menu::AddMenuItem(std::string aItemText)
{
	Font aItem(myHge);
	aItem.SetText(aItemText);
	myItemList.Add(aItem);
}

void Menu::SetPosition(const float aPositionX, const float aPositionY)
{
	myPosition.x = aPositionX;
	myPosition.y = aPositionY;
}

void Menu::SetPosition(const Vector2<float> aPosition)
{
	myPosition = aPosition;
}

Vector2<float> Menu::GetPosition()
{
	return myPosition;
}

void Menu::SetHighlightedColour(const int aAlpha, const int aRed, const int aGreen, const int aBlue)
{
	myHighlightedColour.a = aAlpha;
	myHighlightedColour.r = aRed;
	myHighlightedColour.g = aGreen;
	myHighlightedColour.b = aBlue;
}

void Menu::SetHighlightedColour(const Vector4<int> aHighlightedColour)
{
	myHighlightedColour = aHighlightedColour;
}

void Menu::SetNormalColour(const int aAlpha, const int aRed, const int aGreen, const int aBlue)
{
	myNormalColour.a = aAlpha;
	myNormalColour.r = aRed;
	myNormalColour.g = aGreen;
	myNormalColour.b = aBlue;
}

void Menu::SetNormalColour(const Vector4<int> aNormalColour)
{
	myNormalColour = aNormalColour;
}

void Menu::SetMenuFont(std::string aFontName)
{
	myMenuFont = aFontName;
}

void Menu::SetMenuTextAlign(int aAlign)
{
	myMenuTextAlign = aAlign;
}

int Menu::GetHighlightedItem()
{
	return myHighlightedItem;
}

void Menu::Initialize()
{
	for(signed int itemIndex = 0; itemIndex < myItemList.Count(); itemIndex++)
	{
		if(itemIndex == 0)
		{
			myItemList[itemIndex].SetPosition(myPosition);
		}
		else
		{
			myItemList[itemIndex].SetPosition(myItemList[itemIndex - 1].GetPosition().x, myItemList[itemIndex - 1].GetPosition().y + myItemList[itemIndex - 1].GetFontHeight() + 5);
		}
		myItemList[itemIndex].SetSpriteFont(myMenuFont);
		myItemList[itemIndex].SetTextAlign(myMenuTextAlign);
		myItemList[itemIndex].Initialize();
	}
}

void Menu::MenuMarkingDown()
{
	myHighlightedItem++;
	if(myHighlightedItem >= myItemList.Count())
	{
		myHighlightedItem = 0;
	}
}
void Menu::MenuMarkingUp()
{
	myHighlightedItem--;
	if(myHighlightedItem < 0)
	{
		myHighlightedItem = myItemList.Count() - 1;
	}
}

void Menu::Input(InputWrapper& aInput)
{
	if( aInput.KeyPressed( DIK_UP ) )
	{
		myHighlightedItem--;
	}
	if( aInput.KeyPressed( DIK_DOWN ) )
	{
		myHighlightedItem++;
	}
	
	if(myHighlightedItem < 0)
	{
		myHighlightedItem = myItemList.Count() - 1;
	}
	else if(myHighlightedItem >= myItemList.Count())
	{
		myHighlightedItem = 0;
	}
}

void Menu::Update(float aElapseTime)
{
	aElapseTime = aElapseTime;
	for(signed int itemIndex = 0; itemIndex < myItemList.Count(); itemIndex++)
	{
		if(itemIndex == myHighlightedItem)
		{
			myItemList[itemIndex].SetColor(myHighlightedColour);
		}
		else
		{
			myItemList[itemIndex].SetColor(myNormalColour);
		}
	}
}

void Menu::Render()
{
	for(unsigned int itemIndex = 0; itemIndex < static_cast<unsigned int>(myItemList.Count()); itemIndex++)
	{
		myItemList[itemIndex].Render();
	}
}

Font* Menu::GetFontRef(int index)
{
	return &myItemList[index];
}
Tools::GrowingArray<Font, int> Menu::GetItemList()
{
	return myItemList;
}