#include "MenuComponent.h"


MenuComponent::MenuComponent()
{
}

MenuComponent::MenuComponent(HGE* aHge)
:myHge(aHge)
{
	myItemList.Init(5,5);
	myHighlightedItem = 0;

	myMenuFont = "img/Font/candl.fnt";
	myMenuTextAlign = 0;
	myPosition = Vector2<float>(0.0f, 0.0f);
	myHighlightedColour = Vector4<int>(255, 255, 255, 255);
	myNormalColour =  Vector4<int>(255, 0, 191, 255);
	theInputElapseTime = 0;

	myFirstInputElapsedTime = 0;
	mySecondInputElapsedTime = 0;
}

MenuComponent::MenuComponent(const MenuComponent &aMenuComponent)
:myHge(aMenuComponent.myHge)
{
	myItemList = aMenuComponent.myItemList;
	myHighlightedItem = aMenuComponent.myHighlightedItem;
	
	myMenuFont = aMenuComponent.myMenuFont;
	myMenuTextAlign = aMenuComponent.myMenuTextAlign;
	myPosition = aMenuComponent.myPosition;
	myHighlightedColour = aMenuComponent.myHighlightedColour;
	myNormalColour = aMenuComponent.myNormalColour;
	theInputElapseTime = aMenuComponent.theInputElapseTime;

	myFirstInputElapsedTime = aMenuComponent.myFirstInputElapsedTime;
	mySecondInputElapsedTime = aMenuComponent.mySecondInputElapsedTime;
}

MenuComponent::~MenuComponent()
{
}

void MenuComponent::AddMenuItem(std::string aItemText)
{
	Font aItem(myHge);
	aItem.SetText(aItemText);
	myItemList.Add(aItem);
}

void MenuComponent::SetPosition(const float aPositionX, const float aPositionY)
{
	myPosition.x = aPositionX;
	myPosition.y = aPositionY;
}

void MenuComponent::SetPosition(const Vector2<float> aPosition)
{
	myPosition = aPosition;
}

Vector2<float> MenuComponent::GetPosition()
{
	return myPosition;
}

void MenuComponent::SetHighlightedColour(const int aAlpha, const int aRed, const int aGreen, const int aBlue)
{
	myHighlightedColour.a = aAlpha;
	myHighlightedColour.r = aRed;
	myHighlightedColour.g = aGreen;
	myHighlightedColour.b = aBlue;
}

void MenuComponent::SetHighlightedColour(const Vector4<int> aHighlightedColour)
{
	myHighlightedColour = aHighlightedColour;
}

void MenuComponent::SetNormalColour(const int aAlpha, const int aRed, const int aGreen, const int aBlue)
{
	myNormalColour.a = aAlpha;
	myNormalColour.r = aRed;
	myNormalColour.g = aGreen;
	myNormalColour.b = aBlue;
}

void MenuComponent::SetNormalColour(const Vector4<int> aNormalColour)
{
	myNormalColour = aNormalColour;
}

void MenuComponent::SetMenuFont(std::string aFontName)
{
	myMenuFont = aFontName;
}

void MenuComponent::SetMenuTextAlign(int aAlign)
{
	myMenuTextAlign = aAlign;
}

int MenuComponent::GetHighlightedItem()
{
	return myHighlightedItem;
}

void MenuComponent::Initialize()
{
	for(signed int itemIndex = 0; itemIndex < myItemList.Count(); itemIndex++)
	{
		if(itemIndex == 0)
		{
			myItemList[itemIndex].SetPosition(myPosition);
		}
		else
		{
			myItemList[itemIndex].SetPosition(myItemList[itemIndex - 1].GetPosition().x, myItemList[itemIndex - 1].GetPosition().y + myItemList[itemIndex - 1].GetFontHeight() + 5);
		}
		myItemList[itemIndex].SetSpriteFont(myMenuFont);
		myItemList[itemIndex].SetTextAlign(myMenuTextAlign);
		myItemList[itemIndex].Initialize();
	}
}

void MenuComponent::MenuMarkingDown()
{
	myHighlightedItem++;
	if(myHighlightedItem >= myItemList.Count())
	{
		myHighlightedItem = 0;
	}
}
void MenuComponent::MenuMarkingUp()
{
	myHighlightedItem--;
	if(myHighlightedItem < 0)
	{
		myHighlightedItem = myItemList.Count() - 1;
	}
}

void MenuComponent::Input(InputWrapper& aInput, float aInputElapsdTime)
{
	if( aInput.KeyDown( DIK_UP ) || aInput.KeyDown( DIK_W ) )
	{
		if( myFirstInputElapsedTime > 20.0f )
		{
			myHighlightedItem--;
			if(myHighlightedItem < 0)
			{
				myHighlightedItem = myItemList.Count() - 1;
			}
			myFirstInputElapsedTime = 0;
		}
	}
	else
	{
		myFirstInputElapsedTime += aInputElapsdTime;
	}

	if( aInput.KeyDown( DIK_DOWN ) || aInput.KeyDown( DIK_S ) )
	{
		if( mySecondInputElapsedTime > 20.0f )
		{
			myHighlightedItem++;
			if(myHighlightedItem >= myItemList.Count())
			{
				myHighlightedItem = 0;
			}
			mySecondInputElapsedTime = 0;
		}
	}
	else
	{
		mySecondInputElapsedTime += aInputElapsdTime;
	}

}

void MenuComponent::Update(float aElapsedTime)
{
	aElapsedTime = aElapsedTime;
	for(signed int itemIndex = 0; itemIndex < myItemList.Count(); itemIndex++)
	{
		if(itemIndex == myHighlightedItem)
		{
			myItemList[itemIndex].SetColor(myHighlightedColour);
		}
		else
		{
			myItemList[itemIndex].SetColor(myNormalColour);
		}
	}
}

void MenuComponent::Render()
{
	for(unsigned int itemIndex = 0; itemIndex < static_cast<unsigned int>(myItemList.Count()); itemIndex++)
	{
		myItemList[itemIndex].Render();
	}
}

Font* MenuComponent::GetFontRef(int index)
{
	return &myItemList[index];
}
Tools::GrowingArray<Font, int> MenuComponent::GetItemList()
{
	return myItemList;
}