#include "Notebook.h"
#include "Inventory.h"
#include "EventSingelton.h"


Notebook::Notebook()
{
}

Notebook::Notebook(HGE* aHGE)
{
	mySprite = StaticSprite(aHGE, "img/Inventory-Notepad/notepad.dds");
	myMachineAvailableSprite.push_back( StaticSprite(aHGE, "img/Inventory-Notepad/notepad_highlight_Belt.dds") );
	myMachineAvailableSprite.push_back( StaticSprite(aHGE, "img/Inventory-Notepad/notepad_highlight_Crystal.dds") );
	myMachineAvailableSprite.push_back( StaticSprite(aHGE, "img/Inventory-Notepad/notepad_highlight_Magnet.dds") );
	myMachineAvailableSprite.push_back( StaticSprite(aHGE, "img/Inventory-Notepad/notepad_highlight_Wire.dds") );


	myPosition.myX = 285;
	myPosition.myY = 80;
	mySprite.SetPosition(myPosition);

	for(int index = 0; index < 4; index++)
	{
		myMachineAvailable[index] = 0;
		myMachineAvailableSprite[index].SetPosition(myPosition); 
	}	

	myIsBeingCalled = false;
}

Notebook::~Notebook()
{
}

void Notebook::InputHandling(InputWrapper& aInput)
{
	if(aInput.KeyPressed(DIK_N))
	{
		SwitchState();
	}
}

void Notebook::SwitchState()
{
	if( IsActive() == false )
	{
		MakeActive();
		EventSingelton::GetInstance()->myNotebookActive = true;
		EventSingelton::GetInstance()->myAvatarShouldAnimate = false;
	}
	else
	{
		MakeNotActive();
		EventSingelton::GetInstance()->myNotebookActive = false;
		EventSingelton::GetInstance()->myAvatarShouldAnimate = true;
	}
}

void Notebook::MakeActive()
{
	myIsBeingCalled = true;
}

void Notebook::MakeNotActive()
{
	myIsBeingCalled = false;
}

bool Notebook::IsActive()
{
	if(myIsBeingCalled == true)
	{
		return true;
	}

	return false;
}

void Notebook::ObtainItemUpdate(Inventory &aInventory)
{
	if(aInventory.CheckForItem("Diamond") == true)
	{
		myMachineAvailable[ITEM_CRYSTAL] = 1;
	}
	if(aInventory.CheckForItem("gold necklace") == true)
	{
		myMachineAvailable[ITEM_WIRE] = 1;
	}
	if(aInventory.CheckForItem("MagneticRock") == true)
	{
		myMachineAvailable[ITEM_MAGNET] = 1;
	}
	if(aInventory.CheckForItem("combined rubber") == true)
	{
		myMachineAvailable[ITEM_BELT] = 1;
	}
}

void Notebook::Render(RenderData &someRenderData)
{
	if(IsActive() == true)
	{
		RenderMessage tempMessage;

		tempMessage.myPosition = myPosition;
		tempMessage.myType = spriteRenderMessageType;

		someRenderData.myMessages.push_back(tempMessage);
		someRenderData.mySprites.push_back(&mySprite);

		for(unsigned int index = 0; index < myMachineAvailableSprite.size(); index++)
		{
			if(myMachineAvailable[index] == 1)
			{
				RenderMessage tempMachineMessage;
				tempMachineMessage.myPosition = myMachineAvailableSprite[index].GetPosition();
				tempMachineMessage.myType = spriteRenderMessageType;

				someRenderData.myMessages.push_back(tempMachineMessage);
				someRenderData.mySprites.push_back(&myMachineAvailableSprite[index]);
			}
		}
	}
}

