#include "QuitMenu.h"
#include "EventSingelton.h"

QuitMenu::QuitMenu(HGE* aHge)
{
	myBackground = AnimationBase(aHge);
	myContinueButton = AnimationBase(aHge);
	myExitButton = AnimationBase(aHge);
	myPosition = Vector2<float>(400,150);
	myContinueButtonOffset = Vector2<float>(26,25);
	myExitButtonOffset = Vector2<float>(26,90);
}

QuitMenu::QuitMenu()
{
	myBackground = AnimationBase();
	myPosition = Vector2<float>(400,300);
}

QuitMenu::~QuitMenu(void)
{
}


void QuitMenu::Render(RenderData &someRenderData)
{
	if (EventSingelton::GetInstance()->myQuitMenuActive == true)
	{
		RenderMessage tempMessage;
		tempMessage.myPosition = myPosition;
		tempMessage.myType = animationRenderMessageType;
		someRenderData.myMessages.push_back( tempMessage );
		someRenderData.myAnimations.push_back( &myBackground );

		RenderMessage tempMessage2;
		tempMessage2.myPosition = myPosition + myExitButtonOffset;
		tempMessage2.myType = animationRenderMessageType;
		someRenderData.myMessages.push_back( tempMessage2 );
		someRenderData.myAnimations.push_back( &myExitButton );

		RenderMessage tempMessage3;
		tempMessage3.myPosition = myPosition + myContinueButtonOffset;
		tempMessage3.myType = animationRenderMessageType;
		someRenderData.myMessages.push_back( tempMessage3 );
		someRenderData.myAnimations.push_back( &myContinueButton );
	}	
}

void QuitMenu::Initialize()
{
	myBackground.AddSpriteSheet("img/QuitMenu/Menu background.dds", 1, 0.01f);
	myBackground.Initialize();
	myBackground.SetActiveSheet(0);
	myBackground.SetPosition(myPosition);

	myExitButton.AddSpriteSheet("img/QuitMenu/Button Exit Game Normal.dds", 1, 0.01f);
	myExitButton.Initialize();
	myExitButton.SetActiveSheet(0);
	myExitButton.SetPosition(myExitButtonOffset);

	myContinueButton.AddSpriteSheet("img/QuitMenu/Button Continue Game Normal.dds", 1, 0.01f);
	myContinueButton.Initialize();
	myContinueButton.SetActiveSheet(0);
	myContinueButton.SetPosition(myContinueButtonOffset);

	exitButtonMin = Vector2<float>((myPosition.x + myExitButtonOffset.x), (myPosition.y + myExitButtonOffset.y));
	exitButtonMax = Vector2<float>((myPosition.x + myExitButtonOffset.x + myExitButton.GetSpriteWidth()), (myPosition.y + myExitButtonOffset.y + myExitButton.GetSpriteHeight()));

	continueButtonMin = Vector2<float>((myPosition.x + myContinueButtonOffset.x), (myPosition.y + myContinueButtonOffset.y));
	continueButtonMax = Vector2<float>((myPosition.x + myContinueButtonOffset.x + myContinueButton.GetSpriteWidth()), (myPosition.y + myContinueButtonOffset.y + myContinueButton.GetSpriteHeight()));
}

void QuitMenu::HandleInput( InputWrapper &aInput )
{
	if (EventSingelton::GetInstance()->myQuitMenuActive == true)
	{
		if ( aInput.ButtonClicked(0) == true )
		{
			Vector2<long> mousePosition = aInput.GetMousePosition();
			
			if( mousePosition.x > exitButtonMin.x
				&& mousePosition.y > exitButtonMin.y
				&& mousePosition.x < exitButtonMax.x
				&& mousePosition.y < exitButtonMax.y)
			{
				EventSingelton::GetInstance()->myQuitGame = true;
			}
			else if (mousePosition.x > continueButtonMin.x
				&& mousePosition.y > continueButtonMin.y
				&& mousePosition.x < continueButtonMax.x
				&& mousePosition.y < continueButtonMax.y)
			{
				EventSingelton::GetInstance()->myQuitMenuActive = false;
			}
		}
	}
}
