#include "Robot.h"
#include <math.h> 
#include "EventSingelton.h"

Robot::Robot()
{
	myFollowAvatar = true;
}

Robot::Robot(HGE* aHge)
:myHge(aHge)
{
	mySpeed = 0.9f;

	myWorldSpacePosition = Vector2<float>(480, 880);
	myCameraSpacePosition;
	myAnimation = AnimationBase(aHge);
	myRotation = 0.0f;
	myScale = Vector2<float>(1.0f, 1.0f);

	myDirection = DEACTIVATED;
	myLastDirection = myDirection;
	myActive = false;

	Vector3<float> temporaryDirection(1, 0, 0);
	Matrix33<float> temporaryRotation;
	temporaryRotation = temporaryRotation.CreateRotateAroundZ((-M_PI + (M_PI_4/2)));
	myFirstDirectionSegmentX = (temporaryDirection * temporaryRotation).x;
	temporaryRotation *= temporaryRotation.CreateRotateAroundZ(M_PI_4);
	mySecondDirectionSegmentX = (temporaryDirection * temporaryRotation).x;
	temporaryRotation *= temporaryRotation.CreateRotateAroundZ(M_PI_4);
	myThirdDirectionSegmentX = (temporaryDirection * temporaryRotation).x;
	temporaryRotation *= temporaryRotation.CreateRotateAroundZ(M_PI_4);
	myFourthDirectionSegmentX = (temporaryDirection * temporaryRotation).x;

	myIsAvatarInProximity = false;

	myInterractionButton = StaticSprite( aHge, "img/icon/icon_e.dds" );

	myStateChangeTiming = 0.0f;
	myFollowAvatar = true;
}
Robot::Robot(const Robot &aRobot)
:myHge(aRobot.myHge)
{
	mySpeed = aRobot.mySpeed;

	myWorldSpacePosition = aRobot.myWorldSpacePosition;
	myCameraSpacePosition = aRobot.myCameraSpacePosition;
	myAnimation = aRobot.myAnimation;
	myRotation = aRobot.myRotation;
	myScale = aRobot.myScale;
	myActive = aRobot.myActive;

	myDirection = aRobot.myDirection;
	myLastDirection = aRobot.myLastDirection;

	myFirstDirectionSegmentX = aRobot.myFirstDirectionSegmentX;
	mySecondDirectionSegmentX = aRobot.mySecondDirectionSegmentX;
	myThirdDirectionSegmentX = aRobot.myThirdDirectionSegmentX;
	myFourthDirectionSegmentX = aRobot.myFourthDirectionSegmentX;

	myIsAvatarInProximity = aRobot.myIsAvatarInProximity;

	myInterractionButton = aRobot.myInterractionButton;

	myStateChangeTiming = aRobot.myStateChangeTiming;
	myFollowAvatar = aRobot.myFollowAvatar;
}

Robot::~Robot()
{
}

void Robot::LoadAnimationSheet()
{
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_deactivated/r_deactivated.dds", 1, 1.0f);		// 0
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_activate/r_activate.dds", 16, 20.0f );			// 1
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_walk/r_walk_up.dds", 16, 20.0f );				// 2
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_idling/r_idling_up.dds", 16, 20.0f );			// 3
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_walk/r_walk_right_up.dds", 16, 20.0f );		// 4
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_idling/r_idling_right_up.dds", 16, 20.0f );	// 5
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_walk/r_walk_right.dds", 16, 20.0f );			// 6
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_idling/r_idling_right.dds", 16, 20.0f);		// 7
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_walk/r_walk_right_down.dds", 16, 20.0f );		// 8
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_idling/r_idling_right_down.dds", 16, 20.0f );	// 9
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_walk/r_walk_down.dds", 16, 20.0f );			// 10
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_idling/r_idling_down.dds", 16, 20.0f );		// 11
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_walk/r_walk_left_down.dds", 16, 20.0f );		// 12
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_idling/r_idling_left_down.dds", 16, 20.0f );	// 13
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_walk/r_walk_left.dds", 16, 20.0f );
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_idling/r_idling_left.dds", 16, 20.0f );
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_walk/r_walk_left_up.dds", 16, 20.0f );
	myAnimation.AddSpriteSheet("img/animation/RobotAni/robot_idling/r_idling_left_up.dds", 16, 20.0f);

}

void Robot::Initialize()
{
	LoadAnimationSheet();
	myAnimation.Initialize();
	myAnimation.SetActiveSheet(myDirection);
	myAnimation.SetPosition(myCameraSpacePosition);
}

void Robot::Update(float aDeltaTime, Vector2<float> aCameraPosition, Vector2<float> &anAvatarPosition)
{
	aDeltaTime;
	
	myCameraSpacePosition = myWorldSpacePosition - aCameraPosition;
	myAnimation.SetPosition(myCameraSpacePosition);
	
	if (EventSingelton::GetInstance()->myNotebookActive == true
		|| EventSingelton::GetInstance()->myConversationActive == true
		|| EventSingelton::GetInstance()->myInventoryActive == true
		|| EventSingelton::GetInstance()->myQuitMenuActive == true)
	{
		myAnimation.Pause();
	}
	else
	{
		myAnimation.Play();
	}


	IsAvatarInProximity( anAvatarPosition );
}

void Robot::SetWorldSpacePosition(Vector2<float> aPosition)
{
	myWorldSpacePosition = aPosition;
}

Vector2<float> Robot::GetWorldSpacePosition()
{
	return myWorldSpacePosition;
}

void Robot::SetCameraSpacePosition(Vector2<float> aPosition)
{
	myCameraSpacePosition = aPosition;
}

Vector2<float> Robot::GetCameraSpacePosition()
{
	return myCameraSpacePosition;
}

void Robot::SetDirection(int aDirection)
{
	myDirection = aDirection;
	myAnimation.SetActiveSheet(myDirection);
}

int Robot::GetDirection()
{
	return myDirection;
}

void Robot::ChaseAvatar(Vector2<float> aAvatarWorldPosition, float aDelta)
{
	if ( myActive == true && myFollowAvatar == true)
	{
		Vector2<float> temp = (aAvatarWorldPosition - myWorldSpacePosition);
		float distance = temp.Length();
		if (distance > 100.0f)
		{
			temp = temp / distance;
			SetDirection(temp);
			
			temp *= mySpeed * aDelta;			
			myWorldSpacePosition += temp;

			myStateChangeTiming = 0.0;
		}
		else
		{
			myStateChangeTiming += aDelta;
			if( myStateChangeTiming > 100 )
			{
				Halt();
			}
		}
	}
}

void Robot::MakeActive()
{
	myActive = true;
}

void Robot::MakeNotActive()
{
	myActive = false;
}

void Robot::SetDirection(Vector2<float> aNormalizedHeading)
{
	if ( aNormalizedHeading.x < myFirstDirectionSegmentX )
	{
		SetDirection(WALK_LEFT);
	}
	else if ( aNormalizedHeading.x < mySecondDirectionSegmentX )
	{
		if ( aNormalizedHeading.y < 0 )
		{
			SetDirection(WALK_LEFT_UP);
		}
		else
		{
			SetDirection(WALK_DOWN_LEFT);
		}
	}
	else if ( aNormalizedHeading.x < myThirdDirectionSegmentX )
	{
		if ( aNormalizedHeading.y < 0 )
		{
			SetDirection(WALK_UP);
		}
		else
		{
			SetDirection(WALK_DOWN);
		}
	}
	else if ( aNormalizedHeading.x < myFourthDirectionSegmentX )
	{
		if ( aNormalizedHeading.y < 0 )
		{
			SetDirection(WALK_RIGHT_UP);
		}
		else
		{
			SetDirection(WALK_RIGHT_DOWN);
		}
	}
	else
	{
		SetDirection(WALK_RIGHT);
	}
}
void Robot::Halt()
{
	if(myDirection > ACTIVATING && myDirection % 2 == 0)
	{
		myDirection += 1;
	}
	myAnimation.SetActiveSheet(myDirection);
}

AnimationBase& Robot::GetAnimationRef()
{
	return myAnimation;
}

bool Robot::GetActive()
{
	return myActive;
}

void Robot::Render(RenderData &someRenderData)
{
	RenderMessage tempMessage;
	tempMessage.myPosition = myCameraSpacePosition;
	tempMessage.myPosition.x -= 45.0f;
	tempMessage.myType = animationRenderMessageType;
	someRenderData.myMessages.push_back( tempMessage );
	someRenderData.myAnimations.push_back( &myAnimation );

	if( myActive == false )
	{
		if( myIsAvatarInProximity == true )
		{
			tempMessage.myPosition = Vector2<float>(myCameraSpacePosition.myX + 50, myCameraSpacePosition.myY - 50);
			tempMessage.myType = spriteRenderMessageType;
			someRenderData.myMessages.push_back( tempMessage );
			someRenderData.mySprites.push_back( &myInterractionButton );
		}
	}
}

void Robot::IsAvatarInProximity( Vector2<float> &anAvatarPosition )
{
	if( anAvatarPosition.myX < myWorldSpacePosition.myX + 50 && anAvatarPosition.myX > myWorldSpacePosition.myX - 50 )
	{
		if( anAvatarPosition.myY < myWorldSpacePosition.myY + 50 && anAvatarPosition.myY > myWorldSpacePosition.myY - 50 )
		{
			myIsAvatarInProximity = true;
			return;
		}
	}

	myIsAvatarInProximity = false;
}

bool Robot::GetAvatarNear()
{
	return myIsAvatarInProximity;
}

bool Robot::GetBeenActivated( )
{
	return myHasBeenActivated;
}

void Robot::SetFollowingAvatar(const bool aFollowing)
{
	myFollowAvatar = aFollowing;
	if (aFollowing == false)
	{
		Halt();
	}
}