#ifndef _ROBOT_H_
#define _ROBOT_H_
#include "animationbase.h"
#include "inputwrapper/inputwrapper.h"
#include "RenderInformation.h"

class Robot
{
public:
	enum Direction
	{
		DEACTIVATED,		// 0
		ACTIVATING,			// 1
		WALK_UP,			// 2
		IDLE_UP,			// 3
		WALK_RIGHT_UP,		// 4
		IDLE_RIGHT_UP,		// 5
		WALK_RIGHT,			// 6
		IDLE_RIGHT,			// 7
		WALK_RIGHT_DOWN,	// 8
		IDLE_RIGHT_DOWN,	// 9
		WALK_DOWN,			// 10
		IDLE_DOWN,			// 11
		WALK_DOWN_LEFT,		// 12
		IDLE_DOWN_LEFT,		// 13
		WALK_LEFT,			// 14
		IDLE_LEFT,			// 15
		WALK_LEFT_UP,		// 16
		IDLE_LEFT_UP		// 17
	};
	Robot();
	Robot(HGE* aHge);
	Robot(const Robot& aRobot);
	~Robot();

	void SetWorldSpacePosition(Vector2<float> aPosition);
	void SetCameraSpacePosition(Vector2<float> aPosition);

	
	int GetDirection();
	bool GetAvatarNear();

	Vector2<float> GetCameraSpacePosition();
	Vector2<float> GetWorldSpacePosition();

	void LoadAnimationSheet();
	void Initialize();
	void ChaseAvatar(Vector2<float> aAvatarPosition, float aDelta);
	void Update(float aDeltaTime, Vector2<float> aCameraPosition, Vector2<float> &anAvatarPosition);
	void Render(RenderData &someRenderData);
	void MakeActive();
	void MakeNotActive();
	void IsAvatarInProximity( Vector2<float> &anAvatarPosition );
	void Halt();

	void SetFollowingAvatar(const bool aFollowing);

	bool GetBeenActivated( );
	bool GetActive( );

	AnimationBase& GetAnimationRef();

private:
	void SetDirection(Vector2<float> aHeading);
	void SetDirection(int aDirection);

	bool myHasBeenActivated;
	bool myIsAvatarInProximity;
	bool myActive;
	int myDirection;
	int myLastDirection;
	float mySpeed;

	HGE* myHge;
	AnimationBase myAnimation;

	StaticSprite myInterractionButton;

	Vector2<float> myWorldSpacePosition;
	Vector2<float> myCameraSpacePosition;
	Vector2<float> myScale;
	float myRotation;

	float myFirstDirectionSegmentX;
	float mySecondDirectionSegmentX;
	float myThirdDirectionSegmentX;
	float myFourthDirectionSegmentX;

	float myStateChangeTiming;
	bool myFollowAvatar;
};
#endif