#include "SoundSystem.h"
#include <cstdlib>
SoundSystem *SoundSystem::ourInstance = NULL;

void SoundSystem::Create()
{
	ourInstance = new SoundSystem();
	ourInstance->Init();
}

void SoundSystem::Destroy()
{
	delete ourInstance;
}

SoundSystem &SoundSystem::Instance()
{
	return *ourInstance;
}

void SoundSystem::PlayMusic()
{

	if(myCurrentSound == "")
	{
		mySoundList[GetIndex(myCurrentSound)].myChannel->setPaused(true);
	}
	else
	{
		mySoundList[GetIndex(myCurrentSound)].myChannel->setPaused(false);
	}

}

void SoundSystem::PlayThisMusic(std::string musicID)
{
	mySoundList[GetIndex(myCurrentSound)].myChannel->setPaused(false);
}
void SoundSystem::ChangeMusic(std::string musicID)
{
	if(musicID != "")
	{
		std::string nextMusic = musicID;
		if( myCurrentSound != nextMusic )
		{
			mySoundList[GetIndex(myCurrentSound)].myChannel->setPaused(true);
			myCurrentSound = nextMusic;	
		}
	}
	else
	{
		AllMusicStop();
	}
	PlayMusic();
}
void SoundSystem::AllMusicStop()
{
	for(unsigned int index = 0; index < mySoundList.size(); index++)
	{
		mySoundList[index].myChannel->setPaused(true);
	}
	myCurrentSound = "";
}

void SoundSystem::Update()
{
	myMusicSystem->update();
}

SoundSystem::SoundSystem()
{
	myMusicSystem = NULL;
	myBackgroundStream = NULL;
	myBackgroundChannel = NULL;
	for(unsigned int index = 0; index < mySoundList.size(); index++)
	{
		mySoundList[index].mySound = NULL;
		mySoundList[index].myChannel = NULL;
	}
}

SoundSystem::~SoundSystem()
{
	myMusicSystem->release();
}

void SoundSystem::Init()
{
	FMOD::System_Create(&myMusicSystem);
	myMusicSystem->init(100, FMOD_INIT_NORMAL, NULL);

	mySoundList.push_back(SoundData("StartScreen", "aud/main_screen.mp3"));///Levels/
	mySoundList.push_back(SoundData("Jungle", "aud/the_jungl_3_finished_v.mp3"));
	mySoundList.push_back(SoundData("Village", "aud/the_village_2_finished_v.mp3"));
	mySoundList.push_back(SoundData("Vulcano", "aud/the_vulcano_2_finished.mp3"));
	mySoundList.push_back(SoundData("Machine", "aud/The_time_machine_final.mp3"));

	for(unsigned int index = 0; index < mySoundList.size(); index++)
	{
		FMOD_RESULT result = myMusicSystem->createStream(mySoundList[index].mySoundSource.c_str(), FMOD_LOOP_NORMAL, 0, &mySoundList[index].mySound);
		myMusicSystem->playSound(FMOD_CHANNEL_FREE, mySoundList[index].mySound, true, &mySoundList[index].myChannel);
		mySoundList[index].myChannel->setVolume(2.03f);
		result;
	}

	/*myMusicSystem->createSound("sound/crystal.mp3", FMOD_DEFAULT, 0, &myCrystalSound);
	myMusicSystem->createSound("sound/laser-turn.mp3", FMOD_DEFAULT, 0, &myLaserTurnSound);
	myMusicSystem->createSound("sound/laser-slide.mp3", FMOD_DEFAULT, 0, &myLaserSlideSound);*/

	myCurrentSound = "";
}

int SoundSystem::GetIndex(std::string aSoundID)
{
	int confirmedIndex = 0;
	for(unsigned int index = 0; index < mySoundList.size(); index++)
	{
		if(mySoundList[index].myID == aSoundID)
		{
			confirmedIndex = index;
		}
	}
	return confirmedIndex;
}