#include "ThreadSharedData.h"
#include "HGE\hge.h"
#include "Vector.h"
#include "CollisionMap.h"
#include "EventSingelton.h"

ThreadSharedData::ThreadSharedData()
{
	threadLogicStart = false;
	threadGraphicStart = false;
	threadLogicDone = true;
	threadGraphicDone = true;
	quitGame = false;

	logicIndex = 1;
	graphicIndex = 0;
	myActiveLevel = 0;
}

ThreadSharedData::~ThreadSharedData( )
{
}

void ThreadSharedData::Initialize(HGE *aHGE, Vector2<float> aScreenDimensions)
{
	myHGE = aHGE;

	SoundSystem::Create();

	myStartScreen = StartScreen(myHGE);
	myQuitMsgBox = QuitMsgBox(myHGE);

	myCamera.myPosition.x = 100;
	myCamera.myPosition.y = 100;

	myJungleCrashSiteLevel.Initialize(myHGE);
	myDeepJungleLevel.Initialize(myHGE);
	myTimeMachineLevel.Initialize(myHGE);
	myCaveLevel.Initialize(myHGE);
	myBlownUpCaveLevel.Initialize(myHGE);
	myVillageLevel.Initialize(myHGE);
	myVulcanoLevel.Initialize(myHGE);
	myActiveLevel = &myTimeMachineLevel;

	Avatar tempAvatar(myHGE, aScreenDimensions/2);
	tempAvatar.Initialize(myCamera.myPosition);
	myAvatar = tempAvatar;
	myAvatar.SetWorldPosition(Vector2<float>(2000, 2000)); //Works for: Time Machine
	//myAvatar.SetWorldPosition(Vector2<float>(530, 330)); // jungle crash
	//myAvatar.SetWorldPosition(Vector2<float>(330, 330)); // deep jungle
	//myAvatar.SetWorldPosition(Vector2<float>(1000, 1100)); //Works for: Village
	//myAvatar.SetWorldPosition(Vector2<float>(200, 600)); //Works for: Vulcano
	//myAvatar.SetWorldPosition(Vector2<float>(200, 300)); 

	myCamera.CenterOnAvatar(myAvatar.GetWorldSpacePosition());

	myInventory = Inventory( myHGE );
	myNotebook = Notebook(myHGE);

	RenderData tempRenderData;
	tempRenderData.myAnimations.clear();
	tempRenderData.myMessages.clear();
	tempRenderData.mySprites.clear();
	tempRenderData.myDelta = 0;

	myRenderData.push_back(tempRenderData);
	myRenderData.push_back(tempRenderData);

	Tools::Timer &masterTimer = myTimerFactory.GetMasterTimer();
	myInGameTimer = &myTimerFactory.CreateTimer();
	Tools::Timer &logicTimer = myTimerFactory.CreateTimer();
	masterTimer.AddChild((*myInGameTimer));
	myInGameTimer->AddChild(logicTimer);
	myLogicTimer = &logicTimer;
	myTimerFactory.UpdateTimers();

	Item axe( aHGE, "img/Items/icon_axe.dds", "img/Items/icon_axe.dds", "axe" );
	Item coconutHalf1( aHGE, "img/empty.dds", "img/Items/icon_coconutHalf1.dds", "coconut half 1" );
	Item coconutHalf2( aHGE, "img/empty.dds", "img/Items/icon_coconutHalf2.dds", "coconut half 2" );
	Item fire( aHGE, "img/empty.dds", "img/Items/icon_fire.dds", "fire" );
	Item charcoal( aHGE, "img/empty.dds", "img/Items/icon_woodcoal.dds", "charcoal" );
	Item gunpowder( aHGE, "img/empty.dds", "img/Items/icon_coconutGunpowder.dds", "gunpowder" );
	Item raft( aHGE, "img/empty.dds", "img/empty.dds", "raft" );
	Item birdSuit( aHGE, "img/empty.dds", "img/empty.dds", "bird suit" );
	Item rubberString( aHGE, "img/empty.dds", "img/items/icon_coconutLatexWire.dds", "rubber string" );
	Item wholebelt( aHGE, "img/empty.dds", "img/items/icon_belt_whole.dds", "combined rubber" );
	myInventory.myCombiner.AddItem( axe );
	myInventory.myCombiner.AddItem( coconutHalf1 );
	myInventory.myCombiner.AddItem( coconutHalf2 );
	myInventory.myCombiner.AddItem( fire );
	myInventory.myCombiner.AddItem( charcoal );
	myInventory.myCombiner.AddItem( gunpowder );
	myInventory.myCombiner.AddItem( raft );
	myInventory.myCombiner.AddItem( birdSuit );
	myInventory.myCombiner.AddItem( rubberString );
	myInventory.myCombiner.AddItem( wholebelt );

	myInventory.myItems.reserve( 15 );
	myInventory.myCombiningItems.reserve( 3 );
	Item tempItem( aHGE, "img/Items/icon_wire.dds", "img/Items/icon_wire.dds", Vector2<float>(0, 0), "wire");
	myInventory.AddItem( tempItem );

	myCursor = Cursor( aHGE, "img/cursor.dds" );

	myCurtain = Curtain( aHGE );

	myJungleCrashSiteLevel.myHotSpots.push_back( HotSpot( Vector2<float>( 268, 1000 ), 522, 30, "DeepJungleTransfer" ) );
	myDeepJungleLevel.myHotSpots.push_back( HotSpot( Vector2<float>( 644, 0 ), 297, 36, "JungleCrashTransfer" ) );
	myTimeMachineLevel.myHotSpots.push_back( HotSpot( Vector2<float>( 91, 900 ), 98, 20, "JungleCrashTransferFromTimeMachine" ) );
	myJungleCrashSiteLevel.myHotSpots.push_back( HotSpot( Vector2<float>( 88, 267 ), 107, 70, "TimeMachineLevel" ) );
	myJungleCrashSiteLevel.myHotSpots.push_back( HotSpot( Vector2<float>( 425, 40 ), 250, 40, "VillageTransferFromJungleCrash" ) );
	myVillageLevel.myHotSpots.push_back( HotSpot( Vector2<float>( 543, 2012 ), 991, 36, "JungleCrashTransferFromVillage" ) );

	myTimeMachineLevel.myHotSpots[0].ConnectHotspot( myJungleCrashSiteLevel.myHotSpots[1] );
	myJungleCrashSiteLevel.myHotSpots[1].ConnectHotspot( myTimeMachineLevel.myHotSpots[0] );
	myJungleCrashSiteLevel.myHotSpots[0].ConnectHotspot( myDeepJungleLevel.myHotSpots[0] );
	myDeepJungleLevel.myHotSpots[0].ConnectHotspot( myJungleCrashSiteLevel.myHotSpots[0] );
	myJungleCrashSiteLevel.myHotSpots[2].ConnectHotspot( myVillageLevel.myHotSpots[0] );
	myVillageLevel.myHotSpots[0].ConnectHotspot( myJungleCrashSiteLevel.myHotSpots[2] );

	myJungleCrashSiteLevel.myVerticalHotSpots.push_back( VerticalHotSpot( Vector2<float>( 1962, 257 ), 86, 393, "VolcanoTransferFromJungleCrash" ) );
	myVulcanoLevel.myVerticalHotSpots.push_back( VerticalHotSpot( Vector2<float>( 0, 442 ), 25, 582, "JungleCrashTransferFromVolcano" ) );

	myJungleCrashSiteLevel.myVerticalHotSpots[0].ConnectVerticalHotspot( myVulcanoLevel.myVerticalHotSpots[0] );
	myVulcanoLevel.myVerticalHotSpots[0].ConnectVerticalHotspot( myJungleCrashSiteLevel.myVerticalHotSpots[0] );

	myDeepJungleLevel.myRandomHotSpots.push_back( RandomHotSpot( Vector2<float>( 66, 130 ), 71, 96, "CaveFromDeepJungle" ) );
	myCaveLevel.myRandomHotSpots.push_back( RandomHotSpot( Vector2<float>( 313, 75 ), 57, 22, "DeepJungleFromCave" ) );
	myBlownUpCaveLevel.myRandomHotSpots.push_back( RandomHotSpot( Vector2<float>( 313, 75 ), 57, 22, "DeepJungleFromBlownUpCave" ) );

	myDeepJungleLevel.myRandomHotSpots[0].ConnectRandomHotSpot( myCaveLevel.myRandomHotSpots[0] );
	myCaveLevel.myRandomHotSpots[0].ConnectRandomHotSpot( myDeepJungleLevel.myRandomHotSpots[0] );
	myBlownUpCaveLevel.myRandomHotSpots[0].ConnectRandomHotSpot( myDeepJungleLevel.myRandomHotSpots[0] );

	myQuitMenu = QuitMenu(myHGE);
	myQuitMenu.Initialize();
	myOutro = Outro(myHGE);
}

void ThreadSharedData::HandleInput(InputWrapper &aInputWrapper, float aDelta )
{
	if((myActiveLevel->myConversationList[ myActiveLevel->myActiveConversationIndex ].myActiveConversation == false) 
		&& (myInventory.IsActive() == false) 
		&& (myCurtain.myIsFading == false) 
		&& (myNotebook.IsActive() == false)
		&& (EventSingelton::GetInstance()->myQuitMenuActive == false))
	{
		myAvatar.Input(aInputWrapper, myCamera.myPosition, myActiveLevel->myCollisionMap);
	}	
	
	myActiveLevel->HandleInput(aInputWrapper, aDelta, myInventory);
	myActiveLevel->LevelSpecificHandleInput(aInputWrapper, aDelta, myInventory);
	
	if((myNotebook.IsActive() == false) 
		&& (myActiveLevel->myConversationList[myActiveLevel->myActiveConversationIndex].myActiveConversation == false) 
		&& (myActiveLevel->myRobot.GetActive() == true)
		&& (EventSingelton::GetInstance()->myQuitMenuActive == false))
	{
		myInventory.HandleInput( aInputWrapper );
	}

	if((myInventory.IsActive() == false) 
		&& (myActiveLevel->myConversationList[myActiveLevel->myActiveConversationIndex].myActiveConversation == false)
		&& (EventSingelton::GetInstance()->myQuitMenuActive == false))
	{
		myNotebook.InputHandling(aInputWrapper);
	}

	if(myStartScreen.startTheGame == false)
	{
		myStartScreen.Input(aInputWrapper);
	}

	if(aInputWrapper.KeyDown(DIK_ESCAPE) && EventSingelton::GetInstance()->myIntroHasPlayed == true)
	{
		//Ask if the player wants to quit
		//If y is clicked quit
		//If n is clicked, stop rendering y/n msgbox and continue game
		//else continue displaying y/n msg

		EventSingelton::GetInstance()->myQuitMenuActive = true;
	}

	myQuitMenu.HandleInput(aInputWrapper);
}

void ThreadSharedData::Update(InputWrapper &aInputWrapper, float aDelta)
{	
	if(myStartScreen.startTheGame == false)
	{
		myStartScreen.StartScreenScene(aDelta);
		SoundSystem::Instance().PlayThisMusic("StartScreen");
		if(myStartScreen.startTheGame == true)
		{
			SoundSystem::Instance().AllMusicStop();
		}
	}
	if(myStartScreen.startTheGame == true)
	{
		myCurtain.FadeIn();
		myActiveLevel->Update(myAvatar, myCamera, aDelta, myInventory);
		myActiveLevel->LevelSpecificUpdate(myAvatar, myCamera, aDelta);

		myCamera.LimitToWithinMap(myActiveLevel->myMap.GetMapWidth(), myActiveLevel->myMap.GetMapHeight());
		myAvatar.Update(aDelta, myCamera.myPosition);
		myCursor.Update( aInputWrapper.GetMousePosition() );
		CheckLevelChange( );
	}
	if(myActiveLevel->myEndOfGame == true)
	{
		myOutro.PlayOutro(aDelta);
	}

	myNotebook.ObtainItemUpdate(myInventory);
}

void ThreadSharedData::Render(InputWrapper &aInputWrapper, int aActiveRenderDataIndex)
{
	myActiveLevel->RenderFloor(myRenderData[aActiveRenderDataIndex], myCamera);
	myActiveLevel->LevelSpecificRender(myRenderData[aActiveRenderDataIndex], myCamera);
	myActiveLevel->RenderMyItems(myRenderData[aActiveRenderDataIndex]);
	myAvatar.Render(myRenderData[aActiveRenderDataIndex]);
	myActiveLevel->RenderOverlay(myRenderData[aActiveRenderDataIndex], myCamera);
	myActiveLevel->RenderConversations(myRenderData[aActiveRenderDataIndex], myCamera);
	myInventory.Render( myRenderData[aActiveRenderDataIndex], static_cast<float>( aInputWrapper.GetMousePosition( ).myX ), static_cast<float>( aInputWrapper.GetMousePosition( ).myY ) );
	myNotebook.Render(myRenderData[aActiveRenderDataIndex]);
	myCurtain.Render(myRenderData[aActiveRenderDataIndex]);
	myStartScreen.Render(myRenderData[aActiveRenderDataIndex]);
	myActiveLevel->myBirdCutscene.Render(myRenderData[aActiveRenderDataIndex]);
	myOutro.Render(myRenderData[aActiveRenderDataIndex]);
	myQuitMenu.Render(myRenderData[aActiveRenderDataIndex]);
	myCursor.Render( myRenderData[aActiveRenderDataIndex], myInventory.IsActive( ) );
}

void ThreadSharedData::CheckLevelChange( )
{
	if( myActiveLevel->myChangedLevel == true )
	{
		for( unsigned i = 0; i < 300; i++ )
		{
			myCurtain.FadeOut( );
		}
		if( myActiveLevel->myChangedLevelIndex == 0 )
		{
			myActiveLevel->myChangedLevel = false;
			myActiveLevel->myChangedLevelIndex = 0;
			myActiveLevel = &myJungleCrashSiteLevel;
			myCurtain.FadeIn( );
		}
		else if( myActiveLevel->myChangedLevelIndex == 1 )
		{
			myActiveLevel->myChangedLevel = false;
			myActiveLevel->myChangedLevelIndex = 0;
			myActiveLevel = &myDeepJungleLevel;
			myActiveLevel->myRobot = myJungleCrashSiteLevel.myRobot;
			myActiveLevel->myRobot.SetWorldSpacePosition( Vector2<float>( myAvatar.GetWorldSpacePosition( ).myX - 20, myAvatar.GetWorldSpacePosition( ).myY + 50 ) );
			myCurtain.FadeIn( );
		}
		else if( myActiveLevel->myChangedLevelIndex == 2 )
		{
			myActiveLevel->myChangedLevel = false;
			myActiveLevel->myChangedLevelIndex = 0;
			if( myJungleCrashSiteLevel.myRobot.GetActive( ) == true )
			{
				myTimeMachineLevel.myRobot = myJungleCrashSiteLevel.myRobot;
				myTimeMachineLevel.myRobot.SetWorldSpacePosition( Vector2<float>( myAvatar.GetWorldSpacePosition( ).myX, myAvatar.GetWorldSpacePosition( ).myY - 50 ) );
			}
			myActiveLevel = &myTimeMachineLevel;
			myCurtain.FadeIn( );
		}
		else if( myActiveLevel->myChangedLevelIndex == 3 )
		{
			myActiveLevel->myChangedLevel = false;
			myActiveLevel->myChangedLevelIndex = 0;
			if( myTimeMachineLevel.myRobot.GetActive( ) == true )
			{
				myJungleCrashSiteLevel.myRobot = myTimeMachineLevel.myRobot;
				myJungleCrashSiteLevel.myRobot.SetWorldSpacePosition( Vector2<float>( myAvatar.GetWorldSpacePosition( ).myX, myAvatar.GetWorldSpacePosition( ).myY + 50 ) );
			}
			myActiveLevel = &myJungleCrashSiteLevel;
			myCurtain.FadeIn( );
		}
		else if( myActiveLevel->myChangedLevelIndex == 4 )
		{
			if( myCaveLevel.myCaveHasBeenBlownUp == false )
			{
				myActiveLevel->myChangedLevel = false;
				myActiveLevel->myChangedLevelIndex = 0;
				myActiveLevel = &myCaveLevel;
				myActiveLevel->myRobot.MakeActive( );
				myAvatar.SetWorldPosition( Vector2<float>( myCaveLevel.myRandomHotSpots[0].GetBoxSpot( ).myX + myCaveLevel.myRandomHotSpots[0].myWidth / 2, myCaveLevel.myRandomHotSpots[0].GetBoxSpot( ).myY + 30 ) );
				myCamera.CenterOnAvatar( myAvatar.GetWorldSpacePosition( ) );
				myCurtain.FadeIn( );
			}
			else
			{
				myActiveLevel->myChangedLevel = false;
				myActiveLevel->myChangedLevelIndex = 0;
				myActiveLevel = &myBlownUpCaveLevel;
				myActiveLevel->myRobot.MakeActive( );
				myAvatar.SetWorldPosition( Vector2<float>( myCaveLevel.myRandomHotSpots[0].GetBoxSpot( ).myX + myCaveLevel.myRandomHotSpots[0].myWidth / 2, myCaveLevel.myRandomHotSpots[0].GetBoxSpot( ).myY + 30 ) );
				myCamera.CenterOnAvatar( myAvatar.GetWorldSpacePosition( ) );
				myCurtain.FadeIn( );
			}
		}
		else if( myActiveLevel->myChangedLevelIndex == 5 )
		{
			myActiveLevel->myChangedLevel = false;
			myActiveLevel->myChangedLevelIndex = 0;
			myActiveLevel = &myDeepJungleLevel;
			myAvatar.SetWorldPosition( Vector2<float>( myActiveLevel->myRandomHotSpots[0].GetBoxSpot( ).myZ + 50, myActiveLevel->myRandomHotSpots[0].GetBoxSpot( ).myY + 50 ) );
			myCamera.CenterOnAvatar( myAvatar.GetWorldSpacePosition( ) );
			myCurtain.FadeIn( );
		}
		else if( myActiveLevel->myChangedLevelIndex == 6 )
		{
			myActiveLevel->myChangedLevel = false;
			myActiveLevel->myChangedLevelIndex = 0;
			if( myJungleCrashSiteLevel.myRobot.GetActive( ) == true )
			{
				myVillageLevel.myRobot = myJungleCrashSiteLevel.myRobot;
				myVillageLevel.myRobot.SetWorldSpacePosition( Vector2<float>( myAvatar.GetWorldSpacePosition( ).myX, myAvatar.GetWorldSpacePosition( ).myY + 50 ) );
			}
			myActiveLevel = &myVillageLevel;
			myCamera.CenterOnAvatar( myAvatar.GetWorldSpacePosition( ) );
			myCurtain.FadeIn( );
		}
		else if( myActiveLevel->myChangedLevelIndex == 7 )
		{
			myActiveLevel->myChangedLevel = false;
			myActiveLevel->myChangedLevelIndex = 0;
			if( myVillageLevel.myRobot.GetActive( ) == true )
			{
				myJungleCrashSiteLevel.myRobot = myVillageLevel.myRobot;
				myJungleCrashSiteLevel.myRobot.SetWorldSpacePosition( Vector2<float>( myAvatar.GetWorldSpacePosition( ).myX, myAvatar.GetWorldSpacePosition( ).myY + 50 ) );
			}
			myActiveLevel = &myJungleCrashSiteLevel;
			myCurtain.FadeIn( );
		}
		else if( myActiveLevel->myChangedLevelIndex == 100 )
		{
			myActiveLevel->myChangedLevel = false;
			myActiveLevel->myChangedLevelIndex = 0;
			if( myJungleCrashSiteLevel.myRobot.GetActive( ) == true )
			{
				myVulcanoLevel.myRobot = myJungleCrashSiteLevel.myRobot;
				myVulcanoLevel.myRobot.SetWorldSpacePosition( Vector2<float>( myAvatar.GetWorldSpacePosition( ).myX + 50, myAvatar.GetWorldSpacePosition( ).myY ) );
			}
			myActiveLevel = &myVulcanoLevel;
			myCamera.CenterOnAvatar( myAvatar.GetWorldSpacePosition( ) );
			myCurtain.FadeIn( );
		}
		else if( myActiveLevel->myChangedLevelIndex == 200 )
		{
			myActiveLevel->myChangedLevel = false;
			myActiveLevel->myChangedLevelIndex = 0;
			if( myVulcanoLevel.myRobot.GetActive( ) == true )
			{
				myJungleCrashSiteLevel.myRobot = myVulcanoLevel.myRobot;
				myJungleCrashSiteLevel.myRobot.SetWorldSpacePosition( Vector2<float>( myAvatar.GetWorldSpacePosition( ).myX - 50, myAvatar.GetWorldSpacePosition( ).myY ) );
			}
			myActiveLevel = &myJungleCrashSiteLevel;
			myCamera.CenterOnAvatar( myAvatar.GetWorldSpacePosition( ) );
			myCurtain.FadeIn( );
		}
	}
	SoundSystem::Instance().ChangeMusic(myActiveLevel->myThisLevelMusic);
}
