#include "TimeMachine.h"

TimeMachine::TimeMachine(void)
:LevelBaseData()
{
}

TimeMachine::TimeMachine(const LevelBaseData& aLevelBaseData)
:LevelBaseData(aLevelBaseData)
{
}

TimeMachine::~TimeMachine(void)
{
}

void TimeMachine::Initialize(HGE *aHGE)
{
	myHge = aHGE;

	StaticSprite tempSprite = StaticSprite(myHge, "img/Levels/scene_timemachine.dds");
	myMap.AddFloorSprite(tempSprite, Vector2<float>(0, 0));
	tempSprite = StaticSprite(myHge, "img/Levels/scene_timemachine_overlay.dds");
	myMap.AddOverlaySprite(tempSprite, Vector2<float>(0, 0));
	myCollisionMap = CollisionMap(16, "img/Levels/collision_timemachine.bmp", myHge);

	myRobot = Robot(myHge);
	myRobot.Initialize();

	InitializeConversations();

	myThisLevelMusic = "Machine";

	myItems.push_back(Item( myHge, "img/h_fainted.dds", "img/h_fainted.dds", Vector2<float>(280.0f, 770.0f), "fainted ester"));
	myItems.push_back(Item( myHge, "img/Items/sprite_belt.dds", "img/Items/icon_belt.dds", Vector2<float>(150, 300), "engine belt"));

	myConsole = Console(myHge, "img/empty.dds", Vector2<float>( 131, 223 ));

	RandomHotSpot myRandomHotSpot = RandomHotSpot(Vector2<float>( 33, 172 ), 415, 346, "last mission");
	myRandomHotSpots.push_back(myRandomHotSpot);

	myInsideTheLastPart = false;

	myEndOfGame = false;
}

void TimeMachine::LevelSpecificHandleInput(InputWrapper &aInputWrapper, float aDelta, Inventory &aInventory)
{
	aDelta;
	aInventory;
	if((aInputWrapper.KeyPressed(DIK_E)) || (aInputWrapper.KeyPressed(DIK_RETURN)))
	{

		if( myConsole.IsInterractable( ) == true )
		{
			myConsole.InputHandling( aInputWrapper );
			myTimeMachine.CheckInsettItem(aInventory);
		}

	}

	if( myInsideTheLastPart == true )
	{
		/*if( aInventory.CheckForItem( "Diamond" ) == true )
		{
			if( aInventory.CheckForItem( "combined rubber" ) == true )
			{
				if( aInventory.CheckForItem( "gold necklace" ) == true )
				{
					if( aInventory.CheckForItem( "MagneticRock" ) == true )
					{
						myEndOfGame = true;
					}
				}
			}
		}*/

	}
}

void TimeMachine::LevelSpecificUpdate(Avatar &aAvatar, Camera &aCamera, float aDelta)
{
	Vector2<float> tempVector = aAvatar.GetWorldSpacePosition();
	myRobot.ChaseAvatar( tempVector, aDelta);
	myRobot.Update(aDelta, aCamera.myPosition, tempVector );
	myConsole.Update( aCamera.myPosition, tempVector );
	
	if( myRandomHotSpots[0].IsAvatarInsideMe( tempVector ) == true )
	{
		myInsideTheLastPart = true;
	}
	else
	{
		myInsideTheLastPart = false;
	}
}

void TimeMachine::LevelSpecificRender(RenderData &aRenderData, Camera &aCamera)
{
	aCamera;
	myRobot.Render(aRenderData);
	myConsole.Render( aRenderData );
}

void TimeMachine::InitializeConversations()
{
	myConversationList.reserve(101);
	myConversationList.push_back(LevelConversation(myHge, "NONE", false));

	myConversationList.push_back(LevelConversation(myHge, "wake ester", false));
	myConversationList.push_back(LevelConversation(myHge, "ester woken", false));
	myConversationList.push_back(LevelConversation(myHge, "belt found", false));
	myConversationList.push_back(LevelConversation(myHge, "engine examined", false));
	myConversationList.push_back(LevelConversation(myHge, "missing part", false));
	myConversationList.push_back(LevelConversation(myHge, "all part in", false));
	myConversationList.push_back(LevelConversation(myHge, "maching fixed", false));
	myConversationList.push_back(LevelConversation(myHge, "home trip", false));
	
	for(unsigned int index = 0; index < myConversationList.size(); index++)
	{
		myConversationList[index].Initialize();
		myConversationList[index].myActiveConversation = false;
		myDoneConversation.set(index);
	}

	myConversationList[GetConversationIndex("wake ester")].myDialoge.SetLeftPortrait("img/empty.dds", " ");
	myConversationList[GetConversationIndex("wake ester")].myDialoge.SetRightPortrait("img/Conversation/AIPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("wake ester")].AddDialogeText("Roderick: : Esther!;");
	myConversationList[GetConversationIndex("wake ester")].AddDialogeText("Roderick: : Esther, wake up.;");

	myConversationList[GetConversationIndex("ester woken")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("ester woken")].myDialoge.SetRightPortrait("img/Conversation/AIPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("ester woken")].AddDialogeText("Esther: : Roderick... what happened? Did we crash?;");
	myConversationList[GetConversationIndex("ester woken")].AddDialogeText("Roderick: : Technically, something that stands still, only travelling through time, cannot crash. I have however, lost connection to the time machine engine. Yet we are no longer in your time, according to my calculations. Should I make a note of this as a failure?;");
	myConversationList[GetConversationIndex("ester woken")].AddDialogeText("Esther: : Failu...what? No, never mind that now. It is a success if anything if I'm in another time. But where are we?;");
	myConversationList[GetConversationIndex("ester woken")].AddDialogeText("Roderick: : At the exact same position as before your journey, naturally. We do not move in space, only in time.;");
	myConversationList[GetConversationIndex("ester woken")].AddDialogeText("Esther: : Fine, be that way. WHEN are we, then?;");
	myConversationList[GetConversationIndex("ester woken")].AddDialogeText("Roderick: : As my scanning function is out of order due to unexpected circumstances, I cannot tell to what time we have arrived. You will have to learn this manually.;");
	myConversationList[GetConversationIndex("ester woken")].AddDialogeText("Esther: : I will do so, but first, can you give me a status report on the engine?;");
	myConversationList[GetConversationIndex("ester woken")].AddDialogeText("Roderick: : I am afraid not. As mentioned, I lost connection to it shortly after arrival. You will have to upload a report directly from the engine, it is the first door you'll come across in the corridor and should be easy for you to reach. Simply navigate with the W (up) A (left) S (down) D (right) keys.;");
	myConversationList[GetConversationIndex("ester woken")].AddDialogeText("Esther: : I know how to walk, Roddy, but thank you.;");
	myConversationList[GetConversationIndex("ester woken")].AddDialogeText("Roderick: : You are welcome.;");

	myConversationList[GetConversationIndex("belt found")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("belt found")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("belt found")].AddDialogeText("Esther: : A broken engine belt, this doesn't look good...;");
	myConversationList[GetConversationIndex("belt found")].AddDialogeText("Esther: : Maybe, just maybe, we can repair it to its original state, what do you think Roderick?;");
	myConversationList[GetConversationIndex("belt found")].AddDialogeText("Roderick: : Of course we can Esther, we just need to find some raw rubber and make it into refined rubber so it can be molded and hardened safely.;");
	myConversationList[GetConversationIndex("belt found")].AddDialogeText("Esther: : Challange accepted!;");

	myConversationList[GetConversationIndex("engine examined")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("engine examined")].myDialoge.SetRightPortrait("img/Conversation/AIPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Esther: : Let's see, downloading the status report...;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Roderick: : Download complete.;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Esther: : How does it look?;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Roderick: : Bad. The engine is out of order indefinitely. Major repairs are needed and parts need to be exchanged. See the full status report in the Personal Digital Assistant.;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Esther: : Ah yes, my notebook. I'll look in it. We'll have to hope someone in this time can help.;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Roderick: : If there is 'someone' in this era. We travelled back in time, not forward.;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Esther: : Pessimist.;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Roderick: : I am simply pragmatic. But if you are going out to find help and parts, perhaps you could try to locate my mobile body. I cannot find it on the Time Machine and I believe it has rolled astray. Should you locate and activate it, I can go with you on your search.;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Esther: : I'll go find it, I might need it later when I explore this time.;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Roderick: : You still intend to do this?;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Esther: : Indeed. This is my research, after all.;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Roderick: : I suggest you repair your time machine before you discover the unknown.;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Esther: : Yes, you're right. I'd better look at that report, then. Where can I find my notebook?;");
	myConversationList[GetConversationIndex("engine examined")].AddDialogeText("Roderick: : I am not programmed to keep track of your personal belongings, Esther. You usually access it by pressing 'N'. Perhaps you should try this.;");

	myConversationList[GetConversationIndex("missing part")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("missing part")].AddDialogeText("Esther: : Hm..., there still some part left.;");
	myConversationList[GetConversationIndex("missing part")].AddDialogeText("Esther: : Well, then let's continue to search.;");

	myConversationList[GetConversationIndex("maching fixed")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("maching fixed")].myDialoge.SetRightPortrait("img/Conversation/AIPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("maching fixed")].AddDialogeText("Roderick: : Activating engine.;");
	myConversationList[GetConversationIndex("maching fixed")].AddDialogeText("Roderick: : Magnetic field stable.;");
	myConversationList[GetConversationIndex("maching fixed")].AddDialogeText("Roderick: : Energy transfer stable.;");
	myConversationList[GetConversationIndex("maching fixed")].AddDialogeText("Roderick: : Time calculations finished.;");
	myConversationList[GetConversationIndex("maching fixed")].AddDialogeText("Roderick: : System all green.;");
	myConversationList[GetConversationIndex("maching fixed")].AddDialogeText("Esther: : Yes, now everything is set...;");
	myConversationList[GetConversationIndex("maching fixed")].AddDialogeText("Esther: : Time to say good bye to this era and return home. To the control room!;");
	myConversationList[GetConversationIndex("maching fixed")].AddDialogeText("Roderick: : Which one is the control room?;");
	myConversationList[GetConversationIndex("maching fixed")].AddDialogeText("Esther: : It's the room where I have my bed, you know?;");
	myConversationList[GetConversationIndex("maching fixed")].AddDialogeText("Roderick: : Err...;");

	myConversationList[GetConversationIndex("home trip")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("home trip")].myDialoge.SetRightPortrait("img/Conversation/AIPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : Phu... Thanks goodness that the machine can be fixed.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Roderick: : It's all because I supported you.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : Shut up.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : It was fortunate that we could find the necessary components in this era, or else we would have been stuck here forever.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : But... It was an exciting adventure of discovery and wonder!;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : The feeling of fresh nature before the advanced human society.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : We encountered a long extinct tribe.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : Plus a giant bird with a modern hairstyle.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : It was fun to go around and gather the ingredients to fix the machine.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Roderick: : Yes, but you wouldn't have made it without my item combine system.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : Hey! Stop beeing so cocky. It was I who build you and that combine system of yours.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : If you have time to be proud, then get ready for departing.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Roderick: : Yes, ma'am.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : Geez, I can believe I really created a robot with a real stubborn AI in it.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Roderick: : Systems all set, ready for departure. Setting destination to year 3011.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Roderick: : Starting the countdown.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Roderick: : 10... 9... 8... 7... 6...;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : Wait... year 3011?;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : Oh no, a millenia to much!;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : Roderick!.;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Roderick: : Time tunnel active, Timetanic depart!;");
	myConversationList[GetConversationIndex("home trip")].AddDialogeText("Esther: : TOO LATE!;");

	myActiveConversationID = "";
	myActiveConversationIndex = 0;
}

void TimeMachine::ConversationEvent(float elapsTime, Avatar &aAvatar, Inventory &aInventory)
{
	myConversationDelta += elapsTime;
	static float esterWokeTimeStamp;

	if(myConversationDelta >= 1500)
	{
		myActiveConversationID = "wake ester";
	}

	if ( myDoneConversation.at(GetConversationIndex("wake ester")) == 0 )
	{
		if ( aAvatar.GetWorldSpacePosition().x > 1000 && aAvatar.GetWorldSpacePosition().y > 1000 )
		{
			int itemToEraseIndex = 0;
			if (GetItemIndex("fainted ester", itemToEraseIndex) == true)
			{
				myItems.erase(myItems.begin() + itemToEraseIndex);
			}
			aAvatar.SetWorldPosition(Vector2<float>(310.0f, 770.0f));
			esterWokeTimeStamp = myConversationDelta + 3000;
		}

		if ( myDoneConversation.at(GetConversationIndex("ester woken")) != 0 )
		{
			if ( myConversationDelta >= esterWokeTimeStamp )
			{
				myActiveConversationID = "ester woken";
			}
		}
	}

	if ( aInventory.CheckForItem("engine belt") == true )
	{
		myActiveConversationID = "belt found";	
	}

	if ( myConsole.IsActive( ) == true )
	{
		if( myDoneConversation.at( GetConversationIndex("engine examined") ) == true )
		{
			myActiveConversationID = "engine examined";	
		}
		else
		{
			if(myTimeMachine.AllItemInPlace() == true)
			{
				myActiveConversationID = "maching fixed";
			}
		}
	}

	if( myDoneConversation.at( GetConversationIndex("engine examined") ) == false )
	{
		if(myConsole.myWorldSpacePosition == Vector2<float>(9999,9999) )
		{
		}
		else
		{
			//myConsole = Console(myHge, "img/empty.dds",Vector2<float>(9999,9999));
		}
	}



	if(myTimeMachine.AllItemInPlace() == true && aAvatar.GetWorldSpacePosition().x >= 310.0f && aAvatar.GetWorldSpacePosition().y >= 770.0f)
	{

		myActiveConversationID = "home trip";
	}

	if(myDoneConversation.at( GetConversationIndex("home trip") ) == false)
	{
		myEndOfGame = true;
	}
}
