#ifndef _VECTOR2_H_
#define _VECTOR2_H_
#include <assert.h>
#include <math.h>

template<class Type>
class Vector2
{
public:
	Vector2();
	Vector2( Type aX, Type aY );
	Vector2( const Vector2<Type>& aVector2 );
	~Vector2();

	Vector2<Type> operator=( const Vector2<Type>& aVector2 );
	Vector2<Type> operator+( const Vector2<Type>& aVector2 );
	Vector2<Type> operator-( const Vector2<Type>& aVector2 );
	Vector2<Type> operator*( const Vector2<Type>& aVector2 );
	Vector2<Type> operator*( const Type& aMultitplyNumber );
	Vector2<Type> operator/( const Vector2<Type>& aVector2 );
	Vector2<Type> operator/( const Type& aDivideNumber );

	bool operator==( const Vector2<Type>& aVector2 )const;
	Vector2<Type>& operator+=( const Vector2<Type>& aVector2 );
	Vector2<Type>& operator-=( const Vector2<Type>& aVector2 );
	Vector2<Type>& operator*=( const Vector2<Type>& aVector2 );
	Vector2<Type>& operator*=( const Type& aMultitplyNumber );
	Vector2<Type>& operator/=( const Vector2<Type>& aVector2 );
	Vector2<Type>& operator/=( const Type& aDivideNumber );
	
	// The lenght of this vector
	inline float Length(){ return sqrtf( ( (x * x) + (y * y) ) ); }
	
	// the dot of this and the arguing vector
	template<class Type>
	inline Type Dot( const Vector2<Type> aVector2 ){ return ( ( myX * aVector2.x ) + ( myY * aVector2.y ) ); }

	union
	{
		Type myX;
		Type x;
		Type r;
		Type myR;
	};
	union
	{
		Type myY;
		Type y;
		Type g;
		Type myG;
	};
};
template<class Type>
Vector2<Type>::Vector2()
{
	myX = 0;
	myY = 0;
}
template<class Type>
Vector2<Type>::Vector2(Type aX, Type aY)
{
	myX = aX;
	myY = aY;
}
template<class Type>
Vector2<Type>::Vector2(const Vector2<Type> &aVector2)
{
	myX = aVector2.myX;
	myY = aVector2.myY;
}
template<class Type>
Vector2<Type>::~Vector2()
{}

template<class Type>
Vector2<Type> Vector2<Type>::operator =(const Vector2<Type> &aVector2)
{
	myX = aVector2.myX;
	myY = aVector2.myY;
	return *this;
}
template<class Type>
bool Vector2<Type>::operator ==(const Vector2<Type> &aVector2)const
{
	if( aVector2.x == myX )
	{
		if(aVector2.y == myY )
		{
			return true;
		}
	}
	return false;
}
// Adderings operator
template<class Type>
Vector2<Type> Vector2<Type>::operator +(const Vector2<Type> &aVector2)
{
	Vector2<Type> tempVector;
	tempVector.x = myX + aVector2.x;
	tempVector.y = myY + aVector2.y;
	return tempVector;
}
template<class Type>
Vector2<Type>& Vector2<Type>::operator +=(const Vector2<Type> &aVector2)
{
	myX += aVector2.x;
	myY += aVector2.y;

	return *this;
}
// Subracting operator
template<class Type>
Vector2<Type> Vector2<Type>::operator -(const Vector2<Type> &aVector2)
{
	Vector2<Type> tempVector;
	tempVector.x = myX - aVector2.x;
	tempVector.y = myY - aVector2.y;
	return tempVector;
}
template<class Type>
Vector2<Type>& Vector2<Type>::operator -=(const Vector2<Type> &aVector2)
{
	myX -= aVector2.x;
	myY -= aVector2.y;
	return *this;
}
// Multiplicating operator
template<class Type>
Vector2<Type> Vector2<Type>::operator *(const Vector2<Type> &aVector2)
{
	Vector2<Type> tempVector;
	tempVector.x = myX * aVector2.x;
	tempVector.y = myY * aVector2.y;
	return tempVector;
}
template<class Type>
Vector2<Type> Vector2<Type>::operator *(const Type &aMultitplyNumber)
{
	Vector2<Type> tempVector;
	tempVector.x = myX * aMultitplyNumber;
	tempVector.y = myY * aMultitplyNumber;
	return tempVector;
}
template<class Type>
Vector2<Type>& Vector2<Type>::operator *=(const Vector2<Type> &aVector2)
{
	myX *= aVector2.x;
	myY *= aVector2.y;

	return *this;
}
template<class Type>
Vector2<Type>& Vector2<Type>::operator *=(const Type &aMultitplyNumber)
{
	myX *= aMultitplyNumber;
	myY *= aMultitplyNumber;

	return *this;
}
// dividing
template<class Type>
Vector2<Type> Vector2<Type>::operator /(const Vector2<Type> &aVector2)
{
	Vector2<Type> tempVector;
	assert((aVector2.x != 0) && "Illegal numerator!");
	assert((aVector2.y != 0) && "Illegal numerator!");
	tempVector.x = myX / aVector2.x;
	tempVector.y = myY / aVector2.y;
	return tempVector;
}
template<class Type>
Vector2<Type> Vector2<Type>::operator /(const Type& aDivideNumber)
{
	Vector2<Type> tempVector;
	assert((aDivideNumber != 0) && "Illegal numerator!");
	tempVector.x = myX / aDivideNumber;
	tempVector.y = myY / aDivideNumber;
	return tempVector;
}
template<class Type>
Vector2<Type>& Vector2<Type>::operator /=(const Vector2<Type> &aVector2)
{
	assert((aVector2.x != 0) && "Illegal numerator!");
	assert((aVector2.y != 0) && "Illegal numerator!");
	myX /= aVector2.x;
	myY /= aVector2.y;
	return *this;
}
template<class Type>
Vector2<Type>& Vector2<Type>::operator /=(const Type& aDivideNumber)
{
	assert((aDivideNumber != 0) && "Illegal numerator!");
	myX /= aDivideNumber;
	myY /= aDivideNumber;
	return *this;
}
#endif
//template<class Type>
//float Vector2<Type>::Lenght()
//{
//	return sqrtf( ( (x * x) + (y * y) ) );
//}
//
//template<class Type>
//Type Vector2<Type>::Dot(const Vector2<Type> aVector2)
//{
//	return ( ( myX * aVector2.x ) + ( myY * aVector2.y ) );	
//}