#ifndef _VECTOR3_H_
#define _VECTOR3_H_
#include "matrix.h"
#include <assert.h>
#include <math.h>

template<class Type>
class Vector3
{
public:
	Vector3();
	Vector3( Type aX, Type aY, Type aZ );
	Vector3( const Vector3<Type>& aVector3 );
	~Vector3();
	Vector3<Type> operator=( const Vector3<Type>& aVector3 );
	Vector3<Type> operator+( const Vector3<Type>& aVector3 );
	Vector3<Type> operator-( const Vector3<Type>& aVector3 );
	Vector3<Type> operator*( const Vector3<Type>& aVector3 );
	Vector3<Type> operator*( const Type& aMultitplyNumber );
	Vector3<Type> operator*( const Matrix33<Type>& aMatrix );
	Vector3<Type> operator/( const Vector3<Type>& aVector3 );
	Vector3<Type> operator/( const Type& aDivideNumber );

	bool operator==( const Vector3<Type>& aVector3 )const;
	Vector3<Type>& operator+=( const Vector3<Type>& aVector3 );
	Vector3<Type>& operator-=( const Vector3<Type>& aVector3 );
	Vector3<Type>& operator*=( const Vector3<Type>& aVector3 );
	Vector3<Type>& operator*=( const Type& aMultitplyNumber );
	Vector3<Type>& operator/=( const Vector3<Type>& aVector3 );
	Vector3<Type>& operator/=( const Type& aDivideNumber );
	
	// The lenght of this vector
	inline float Length(){ return sqrtf( ( (x * x) + (y * y) + (z * z) ) ); }

	// the dot of this and the arguing vector
	template<class Type>
	inline Type Dot( const Vector3<Type> aVector3 ){ return ( (myX * aVector3.x) + (myY * aVector3.y) + (myZ * aVector3.z) ); }
	
	// the cross product of this and the arguing vector
	template<class Type>
	inline Vector3<Type> Cross(Vector3<Type> aVector3){
		Vector3<Type> resultVector;

		resultVector.x = (myY * aVector3.myZ) - (myZ * aVector3.myY);
		resultVector.y = (myZ * aVector3.myX) - (myX * aVector3.myZ);
		resultVector.z = (myX * aVector3.myY) - (myY * aVector3.myX);

		return resultVector;
	}

	union
	{
		Type myX;
		Type x;
		Type r;
		Type myR;
	};
	union
	{
		Type myY;
		Type y;
		Type g;
		Type myG;
	};
	union
	{
		Type myZ;
		Type z;
		Type b;
		Type myB;
	};
};
template<class Type>
Vector3<Type>::Vector3()
{
	myX = 0;
	myY = 0;
	myZ = 0;
}
template<class Type>
Vector3<Type>::Vector3(Type aX, Type aY, Type aZ)
{
	myX = aX;
	myY = aY;
	myZ = aZ;
}
template<class Type>
Vector3<Type>::Vector3(const Vector3<Type> &aVector3)
{
	myX = aVector3.myX;
	myY = aVector3.myY;
	myZ = aVector3.myZ;
}
template<class Type>
Vector3<Type>::~Vector3()
{}
template<class Type>
Vector3<Type> Vector3<Type>::operator =(const Vector3<Type> &aVector3)
{

	myX = aVector3.x;
	myY = aVector3.y;
	myZ = aVector3.z;

	return *this;
}
template<class Type>
bool Vector3<Type>::operator ==(const Vector3<Type> &aVector3)const
{
	if( aVector3.x == myX )
	{
		if(aVector3.y == myY )
		{
			if(aVector3.z == myZ )
			{
				return true;
			}
		}	
	}

	return false;
}
// Add operator
template<class Type>
Vector3<Type> Vector3<Type>::operator +(const Vector3<Type> &aVector3)
{
	Vector3<Type> tempVector;
	
	tempVector.x = myX + aVector3.myX;
	tempVector.y = myY + aVector3.myY;
	tempVector.z = myZ + aVector3.myZ;

	return tempVector;
}
template<class Type>
Vector3<Type>& Vector3<Type>::operator +=(const Vector3<Type> &aVector3)
{
	myX += aVector3.x;
	myY += aVector3.y;
	myZ += aVector3.z;

	return *this;
}
// subtrack operator
template<class Type>
Vector3<Type> Vector3<Type>::operator -(const Vector3<Type> &aVector3)
{
	Vector3<Type> tempVector;

	tempVector.x = myX - aVector3.myX;
	tempVector.y = myY - aVector3.myY;
	tempVector.z = myZ - aVector3.myZ;

	return tempVector;
}
template<class Type>
Vector3<Type>& Vector3<Type>::operator -=(const Vector3<Type> &aVector3)
{
	myX -= aVector3.x;
	myY -= aVector3.y;
	myZ -= aVector3.z;

	return *this;
}
// multiply operator
template<class Type>
Vector3<Type> Vector3<Type>::operator *(const Vector3<Type> &aVector3)
{
	Vector3<Type> tempVector;

	tempVector.x = myX * aVector3.myX;
	tempVector.y = myY * aVector3.myY;
	tempVector.z = myZ * aVector3.myZ;

	return tempVector;
}
template<class Type>
Vector3<Type> Vector3<Type>::operator *(const Type &aMultitplyNumber)
{
	Vector3<Type> tempVector;

	tempVector.x = myX * aMultitplyNumber;
	tempVector.y = myY * aMultitplyNumber;
	tempVector.z = myZ * aMultitplyNumber;

	return tempVector;
}
template<class Type>
Vector3<Type> Vector3<Type>::operator *(const Matrix33<Type> &aMatrix)
{
	Vector3<Type> tempVector;

	tempVector.x = ((myX * aMatrix.myMatrix[0]) + (myY * aMatrix.myMatrix[3]) + (myZ * aMatrix.myMatrix[6]));
	tempVector.y = ((myX * aMatrix.myMatrix[1]) + (myY * aMatrix.myMatrix[4]) + (myZ * aMatrix.myMatrix[7]));
	tempVector.z = ((myX * aMatrix.myMatrix[2]) + (myY * aMatrix.myMatrix[5]) + (myZ * aMatrix.myMatrix[8]));

	return tempVector;
}
template<class Type>
Vector3<Type>& Vector3<Type>::operator *=(const Vector3<Type> &aVector3)
{
	myX *= aVector3.x;
	myY *= aVector3.y;
	myZ *= aVector3.z;

	return *this;
}
template<class Type>
Vector3<Type>& Vector3<Type>::operator *=(const Type &aMultitplyNumber)
{
	myX *= aMultitplyNumber;
	myY *= aMultitplyNumber;
	myZ *= aMultitplyNumber;

	return *this;
}
// dividing
template<class Type>
Vector3<Type> Vector3<Type>::operator /(const Vector3<Type> &aVector3)
{
	Vector3<Type> tempVector;
	assert((aVector3.x != 0) && "Illegal numerator!");
	assert((aVector3.y != 0) && "Illegal numerator!");
	assert((aVector3.z != 0) && "Illegal numerator!");
	tempVector.x = myX / aVector3.x;
	tempVector.y = myY / aVector3.y;
	tempVector.z = myZ / aVector3.z;

	return tempVector;
}
template<class Type>
Vector3<Type> Vector3<Type>::operator /(const Type& aDivideNumber)
{
	Vector3<Type> tempVector;
	assert((aDivideNumber != 0) && "Illegal numerator!");
	
	tempVector.x = myX / aDivideNumber;
	tempVector.y = myY / aDivideNumber;
	tempVector.z = myZ / aDivideNumber;

	return tempVector;
}
template<class Type>
Vector3<Type>& Vector3<Type>::operator /=(const Vector3<Type> &aVector3)
{
	assert((aVector3.x != 0) && "Illegal numerator!");
	assert((aVector3.y != 0) && "Illegal numerator!");
	assert((aVector3.z != 0) && "Illegal numerator!");
	
	myX /= aVector3.x;
	myY /= aVector3.y;
	myZ /= aVector3.z;

	return *this;
}
template<class Type>
Vector3<Type>& Vector3<Type>::operator /=(const Type& aDivideNumber)
{
	assert((aDivideNumber != 0) && "Illegal numerator!");
	
	myX /= aDivideNumber;
	myY /= aDivideNumber;
	myZ /= aDivideNumber;

	return *this;
}
#endif
//// The lenght of this vector
//template<class Type>
//float Vector3<Type>::Lenght()
//{
//	return sqrtf( ( (x * x) + (y * y) + (z * z) ) );
//}
//// the dot of this and the arguing vector
//template<class Type>
//Type Vector3<Type>::Dot(const Vector3<Type> aVector3)
//{
//	return ( (myX * aVector3.x) + (myY * aVector3.y) + (myZ * aVector3.z) );
//}
//// the cross product of this and the arguing vector
//template<class Type>
//Vector3<Type> Vector3<Type>::Cross(Vector3<Type> aVector3)
//{
//	Vector3<Type> resultVector;
//
//	resultVector.x = (myY * aVector3.myZ) - (myZ * aVector3.myY);
//	resultVector.y = (myZ * aVector3.myX) - (myX * aVector3.myZ);
//	resultVector.z = (myX * aVector3.myY) - (myY * aVector3.myX);
//
//	return resultVector;
//}