#ifndef _VECTOR4_H_
#define _VECTOR4_H_
#include <assert.h>
#include <math.h>

template<class Type>
class Vector4
{
public:
	Vector4();
	Vector4( Type aX, Type aY, Type aZ, Type aW );
	Vector4( const Vector4<Type>& aVector4 );
	~Vector4();
	Vector4<Type> operator=( const Vector4<Type>& aVector4 );
	Vector4<Type> operator+( const Vector4<Type>& aVector4 );
	Vector4<Type> operator-( const Vector4<Type>& aVector4 );
	Vector4<Type> operator*( const Vector4<Type>& aVector4 );
	Vector4<Type> operator*( const Type& aMultitplyNumber );
	Vector4<Type> operator/( const Vector4<Type>& aVector4 );
	Vector4<Type> operator/( const Type& aDivideNumber );

	bool operator==( const Vector4<Type>& aVector4 );
	Vector4<Type>& operator+=( const Vector4<Type>& aVector4 );
	Vector4<Type>& operator-=( const Vector4<Type>& aVector4 );
	Vector4<Type>& operator*=( const Vector4<Type>& aVector4 );
	Vector4<Type>& operator*=( const Type& aMultitplyNumber );
	Vector4<Type>& operator/=( const Vector4<Type>& aVector4 );
	Vector4<Type>& operator/=( const Type& aDivideNumber );
	
	// The lenght of this vector
	inline float Length(){ return sqrtf( ( (x * x) + (y * y) + (z * z) + (w * w)) ); }
	
	// the dot of this and the arguing vector
	template<class Type>
	inline Type Dot( const Vector4<Type> aVector4 ){ return ( (myX * aVector4.x) + (myY * aVector4.y) + (myZ * aVector4.z) + (myW * aVector4.w) ); }

	union
	{
		Type myX;
		Type x;
		Type r;
		Type myR;
	};
	union
	{
		Type myY;
		Type y;
		Type g;
		Type myG;
	};
	union
	{
		Type myZ;
		Type z;
		Type b;
		Type myB;
	};
	union
	{
		Type myW;
		Type w;
		Type a;
		Type myA;
	};
};
template<class Type>
Vector4<Type>::Vector4()
{
	myX = 0;
	myY = 0;
	myZ = 0;
	myW = 0;
}
template<class Type>
Vector4<Type>::Vector4(Type aX, Type aY, Type aZ, Type aW)
{
	myX = aX;
	myY = aY;
	myZ = aZ;
	myW = aW;
}
template<class Type>
Vector4<Type>::Vector4(const Vector4<Type> &aVector4)
{
	myX = aVector4.myX;
	myY = aVector4.myY;
	myZ = aVector4.myZ;
	myW = aVector4.myW;
}
template<class Type>
Vector4<Type>::~Vector4()
{}
template<class Type>
Vector4<Type> Vector4<Type>::operator =(const Vector4<Type> &aVector4)
{

	myX = aVector4.x;
	myY = aVector4.y;
	myZ = aVector4.z;
	myW = aVector4.w;

	return *this;
}
template<class Type>
bool Vector4<Type>::operator ==(const Vector4<Type> &aVector4)
{
	if(aVector4.x == myX )
	{
		if(aVector4.y == myY )
		{
			if(aVector4.z == myZ )
			{
				if(aVector4.w == myW )
				{
					return true;
				}
			}
		}
	}

	return false;
}
// Add operator
template<class Type>
Vector4<Type> Vector4<Type>::operator +(const Vector4<Type> &aVector4)
{
	Vector4<Type> tempVector;
	
	tempVector.x = myX + aVector4.myX;
	tempVector.y = myY + aVector4.myY;
	tempVector.z = myZ + aVector4.myZ;
	tempVector.w = myW + aVector4.myW;

	return tempVector;
}
template<class Type>
Vector4<Type>& Vector4<Type>::operator +=(const Vector4<Type> &aVector4)
{
	myX += aVector4.x;
	myY += aVector4.y;
	myZ += aVector4.z;
	myW += aVector4.w;

	return *this;
}
// subtrack operator
template<class Type>
Vector4<Type> Vector4<Type>::operator -(const Vector4<Type> &aVector4)
{
	Vector4<Type> tempVector;

	tempVector.x = myX - aVector4.myX;
	tempVector.y = myY - aVector4.myY;
	tempVector.z = myZ - aVector4.myZ;
	tempVector.w = myW - aVector4.myW;

	return tempVector;
}
template<class Type>
Vector4<Type>& Vector4<Type>::operator -=(const Vector4<Type> &aVector4)
{
	myX -= aVector4.x;
	myY -= aVector4.y;
	myZ -= aVector4.z;
	myW -= aVector4.w;

	return *this;
}
// multiply operator
template<class Type>
Vector4<Type> Vector4<Type>::operator *(const Vector4<Type> &aVector4)
{
	Vector4<Type> tempVector;

	tempVector.x = myX * aVector4.myX;
	tempVector.y = myY * aVector4.myY;
	tempVector.z = myZ * aVector4.myZ;
	tempVector.w = myW * aVector4.myW;

	return tempVector;
}
template<class Type>
Vector4<Type> Vector4<Type>::operator *(const Type &aMultitplyNumber)
{
	Vector4<Type> tempVector;

	tempVector.x = myX * aMultitplyNumber;
	tempVector.y = myY * aMultitplyNumber;
	tempVector.z = myZ * aMultitplyNumber;
	tempVector.w = myW * aMultitplyNumber;

	return tempVector;
}
template<class Type>
Vector4<Type>& Vector4<Type>::operator *=(const Vector4<Type> &aVector4)
{
	myX *= aVector4.x;
	myY *= aVector4.y;
	myZ *= aVector4.z;
	myW *= aVector4.w;

	return *this;
}
template<class Type>
Vector4<Type>& Vector4<Type>::operator *=(const Type &aMultitplyNumber)
{
	myX *= aMultitplyNumber;
	myY *= aMultitplyNumber;
	myZ *= aMultitplyNumber;
	myW *= aMultitplyNumber;

	return *this;
}
// dividing
template<class Type>
Vector4<Type> Vector4<Type>::operator /(const Vector4<Type> &aVector4)
{
	Vector4<Type> tempVector;
	assert((aVector4.x != 0) && "Illegal numerator!");
	assert((aVector4.y != 0) && "Illegal numerator!");
	assert((aVector4.z != 0) && "Illegal numerator!");
	assert((aVector4.w != 0) && "Illegal numerator!");
	
	tempVector.x = myX / aVector4.x;
	tempVector.y = myY / aVector4.y;
	tempVector.z = myZ / aVector4.z;
	tempVector.w = myW / aVector4.w;

	return tempVector;
}
template<class Type>
Vector4<Type> Vector4<Type>::operator /(const Type& aDivideNumber)
{
	Vector4<Type> tempVector;
	assert((aDivideNumber != 0) && "Illegal numerator!");
	
	tempVector.x = myX / aDivideNumber;
	tempVector.y = myY / aDivideNumber;
	tempVector.z = myZ / aDivideNumber;
	tempVector.w = myW / aDivideNumber;

	return tempVector;
}
template<class Type>
Vector4<Type>& Vector4<Type>::operator /=(const Vector4<Type> &aVector4)
{
	assert((aVector4.x != 0) && "Illegal numerator!");
	assert((aVector4.y != 0) && "Illegal numerator!");
	assert((aVector4.z != 0) && "Illegal numerator!");
	assert((aVector4.w != 0) && "Illegal numerator!");
	
	myX /= aVector4.x;
	myY /= aVector4.y;
	myZ /= aVector4.z;
	myW /= aVector4.w;

	return *this;
}
template<class Type>
Vector4<Type>& Vector4<Type>::operator /=(const Type& aDivideNumber)
{
	assert((aDivideNumber != 0) && "Illegal numerator!");
	
	myX /= aDivideNumber;
	myY /= aDivideNumber;
	myZ /= aDivideNumber;
	myW /= aDivideNumber;

	return *this;
}
#endif