#ifndef ___VERTICAL_SPOT_H___
#define ___VERTICAL_SPOT_H___


#include "Vector.h"


class VerticalHotSpot
{

public:

	VerticalHotSpot( const VerticalHotSpot &aVerticalHotSpot )
	{

		myBoxSpot = aVerticalHotSpot.myBoxSpot;
		myVerticalHotSpotConnex = aVerticalHotSpot.myVerticalHotSpotConnex;
		myHeight = aVerticalHotSpot.myHeight;
		myWidth = aVerticalHotSpot.myWidth;
		myID = aVerticalHotSpot.myID;

	}

	/*By sending in position the height and width starting point
	is already defined so therefor you only need to send in a
	height and a width.
	
	
	 ______________________________________________________
	|<-aPosition.myX && aPosition.myY              aWidth->|
	|                                                      |
	|                                                      |
	|                                                      |
	|                                                      |
	|                                                      |
	|                                                      |
	|                                                      |
	|                       aHeight                        |
	|                          |                           |
	|                          v                           |
	 */
	VerticalHotSpot( Vector2<float> aPosition, float aWidth, float aHeight, std::string anID )
	{

		myBoxSpot.myX = aPosition.myX;
		myBoxSpot.myY = aPosition.myY;
		myBoxSpot.myZ = aPosition.myX + aWidth;
		myBoxSpot.myW = aPosition.myY + aHeight;

		myHeight = aHeight;
		myWidth = aWidth;

		myID = anID;

	}

	~VerticalHotSpot( )
	{
	}

	bool IsAvatarInsideMe( Vector2<float> &aPosition )
	{
		if( aPosition.myX < myBoxSpot.myX )
		{
			return false;
		}
		if( aPosition.myY < myBoxSpot.myY )
		{
			return false;
		}
		if( aPosition.myX > myBoxSpot.myZ )
		{
			return false;
		}
		if( aPosition.myY > myBoxSpot.myW )
		{
			return false;
		}

		if( InsideLeftOrRight( aPosition ) == true )
		{
			aPosition = Vector2<float>( myVerticalHotSpotConnex->GetBoxSpot( ).myZ + myVerticalHotSpotConnex->myWidth / 2, myVerticalHotSpotConnex->GetBoxSpot( ).myY + myVerticalHotSpotConnex->myHeight / 2 );
			aPosition.myX -= 32.0f;
			aPosition.myY -= 100.0f;
			return true;
		}

		aPosition = Vector2<float>( myVerticalHotSpotConnex->GetBoxSpot( ).myX - myVerticalHotSpotConnex->myWidth / 2, myVerticalHotSpotConnex->GetBoxSpot( ).myY + myVerticalHotSpotConnex->myHeight / 2 );
		aPosition.myX -= 32.0f;
		aPosition.myY -= 100.0f;
		return true;

	}

	Vector4<float> GetBoxSpot( )
	{
		return myBoxSpot;
	}

	void ConnectVerticalHotspot( VerticalHotSpot &aVerticalHotSpot )
	{
		myVerticalHotSpotConnex = &aVerticalHotSpot;
	}

	std::string GetVerticalHotSpotID( )
	{
		return myID;
	}

private:

	float myHeight;
	float myWidth;

	std::string myID;

	VerticalHotSpot *myVerticalHotSpotConnex;

	Vector4<float> myBoxSpot;

	VerticalHotSpot( )
	{

	}

	bool InsideLeftOrRight( Vector2<float> &aPosition )
	{
		if( aPosition.myX <= myBoxSpot.myX + myWidth / 2 )
		{
			return true;
		}

		return false;
	}

};


#endif