#include "Village.h"

Village::Village(void)
:LevelBaseData()
{
}

Village::Village(const LevelBaseData& aLevelBaseData)
:LevelBaseData(aLevelBaseData)
{

}

Village::~Village(void)
{
}

void Village::Initialize(HGE *aHGE)
{
	myHge = aHGE;

	StaticSprite tempSprite = StaticSprite(myHge, "img/Levels/scene_village.dds");
	myMap.AddFloorSprite(tempSprite, Vector2<float>(0, 0));
	tempSprite = StaticSprite(myHge, "img/Levels/scene_village_overlay.dds");
	myMap.AddOverlaySprite(tempSprite, Vector2<float>(0, 0));
	myCollisionMap = CollisionMap(16, "img/Levels/collission_village.bmp", myHge);

	myItems.push_back(Item (myHge, "img/items/sprite_rope.dds", "img/items/icon_rope.dds", Vector2<float>(1000.0f, 600.0f), "Rope"));
	myItems.push_back(Item (myHge, "img/NPCs/sprite_sheep.dds", "img/empty.dds", Vector2<float>(1073, 90 ), "Sheep", false));
	//myItems.push_back(Item (myHge, "img/NPCs/sprite_native_npc_standing.dds", "img/empty.dds", Vector2<float>(1829, 962), "Native One", false));
	//myItems.push_back(Item (myHge, "img/NPCs/sprite_boy.dds", "img/empty.dds", Vector2<float>(142, 791), "Boy One", false));

	myNativeVillager = NativeVillager(myHge);
	myNativeBoy = NativeBoy(myHge);

	myRobot = Robot(myHge);
	myRobot.Initialize();

	myVillager = Villager(myHge);

	InitializeConversations();

	myThisLevelMusic = "Village";

	myEndOfGame = false;
}

void Village::LevelSpecificHandleInput(InputWrapper &aInputWrapper, float aDelta, Inventory &aInventory)
{
	aDelta = aDelta;
	if( aInputWrapper.KeyPressed(DIK_H) && aInventory.CheckForItem("Diamond") == false )
	{
		Item tempDiamond(myHge, "img/items/sprite_diamond.dds", "img/items/icon_diamond.dds", Vector2<float>(690.0f, 180.0f), "Diamond");
		aInventory.AddItem( tempDiamond );
	}

	if((aInputWrapper.KeyPressed(DIK_E)) || (aInputWrapper.KeyPressed(DIK_RETURN)))
	{
		if(myDoneConversation.at(GetConversationIndex("Activate Roderick")) == false)
		{
			myRobot.MakeActive();
		}

		if( (myVillager.IsInterractable( ) == true && aInventory.CheckForItem("Diamond") == false && myDoneConversation.at(GetConversationIndex("The main quest1")) == true) || (myVillager.IsInterractable( ) == true && aInventory.CheckForItem("Diamond") == true && myDoneConversation.at(GetConversationIndex("The main quest1")) == true) )
		{
			myActiveConversationID = "The main quest1";
		}

		if(myVillager.IsInterractable( ) == true && aInventory.CheckForItem("Diamond") == true && myDoneConversation.at(GetConversationIndex("The main quest5")) == false )
		{
			myActiveConversationID = "Call me Ester";
			if( aInventory.CheckForItem("gold necklace") == false )
			{
				Item tempItem(myHge,"img/empty.dds", "img/items/icon_gold.dds", "gold necklace");
				aInventory.AddItem( tempItem );
			}
		}
	}
}

void Village::LevelSpecificUpdate(Avatar &aAvatar, Camera &aCamera, float aDelta)
{
	Vector2<float> tempVector = aAvatar.GetWorldSpacePosition();
	myRobot.ChaseAvatar(tempVector, aDelta);
	myRobot.Update(aDelta, aCamera.myPosition, tempVector );


	myVillager.Update(aCamera.myPosition, tempVector);


	myNativeVillager.Update( aCamera.myPosition );
	myNativeBoy.Update( aCamera.myPosition );
}

void Village::LevelSpecificRender(RenderData &aRenderData, Camera &aCamera)
{
	aCamera;
	myRobot.Render(aRenderData);
	myVillager.Render(aRenderData);
	myNativeVillager.Render(aRenderData);
	myNativeBoy.Render(aRenderData);
}

void Village::InitializeConversations()
{
	myConversationList.reserve(101);
	myConversationList.push_back(LevelConversation(myHge, "NONE", false));

	myConversationList.push_back(LevelConversation(myHge, "The main quest1", false));
	myConversationList.push_back(LevelConversation(myHge, "The main quest2", false));
	myConversationList.push_back(LevelConversation(myHge, "The main quest3", false));
	myConversationList.push_back(LevelConversation(myHge, "The main quest4", false));
	myConversationList.push_back(LevelConversation(myHge, "The main quest5", false));
	myConversationList.push_back(LevelConversation(myHge, "Call me Ester", false));
	myConversationList.push_back(LevelConversation(myHge, "Call me Ester2", false));
	myConversationList.push_back(LevelConversation(myHge, "rope", false));

	for(unsigned int index = 0; index < myConversationList.size(); index++)
	{
		myConversationList[index].Initialize();
		myConversationList[index].myActiveConversation = false;
		myDoneConversation.set(index);
	}

	myConversationList[GetConversationIndex("The main quest1")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("The main quest1")].AddDialogeText("Ester: : Hi.;");
	myConversationList[GetConversationIndex("The main quest1")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("The main quest1")].AddDialogeText("Roderick: : Do you actually think she will understand? We are in anoth...;");

	myConversationList[GetConversationIndex("The main quest2")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("The main quest2")].myDialoge.SetRightPortrait("img/Conversation/villagerWomanPortrait.dds", "Village Woman");
	myConversationList[GetConversationIndex("The main quest2")].AddDialogeText("Village Woman: : ...Hello?;");

	myConversationList[GetConversationIndex("The main quest3")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("The main quest3")].myDialoge.SetRightPortrait("img/Conversation/villagerWomanPortrait.dds", "Village Woman");
	myConversationList[GetConversationIndex("The main quest3")].AddDialogeText("Ester: : Yes, Hello! You seem to be from here and I am a little lost. Perhaps you could help me out?;");

	myConversationList[GetConversationIndex("The main quest4")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("The main quest4")].myDialoge.SetRightPortrait("img/Conversation/villagerWomanPortrait.dds", "Village Woman");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Village Woman: : I am indeed from here. But you are not, wearing such funny clothes and speaking with such a funny accent... and what is this speaking box?;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Ester: : Oh, Roderick? He's a friend.;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Village Woman: : A very odd friend.;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Ester: : I'm not going to argue with that.;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Village Woman: : No... but pray tell, Miss, what did you need my help with?;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Ester: : For one, I am looking for something akin to a diamond and for another; I couldn't help but notice that you are wearing a gold necklace. I need something that can conduct electricity and gold is very good for this.;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Village Woman: : El...ectricity?;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Ester: : Never mind, the important thing is that I'm looking for something like that necklace. Is there any chance of me getting one here?;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Village Woman: : You could have mine, on one condition.;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Ester: : Oh? Do tell.;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Village Woman: : My children have been missing for a few days, and I have looked everywhere but I can't find them. I'm starting to believe they have crossed the river to go to the volcano...;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Ester: : Do you want me to find them?;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Village Woman: : You can help me look for them and if you find them, I will give you the necklace.;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Ester: : It's a deal!;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Village Woman: : And if you're lucky, you might find a diamond as well. The giant bird seems to be very fond of them and it has a nest by the volcano. It might not be there either... I think it's out looking for it's babies.;");
	myConversationList[GetConversationIndex("The main quest4")].AddDialogeText("Ester: : Did you hear that Roderick? A diamond collecting bird!;");

	myConversationList[GetConversationIndex("The main quest5")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("The main quest5")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("The main quest5")].AddDialogeText("Roderick: : It seems unbelievably lucky.;");
	myConversationList[GetConversationIndex("The main quest5")].AddDialogeText("Ester: : But it won't hurt to look, so let's be off!;");

	myConversationList[GetConversationIndex("Call me Ester")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("Call me Ester")].myDialoge.SetRightPortrait("img/Conversation/villagerWomanPortrait.dds", "Village Woman");
	myConversationList[GetConversationIndex("Call me Ester")].AddDialogeText("Woman: : Oh, there you are! Thank you so much!;");
	myConversationList[GetConversationIndex("Call me Ester")].AddDialogeText("Ester: : I take it your children have returned home?  They were trapped inside the volcano. It seemed as if the way in had collapsed so they couldn't get out.;");
	myConversationList[GetConversationIndex("Call me Ester")].AddDialogeText("Woman: : Yes, they told me everything, even about that bird suit of yours!  I am very, very grateful. It's not much, but my necklace is what I can offer other than my heartfelt thanks.;");
	myConversationList[GetConversationIndex("Call me Ester")].AddDialogeText("Ester: : No, no! It's more than enough. It's just what I need and it will come to good use. I am glad I could help and that your children have returned home safely.;");
	myConversationList[GetConversationIndex("Call me Ester")].AddDialogeText("Woman: : I am too, but now I have to scold them for running off like that. Thank you again, strange woman.;");
	myConversationList[GetConversationIndex("Call me Ester")].AddDialogeText("Ester: : Call me Ester.;");
	myConversationList[GetConversationIndex("Call me Ester")].AddDialogeText("Woman: : Ester.;");

	myConversationList[GetConversationIndex("Call me Ester2")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("Call me Ester2")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("Call me Ester2")].AddDialogeText("Esther: : Look Roderick! We have some gold now! With pretty decorations and all!;");
	myConversationList[GetConversationIndex("Call me Ester2")].AddDialogeText("Roderick: : Indeed, pretty decorations. Now we have a semi-conductor component for the Time-Machine at last.;");
	myConversationList[GetConversationIndex("Call me Ester2")].AddDialogeText("Esther: : Let's get moving.;");
	myConversationList[GetConversationIndex("Call me Ester2")].AddDialogeText("Roderick: : Aye, aye Capt'n.;");

	myConversationList[GetConversationIndex("rope")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Ester");
	myConversationList[GetConversationIndex("rope")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("rope")].AddDialogeText("Esther: : Hey Roderick I found a rope!.;");
	myConversationList[GetConversationIndex("rope")].AddDialogeText("Roderick: : Yes Esther, you found something, again, whoopiedoh.;");
	myConversationList[GetConversationIndex("rope")].AddDialogeText("Esther: : Why do you always have to be so mean Roderick!?;");
	myConversationList[GetConversationIndex("rope")].AddDialogeText("Roderick: : I don't know, you're the one who programmed me, it's all your fault.;");
	myConversationList[GetConversationIndex("rope")].AddDialogeText("Esther: : If you don't stop being so mean I'll reprogram you when we come home!;");
	myConversationList[GetConversationIndex("rope")].AddDialogeText("Roderick: : Then I may not be so cooperative as to start the time-machine to get back home.;");
	myConversationList[GetConversationIndex("rope")].AddDialogeText("Esther: : ...;");

	myActiveConversationID = "";
	myActiveConversationIndex = 0;
}

void Village::ConversationEvent(float elapsTime, Avatar &aAvatar, Inventory &aInventory)
{
	aAvatar;
	myConversationDelta += elapsTime;

	if( myDoneConversation.at(GetConversationIndex("The main quest1")) == false && myDoneConversation.at(GetConversationIndex("The main quest2")) == true)//( myDoneConversation.at(GetConversationIndex("The main quest1")) == false && myDoneConversation.at(GetConversationIndex("The main quest2")) == true && aInventory.CheckForItem("Diamond")
	{
		myActiveConversationID = "The main quest2";
	}
	if( myDoneConversation.at(GetConversationIndex("The main quest2")) == false && myDoneConversation.at(GetConversationIndex("The main quest3")) == true)
	{
		myActiveConversationID = "The main quest3";
	}
	if( myDoneConversation.at(GetConversationIndex("The main quest3")) == false && myDoneConversation.at(GetConversationIndex("The main quest4")) == true)
	{
		myActiveConversationID = "The main quest4";
	}
	if( myDoneConversation.at(GetConversationIndex("The main quest4")) == false && myDoneConversation.at(GetConversationIndex("The main quest5")) == true)
	{
		myActiveConversationID = "The main quest5";
	}

	if( aInventory.CheckForItem("Rope") == true )
	{
		if( myDoneConversation.at(GetConversationIndex("rope")) == true )
		{
			myActiveConversationID = "rope";
		}
	}

	if( myDoneConversation.at(GetConversationIndex("Call me Ester")) == false )
	{
		myActiveConversationID = "Call me Ester2";
	}

	if(myActiveConversationID != "")
	{
		myActiveConversationIndex = GetConversationIndex(myActiveConversationID);
	}
}