#include "Vulcano.h"
#include "Diamond.h"

Vulcano::Vulcano(void)
:LevelBaseData()
{
}

Vulcano::Vulcano(const LevelBaseData& aLevelBaseData)
:LevelBaseData(aLevelBaseData)
{
}

Vulcano::~Vulcano(void)
{
}

void Vulcano::Initialize(HGE *aHGE)
{
	myHge = aHGE;

	StaticSprite tempSprite = StaticSprite(myHge, "img/Levels/scene_volcano.dds");
	myMap.AddFloorSprite(tempSprite, Vector2<float>(0, 0));
	tempSprite = StaticSprite(myHge, "img/Levels/scene_volcano_overlay.dds");
	myMap.AddOverlaySprite(tempSprite, Vector2<float>(0, 0));
	myCollisionMap = CollisionMap(16, "img/Levels/collission_volcano.bmp", myHge);
	
	myItems.push_back(Diamond (myHge, "img/items/sprite_diamond.dds", "img/items/icon_diamond.dds", Vector2<float>(690.0f, 180.0f), "Diamond"));
	myItems.push_back(Item (myHge, "img/items/sprite_banana_tree.dds", "img/items/empty.dds", Vector2<float>(153,132), "banana tree sprite"));
	myItems.push_back(Item (myHge, "img/items/empty.dds", "img/items/empty.dds", Vector2<float>(290,455), "banana tree"));
	myItems.push_back(Item (myHge, "img/items/empty.dds", "img/items/empty.dds", Vector2<float>(972,415), "vulcanization"));
	myItems.push_back(Item (myHge, "img/items/empty.dds", "img/items/empty.dds", Vector2<float>(750,438), "stair kit"));
	myItems.push_back(Item (myHge, "img/items/empty.dds", "img/items/empty.dds", Vector2<float>(617, 50), "children"));
	myItems.push_back(Item (myHge, "img/NPCs/sprite_boy.dds", "img/NPCs/sprite_boy.dds", Vector2<float>(600,53), "stale boy" ));
	myItems.back().MakeNotPickupable();
	myBoy = &myItems.back();
	myItems.push_back(Item (myHge, "img/NPCs/sprite_girl.dds", "img/NPCs/sprite_girl.dds", Vector2<float>(635,47), "stale girl" ));
	myItems.back().MakeNotPickupable();
	myGirl = &myItems.back();

	myPalmOverlay = Item(myHge, "img/Items/palm_overlay.dds", "img/Items/palm_overlay.dds", Vector2<float>(624, 84), "palm overlay");
	myPalmOverlay.MakeInvisible();
	myPalmOverlay.MakeNotPickupable();
	
	myRobot = Robot(myHge);
	myRobot.Initialize();

	myRobot.MakeActive( );

	InitializeConversations();	

	myBirdCutscene = BirdCutscene(myHge);
	myThisLevelMusic = "Vulcano";
	myCurtain = Curtain(myHge);

	myMoveAvatarToNest = false;
	myAvaterHaveSavedTheChildren = false;
	myClimbingDownTree = false;
	myVulcanoClimbable = true;
	myTravelDown = false;
	myTravelUp = false;
	myStairUpCreated = false;
	myClimbedDownFromTree = false;
	myClimbingTimer = 0;
	myChildrenSaved = false;

	myEndOfGame = false;
}

void Vulcano::LevelSpecificHandleInput(InputWrapper &aInputWrapper, float aDelta, Inventory &aInventory)
{
	aDelta;
	aInventory;
	if((aInputWrapper.KeyPressed(DIK_E)) || (aInputWrapper.KeyPressed(DIK_RETURN)))
	{
		if(myDoneConversation.at(GetConversationIndex("Activate Roderick")) == false)
		{
			myRobot.MakeActive();
		}
	}
}

void Vulcano::LevelSpecificUpdate(Avatar &aAvatar, Camera &aCamera, float aDelta)
{
	Vector2<float> tempVector = aAvatar.GetWorldSpacePosition( );
	myRobot.ChaseAvatar(tempVector, aDelta);
	myRobot.Update(aDelta, aCamera.myPosition, tempVector );
	myCurtain.FadeIn();

	Vector2<float> cameraPosition = aCamera.myPosition;
	Vector2<float> avatarPosition = aAvatar.GetWorldSpacePosition();
	myPalmOverlay.Update(cameraPosition, avatarPosition);

	if (myClimbingDownTree == true
		&& myClimbingTimer < 1)
	{
		for( unsigned i = 0; i < 300; i++ )
		{
			myCurtain.FadeOut( );
		}
		myClimbingTimer += aDelta;
		aAvatar.SetWorldPosition(Vector2<float>(690.0f, 240.0f));
		myPalmOverlay.MakeVisible();
	}

	if(myMoveAvatarToNest == true && myAvaterHaveSavedTheChildren == false)
	{
		aAvatar.SetWorldPosition(Vector2<float>(690.0f, 140.0f));
		aCamera.CenterOnAvatar(aAvatar.GetWorldSpacePosition());
		myMoveAvatarToNest = false;
	}
// 	if(myAvaterHaveSavedTheChildren == true && myMoveAvatarToNest == true)
// 	{
// 		aAvatar.SetWorldPosition(Vector2<float>(140.0f, 600.0f));
// 		aCamera.CenterOnAvatar(aAvatar.GetWorldSpacePosition());
// 		myMoveAvatarToNest = false;
// 	}

	if ( myTravelDown == true)
	{
		for( unsigned i = 0; i < 300; i++ )
		{
			myCurtain.FadeOut( );
		}
		aAvatar.SetWorldPosition( Vector2<float>(428,689) );
		myRobot.SetFollowingAvatar(true);
		aCamera.CenterOnAvatar( aAvatar.GetWorldSpacePosition( ) );
		myTravelDown = false;
	} 

	if ( myTravelUp == true)
	{
		for( unsigned i = 0; i < 300; i++ )
		{
			myCurtain.FadeOut( );
		}
		aAvatar.SetWorldPosition( Vector2<float>(775,353) );
		myRobot.SetFollowingAvatar(false);
		aCamera.CenterOnAvatar( aAvatar.GetWorldSpacePosition( ) );
		myTravelUp = false;
	} 

	if ( myChildrenSaved == true && EventSingelton::GetInstance()->myConversationActive == false)
	{
		for( unsigned i = 0; i < 300; i++ )
		{
			myCurtain.FadeOut( );
		}
		
		int boyIndex = -1;
		bool failure = false;
		failure = GetItemIndex("stale boy", boyIndex);
		assert((failure == true) && "Couldn't get item with ID.");
		myItems.at(boyIndex).MakeInvisible();

		int girlIndex = -1;
		failure = GetItemIndex("stale girl", girlIndex);
		assert((failure == true) && "Couldn't get item with ID.");
		myItems.at(girlIndex).MakeInvisible();

		myChildrenSaved = false;
	} 
}

void Vulcano::LevelSpecificRender(RenderData &aRenderData, Camera &aCamera)
{
	aCamera;
	myRobot.Render(aRenderData);
	myCurtain.Render(aRenderData);	
}

void Vulcano::RenderOverlay(RenderData &aRenderData, Camera &aCamera)
{
	LevelBaseData::RenderOverlay(aRenderData, aCamera);
	myPalmOverlay.RenderAtWorld(aRenderData, true);
}

void Vulcano::InitializeConversations()
{
	myConversationList.reserve(101);
	myConversationList.push_back(LevelConversation(myHge, "NONE", false));
	myConversationList.push_back(LevelConversation(myHge, "A bird", false));
	myConversationList.push_back(LevelConversation(myHge, "The bird suit", false));
	myConversationList.push_back(LevelConversation(myHge, "Diamond", false));
	myConversationList.push_back(LevelConversation(myHge, "Saving children", false));
	myConversationList.push_back(LevelConversation(myHge, "children no exit", false));
	myConversationList.push_back(LevelConversation(myHge, "complete children", false));
	myConversationList.push_back(LevelConversation(myHge, "Banana Tree", false));
	myConversationList.push_back(LevelConversation(myHge, "Landing", false));
	myConversationList.push_back(LevelConversation(myHge, "Vulcanization failed", false));
	myConversationList.push_back(LevelConversation(myHge, "Vulcanization success", false));
	myConversationList.push_back(LevelConversation(myHge, "building stairs", false));
	
	for(unsigned int index = 0; index < myConversationList.size(); index++)
	{
		myConversationList[index].Initialize();
		myConversationList[index].myActiveConversation = false;
		myDoneConversation.set(index);
	}

	myConversationList[GetConversationIndex("A bird")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("A bird")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Ester: : Wow. Can you see that?;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Roderick: : See what?;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Ester: : The bird. It's enormous.;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Roderick: : Is that the first thing you notice?;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Ester: : And it has my hairstyle!;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Ester: : Actually, not counting the beak and the wings, it  looks exactly like me. Even birds have taste, it seems.;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Ester: : I can't see any baby birds in the nest, though. It can't have found them yet. But I think I see a diamond in there, like the villager woman said.;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Roderick: : Wait, is it a cut diamond? How is that possible?;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Ester: : Perhaps it stole it? I don't know  and I don't care, I just need to figure out a way to get up there.;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Ester: : Hm. Speaking of children, have you seen any sign of them?;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Roderick: : I am afraid not.;");
	myConversationList[GetConversationIndex("A bird")].AddDialogeText("Ester: : Perhaps we'll get a better view from up there. Let's do it.;");

	myConversationList[GetConversationIndex("Banana Tree")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("Banana Tree")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Ester: : Hey Roderick, this is one mighty looking Banana Tree.;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Roderick: : Yes Esther, it sure is.;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Ester: : Hmmm.... Roderick, zoom a look at the large bird in the sky up over by the river please.;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Roderick: : Zooooming... Target aquired, is there anything you want to know about it?;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Ester: : Could you describe it's looks for me?.;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Roderick: : It's very large, it may be able to carry alot of weight. It has large feathers, with a tint of green to them. It also has a pequlier beak that looks something alike a banana...;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Ester: : Interesting... The woman in the village told us that it was out looking for it's babies...;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Ester: : The leaves on the banana tree looks similar to the birds feathers, am I right?;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Roderick: : Let me take a look... Yes, yes indeed they do, what are you thinking about Ester?;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Ester: : That maybe... just maybe we could get a few leaves and a banan or 2 to make a suit that would make us look like a baby bird...;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Roderick: : That's a crazy idea, either the bird will carry you to safety or you'll get eaten when she discovers that it's not her child! This is madness!;");
	myConversationList[GetConversationIndex("Banana Tree")].AddDialogeText("Ester: : Perhaps Roderick, but I don't see any other way at the moment! I'm going for it! All I have to do is put it together and I should be ready to go!;");

	myConversationList[GetConversationIndex("The bird suit")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("The bird suit")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("The bird suit")].AddDialogeText("Ester: : Hm. This might work.;");
	myConversationList[GetConversationIndex("The bird suit")].AddDialogeText("Roderick: : What might work, Ester? How might a banana and a pair of large leaves work?;");
	myConversationList[GetConversationIndex("The bird suit")].AddDialogeText("Ester: : I'll use this banana as a beak like this... and then these leaves as wings, like this...;");
	myConversationList[GetConversationIndex("The bird suit")].AddDialogeText("Roderick: : You look ridiculous.;");
	myConversationList[GetConversationIndex("The bird suit")].AddDialogeText("Ester: : But I do look like a little baby volcano bird.;");
	myConversationList[GetConversationIndex("The bird suit")].AddDialogeText("Roderick: : Indeed.;");
	myConversationList[GetConversationIndex("The bird suit")].AddDialogeText("Ester: : Yeah I wish I had a mirr-WHOA!;");

	myConversationList[GetConversationIndex("Diamond")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("Diamond")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("Diamond")].AddDialogeText("Ester: : Good lord. Look at the size of that! It's as big as a head!;");
	myConversationList[GetConversationIndex("Diamond")].AddDialogeText("Roderick: : I'm sure it is. Now get down from the tree before the bird realizes what you're doing.;");
	myConversationList[GetConversationIndex("Diamond")].AddDialogeText("Ester: : On it! I need to check if the children are allright!;");
	
	myConversationList[GetConversationIndex("Saving children")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Boy: : Look, sister! A horrible creature has come to eat us!;");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Ester: : Excuse me! I'm not a horrible creature!;");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Boy: : It talks! And it's trying to decieve us!;");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Ester: : Now listen here young man! I'll have none of that!;");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Ester: : Your mother is very worried and has sent me to look for you.;");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Boy: : Oh, well why didn't you just say so. Who are you, then?;");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Ester: : It's... complicated.;");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Ester: : What are you doing up here? Your mum has been worried sick.;");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Boy: : Mommy has a birthday soon, and we were trying to get her that big diamond as a present.;");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Boy: : But once we got up here, the earths tummy growled and rocks fell, blocking the enterance.;");
	myConversationList[GetConversationIndex("Saving children")].AddDialogeText("Ester: : That's very sweet of you to try and bring a present for your mom!;");

	myConversationList[GetConversationIndex("children no exit")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("children no exit")].AddDialogeText("Ester: : I'm going to go and see if I can't do something about the stones.;");
	myConversationList[GetConversationIndex("children no exit")].AddDialogeText("Ester: : And then you can run off back to the village.;");
	myConversationList[GetConversationIndex("children no exit")].AddDialogeText("Boy: : Thank you, miss! ... You are a girl, right?;");
	myConversationList[GetConversationIndex("children no exit")].AddDialogeText("Ester: : ... I'll just go look at those rocks.;");	

	myConversationList[GetConversationIndex("complete children")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("complete children")].AddDialogeText("Ester: : I've made a stair down the rocks!;");
	myConversationList[GetConversationIndex("complete children")].AddDialogeText("Ester: : Now you can safetly climb down and run back to your mother.;");
	myConversationList[GetConversationIndex("complete children")].AddDialogeText("Boy: : Oh thank you so much, miss!;");
	myConversationList[GetConversationIndex("complete children")].AddDialogeText("Boy: : We'll run back home and tell mommy how you saved us!;");
	myConversationList[GetConversationIndex("complete children")].AddDialogeText("Boy: : But, before we go... that diamond...;");
	myConversationList[GetConversationIndex("complete children")].AddDialogeText("Ester: : Sorry, kid. I need it for my time machine.;");
	myConversationList[GetConversationIndex("complete children")].AddDialogeText("Ester: : Run along now!;");

	myConversationList[GetConversationIndex("Landing")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("Landing")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("Landing")].AddDialogeText("Ester: : OMPH! What a landing!;");
	myConversationList[GetConversationIndex("Landing")].AddDialogeText("Ester: : Hey Roderick! Can you hear me from up here?;");
	myConversationList[GetConversationIndex("Landing")].AddDialogeText("Roderick: : Yes, Ester. Did you enjoy your flight?;");
	myConversationList[GetConversationIndex("Landing")].AddDialogeText("Ester: : Nevermind that, I can see the children from up in this tree!;");
	myConversationList[GetConversationIndex("Landing")].AddDialogeText("Roderick: : Well that's too bad, seeing as you are all stuck behind all those stones.;");
	myConversationList[GetConversationIndex("Landing")].AddDialogeText("Ester: : Right. I should have a look around and try to find a way back down.;");
	myConversationList[GetConversationIndex("Landing")].AddDialogeText("Ester: : But first, the diamond.;");

	myConversationList[GetConversationIndex("Vulcanization success")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("Vulcanization success")].myDialoge.SetRightPortrait("img/Conversation/RobotPortrait.dds", "Roderick");
	myConversationList[GetConversationIndex("Vulcanization success")].AddDialogeText("Ester: : Ok, let's lower this contraption close to the lava for it to heat up...;");
	myConversationList[GetConversationIndex("Vulcanization success")].AddDialogeText("Ester: : Voila! A coconut shell filled with rubber perfect for fixing my ship!;");
	myConversationList[GetConversationIndex("Vulcanization success")].AddDialogeText("Roderick: : Again with the foreign catch phrases...;");
	myConversationList[GetConversationIndex("Vulcanization success")].AddDialogeText("Ester: : Oh hush, you...;");

	myConversationList[GetConversationIndex("Vulcanization failed")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("Vulcanization failed")].AddDialogeText("Ester: : If I only had some rubber in a container, I could lower it down to the lava and vulcanize it!;");

	myConversationList[GetConversationIndex("building stairs")].myDialoge.SetLeftPortrait("img/Conversation/heroinePortrait.dds", "Esther");
	myConversationList[GetConversationIndex("building stairs")].AddDialogeText("Ester: : Hmm... Maybe I can roll down some stones from here, and create a stair like pile at the bottom...;");
	myConversationList[GetConversationIndex("building stairs")].AddDialogeText("Ester: : Let's try.;");
	myConversationList[GetConversationIndex("building stairs")].AddDialogeText("RUMBLE! RUMBLE!;");
	myConversationList[GetConversationIndex("building stairs")].AddDialogeText("Ester: : A few more...;");
	myConversationList[GetConversationIndex("building stairs")].AddDialogeText("RUMBLE!;");
	myConversationList[GetConversationIndex("building stairs")].AddDialogeText("Ester: : Yes! Now I will be able to climb up and down the volcano!;");


	myActiveConversationID = "";
	myActiveConversationIndex = 0;
}

void Vulcano::ConversationEvent(float elapsTime, Avatar &aAvatar, Inventory &aInventory)
{
	aAvatar;
	
	if ( aInventory.CheckForItem("banana tree") == true)
	{
		myActiveConversationID = "Banana Tree";
		Item banana(myHge, "img/items/icon_banana.dds", "img/items/icon_banana.dds", Vector2<float>(280.0f, 450.0f), "banana");
		Item leafOne(myHge, "img/items/icon_big_leaves.dds", "img/items/icon_big_leaves.dds", Vector2<float>(140.0f, 500.0f), "banana tree leaf 1");
		Item leafTwo(myHge, "img/items/icon_big_leaves.dds", "img/items/icon_big_leaves.dds", Vector2<float>(600.0f, 800.0f), "banana tree leaf 2");
		
		aInventory.DeleteItem("banana tree");

		aInventory.AddItem(banana);
		aInventory.AddItem(leafOne);
		aInventory.AddItem(leafTwo);
	}

	myConversationDelta += elapsTime;

	if(myConversationDelta >= 2500 
		&& myConversationDelta < 10000
		&& myDoneConversation.at(GetConversationIndex("A bird")) == true)
	{
		myActiveConversationID = "A bird";
	}
	
	if ( aInventory.CheckForItem("bird suit"))
	{
		aInventory.MakeNotActive();
		myActiveConversationID = "The bird suit";
	}
	if( myDoneConversation.at(GetConversationIndex("The bird suit")) == 0 )
	{
		if(aInventory.CheckForItem("bird suit"))
		{
			aInventory.DeleteItem("bird suit");
// 			Item tempItem(myHge, "img/empty.dds", "img/Items/icon_coconutLatexRefined.dds", "refined rubber");
// 			aInventory.AddItem( tempItem );
// 			aInventory.DeleteItem("rubber string");
			myMoveAvatarToNest = true;
			myRobot.SetFollowingAvatar(false);
		}
		myBirdCutscene.Play(elapsTime);
	}

	if ( myDoneConversation.at(GetConversationIndex("Landing")) == true 
		&& myBirdCutscene.myCutsceneFinish == true )
	{
		myActiveConversationID = "Landing";
	}

	if (aInventory.CheckForItem("Diamond") == true
		&& myDoneConversation.at(GetConversationIndex("Landing")) == false
		&&  myDoneConversation.at(GetConversationIndex("Diamond")) == true)
	{
		myActiveConversationID = "Diamond";
	}

	if (myDoneConversation.at(GetConversationIndex("Diamond")) == false)
	{
		myClimbingDownTree = true;
		myClimbedDownFromTree = true;
	}

	if (aInventory.CheckForItem("vulcanization") == true)
	{
		bool returnItem = true;
		if (aInventory.CheckForItem("rubber string") == true)
		{
			if (myDoneConversation.at(GetConversationIndex("Vulcanization success")) == true)
			{
				aInventory.DeleteItem("rubber string");
				aInventory.DeleteItem("vulcanization");

				Item tempItem(myHge, "img/empty.dds", "img/Items/icon_coconutLatexRefined.dds", "refined rubber");
				aInventory.AddItem( tempItem );

				myActiveConversationID = "Vulcanization success";
				myDoneConversation.at(GetConversationIndex("Vulcanization failed")) = false;
				returnItem = false;
			}
		}
		else
		{
			if (myDoneConversation.at(GetConversationIndex("Vulcanization failed")) == true)
			{
				myActiveConversationID = "Vulcanization failed";
			}
		}		

		if ( returnItem == true )
		{
			Item aReturningItem;
			aInventory.TakeOutItem("vulcanization", aReturningItem);
			myItems.push_back(aReturningItem);
		}
	}	

	if (aInventory.CheckForItem("stair kit") == true) 
	{
		if ( myDoneConversation.at(GetConversationIndex("building stairs")) == true)
		{
			myActiveConversationID = "building stairs";
		}
		else
		{
			Item aReturningItem;
			aInventory.TakeOutItem("stair kit", aReturningItem);
			myItems.push_back(aReturningItem);
			myTravelDown = true;
		}
		if (myStairUpCreated == false)
		{
			myStairUpCreated = true;
			myItems.push_back(Item (myHge, "img/items/empty.dds", "img/items/empty.dds", Vector2<float>(580	,725), "stair up"));
		}
	}

	if (aInventory.CheckForItem("stair up") == true)
	{
		Item aReturningItem;
		aInventory.TakeOutItem("stair up", aReturningItem);
		myItems.push_back(aReturningItem);
		myTravelUp = true;
	}

	if (aInventory.CheckForItem("children") == true) 
	{
		bool returnItem = true;
		if (myClimbedDownFromTree == true)
		{
			if ( myDoneConversation.at(GetConversationIndex("Saving children")) == true)
			{
				returnItem = false;
				myActiveConversationID = "Saving children";
			}
			if ( myDoneConversation.at(GetConversationIndex("Saving children")) == false)
			{
				if (myStairUpCreated == true)
				{
					if (myDoneConversation.at(GetConversationIndex("complete children")) == true)
					{
						myActiveConversationID = "complete children";
						myDoneConversation.at(GetConversationIndex("children no exit")) = false;
						returnItem = false;
						Item aReturnItem;
						aInventory.TakeOutItem("children", aReturnItem);
						myChildrenSaved = true;
					}
				}
				else
				{
					if (myDoneConversation.at(GetConversationIndex("children no exit")) == true)
					{
						myActiveConversationID = "children no exit";
					}
				}
			}
		}
		if (returnItem == true)
		{
			Item aReturningItem;
			aInventory.TakeOutItem("children", aReturningItem);
			myItems.push_back(aReturningItem);
		}
	}

}